@@ -9,7 +9,8 @@ package Tk::NoteBook;
 
 use vars qw($VERSION);
 
-$VERSION = sprintf '4.%03d', q$Revision: #9 $ =~ /\D(\d+)\s*$/;
+#$VERSION = sprintf '4.%03d', q$Revision: #9 $ =~ /\D(\d+)\s*$/;
+$VERSION = '4.011';
 require Tk::NBFrame;
 
 use base  qw(Tk::Derived Tk::NBFrame);
@@ -231,7 +232,7 @@ sub delete
   {
    $childw->bind('<Destroy>', undef);
    $childw->destroy;
-   @{$w->{'windows'}} = grep($_ !~ /$child/, @{$w->{'windows'}});
+   @{$w->{'windows'}} = grep($_ ne $child, @{$w->{'windows'}});
    $w->{'nWindows'}--;
    $w->SUPER::delete($child);
    # see if the child to be deleted was the top child
@@ -390,8 +391,8 @@ sub Resize {
 
     if ($reqW * $reqH == 0)
      {
-	if ((not defined $w->{-dynamicgeometry}) ||
-	    ($w->{-dynamicgeometry} == 0)) {
+	if ((not defined $w->cget('-dynamicgeometry')) ||
+	    ($w->cget('-dynamicgeometry') == 0)) {
 	    $reqW = 1;
 	    $reqH = 1;
 
@@ -445,8 +446,35 @@ sub InitTabSize {
     $w->{'minH'} = $tH;
 }
 
+sub BalloonInfo
+{
+ my ($notebook,$balloon,$X,$Y,@opt) = @_;
+ my $page = $notebook->identify($X-$notebook->rootx,$Y-$notebook->rooty);
+ foreach my $opt (@opt)
+  {
+   my $info = $balloon->GetOption($opt,$notebook);
+   if ($opt =~ /^-(statusmsg|balloonmsg)$/ && UNIVERSAL::isa($info,'HASH'))
+    {
+     if (!defined $page)
+      {
+       $balloon->Deactivate;
+       return;
+      }
+     $balloon->Subclient($page);
+     if (exists $info->{$page})
+      {
+       return $info->{$page}
+      }
+     else
+      {
+       return '';
+      }
+    }
+   return $info;
+  }
+}
+
 1;
 
 __END__
 
-=cut
