@@ -15,6 +15,7 @@ use     AutoLoader qw(AUTOLOAD);
 use     DynaLoader;
 use     Cwd();
 use base qw(Exporter DynaLoader);
+use     File::Spec qw();
 
 *fileevent = \&Tk::Event::IO::fileevent;
 
@@ -62,13 +63,15 @@ use strict;
 use Carp;
 
 # Record author's perforce depot record
-$Tk::CHANGE      = q$Change: 3279 $;
+#$Tk::CHANGE      = q$Change: 3279 $;
+$Tk::CHANGE      = 'sfsvn-' . q$Change: 27 $;
 
 # $tk_version and $tk_patchLevel are reset by pTk when a mainwindow
 # is created, $VERSION is checked by bootstrap
 $Tk::version     = '8.4';
 $Tk::patchLevel  = '8.4';
-$Tk::VERSION     = '804.027';
+$Tk::VERSION     = '804.028';
+$Tk::VERSION     = eval $Tk::VERSION;
 $Tk::XS_VERSION  = $Tk::VERSION;
 $Tk::strictMotif = 0;
 
@@ -314,15 +317,29 @@ sub messageBox
  $args{'-default'} = lc($args{'-default'}) if (exists $args{'-default'});
  ucfirst tk_messageBox(-parent => $widget, %args);
 }
-
+sub _adapt_path_to_os
+{
+ # adapting the path of -initalfile and -initialdir to the operating system
+ # (like that getOpenFile(-initialdir => 'c:/WINNT') will work, as it will
+ #  be converted to c:\WINNT)
+ my %args = @_;
+ foreach my $option (qw(-initialfile -initialdir))
+  {
+   if ($args{$option})
+    {
+     $args{$option} = File::Spec->catfile($args{$option});
+    }
+   }
+ return %args;
+}    
 sub getOpenFile
 {
- tk_getOpenFile(-parent => shift,@_);
+ tk_getOpenFile(-parent => shift,_adapt_path_to_os(@_));
 }
 
 sub getSaveFile
 {
- tk_getSaveFile(-parent => shift,@_);
+ tk_getSaveFile(-parent => shift,_adapt_path_to_os(@_));
 }
 
 sub chooseColor
@@ -332,7 +349,7 @@ sub chooseColor
 
 sub chooseDirectory
 {
- tk_chooseDirectory(-parent => shift,@_);
+ tk_chooseDirectory(-parent => shift,_adapt_path_to_os(@_));
 }
 
 sub DialogWrapper
@@ -533,6 +550,16 @@ sub FocusChildren { shift->children }
 # to the next window after this one in the traversal
 # order.
 sub focusNext
+{
+ my $w = shift;
+ my $cur = $w->getNextFocus;
+ if ($cur)
+  {
+   $cur->tabFocus;
+  }
+}
+
+sub getNextFocus
 {
  my $w = shift;
  my $cur = $w;
@@ -563,8 +590,7 @@ sub focusNext
     }
    if ($cur == $w || $cur->FocusOK)
     {
-     $cur->tabFocus;
-     return;
+     return $cur;
     }
   }
 }
@@ -579,6 +605,16 @@ sub focusNext
 # to the previous window before this one in the traversal
 # order.
 sub focusPrev
+{
+ my $w = shift;
+ my $cur = $w->getPrevFocus;
+ if ($cur)
+  {
+   $cur->tabFocus;
+  }
+}
+
+sub getPrevFocus
 {
  my $w = shift;
  my $cur = $w;
@@ -618,8 +654,7 @@ sub focusPrev
    $cur = $parent;
    if ($cur == $w || $cur->FocusOK)
     {
-     $cur->tabFocus;
-     return;
+     return $cur;
     }
   }
 
