@@ -1,11 +1,11 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2003 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Clipboard;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '4.003'; # $Id: //depot/Tkutf8/Tk/Clipboard.pm#3 $
+$VERSION = sprintf '4.%03d', q$Revision: #8 $ =~ /\D(\d+)\s*$/;
 
 use AutoLoader qw(AUTOLOAD);
 use Tk qw(catch);
@@ -64,7 +64,19 @@ sub clipboardPaste
 {
  my $w = shift;
  local $@;
- catch { $w->insert('insert',$w->clipboardGet)};
+ catch
+  {
+## Different from Tcl/Tk version:
+#    if ($w->windowingsystem eq 'x11')
+#     {
+#      catch
+#       {
+#        $w->deleteSelected;
+#       };
+#     }
+   $w->insert("insert", $w->clipboardGet);
+   $w->SeeInsert if $w->can('SeeInsert');
+  };
 }
 
 sub clipboardOperations
