@@ -1,8 +1,8 @@
-package Tk::ColorSelect;
+package Tk::ColorSelect; # XXX why is this the Tk::ColorSelect package?
 use strict;
 
 use vars qw($VERSION);
-$VERSION = sprintf '4.%03d', q$Revision: #8 $ =~ /\D(\d+)\s*$/;
+$VERSION = '4.010'; # was: sprintf '4.%03d', q$Revision: #8 $ =~ /\D(\d+)\s*$/;
 
 use Tk qw(Ev);
 
@@ -18,11 +18,14 @@ sub Populate
     require Tk::Config;
     my(@xlibpath) = map { s/^-L//; "$_/X11/rgb.txt" }
                     split /\s+/, $Tk::Config::xlib;
+    my %seen_names;
     foreach $i (@xlibpath,
 		'/usr/local/lib/X11/rgb.txt', '/usr/lib/X11/rgb.txt',
 		'/usr/X11R6/lib/X11/rgb.txt',
                 '/usr/local/X11R5/lib/X11/rgb.txt', '/X11/R5/lib/X11/rgb.txt',
-                '/X11/R4/lib/rgb/rgb.txt', '/usr/openwin/lib/X11/rgb.txt') {
+                '/X11/R4/lib/rgb/rgb.txt', '/usr/openwin/lib/X11/rgb.txt',
+		'/usr/share/X11/rgb.txt', # This is the Debian location
+	       ) {
         local *FOO;
         next if ! open FOO, $i;
         my $middle_left = $middle->Frame;
@@ -38,6 +41,7 @@ sub Populate
             -borderwidth     => 2,
             -exportselection => 0,
         );
+	$middle->Advertise(Names => $names);
 
         $names->bind('<Double-1>' => [$middle,'color',Ev(['getSelected'])]);
 
@@ -67,9 +71,9 @@ sub Populate
 		    die $@;
 		}
             }
-            if (!exists($Tk::ColorEditor::names{$hex}) ||
-                length($Tk::ColorEditor::names{$hex}) > length($color)) {
-                  $Tk::ColorEditor::names{$hex} = $color;
+            if (!exists($seen_names{$hex}) ||
+                length($seen_names{$hex}) > length($color)) {
+                $seen_names{$hex} = $color;
                 $names->insert('end', $color);
             }
         }
@@ -435,15 +439,23 @@ sub Show
  my $cw = shift;
  $cw->configure(@_) if @_;
  $cw->Popup();
+ $cw->OnDestroy(sub { $cw->{'done'} = 0 }); # auto-cancel
  $cw->waitVariable(\$cw->{'done'});
- $cw->withdraw;
- return $cw->cget('-color');
+ if (Tk::Exists($cw))
+  {
+   $cw->withdraw;
+   $cw->cget('-color');
+  }
+ else
+  {
+   undef;
+  }
 }
 
 package Tk::ColorEditor;
 
 use vars qw($VERSION $SET_PALETTE);
-$VERSION = sprintf '4.%03d', q$Revision: #8 $ =~ /\D(\d+)\s*$/;
+$VERSION = '4.009'; # was: sprintf '4.%03d', q$Revision: #8 $ =~ /\D(\d+)\s*$/;
 
 use Tk qw(lsearch Ev);
 use Tk::Toplevel;
@@ -451,9 +463,6 @@ use base  qw(Tk::Toplevel);
 use Tk::widgets qw(Pixmap);
 Construct Tk::Widget 'ColorEditor';
 
-%Tk::ColorEditor::names = ();
-
-
 use Tk::Dialog;
 use Tk::Pretty;
 
@@ -757,5 +766,3 @@ sub set_colors {
 
 __END__
 
-=cut
-
