@@ -1,4 +1,4 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2003 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Derived;
@@ -8,7 +8,7 @@ use strict;
 use Carp;
 
 use vars qw($VERSION);
-$VERSION = '4.005'; # $Id: //depot/Tkutf8/Tk/Derived.pm#5 $
+$VERSION = sprintf '4.%03d', q$Revision: #10 $ =~ /\D(\d+)\s*$/;
 
 $Tk::Derived::Debug = 0;
 
@@ -336,14 +336,16 @@ sub ConfigDefault
 
  unless (exists($specs->{'-background'}))
   {
-   my (@bg) = ('SELF');
+   Tk::catch { $cw->Tk::cget('-background') };
+   my (@bg) = $@ ? ('PASSIVE') : ('SELF');
    push(@bg,'CHILDREN') if $child;
    $specs->{'-background'} = [\@bg,'background','Background',NORMAL_BG];
   }
  unless (exists($specs->{'-foreground'}))
   {
-   my (@fg) = ('PASSIVE');
-   unshift(@fg,'CHILDREN') if $child;
+   Tk::catch { $cw->Tk::cget('-foreground') };
+   my (@fg) = $@ ? ('PASSIVE') : ('SELF');
+   push(@fg,'CHILDREN') if $child;
    $specs->{'-foreground'} = [\@fg,'foreground','Foreground',BLACK];
   }
  $cw->ConfigAlias(-fg => '-foreground', -bg => '-background');
