@@ -1,13 +1,13 @@
-# Copyright (c) 1995-2000 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-2003 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::ROText;
 
 use vars qw($VERSION);
-$VERSION = '4.005'; # $Id: //depot/Tkutf8/Tk/ROText.pm#5 $
+$VERSION = sprintf '4.%03d', q$Revision: #10 $ =~ /\D(\d+)\s*$/;
 
 use Tk::Text;
-use base  qw(Tk::Text);
+use base  qw(Tk::Derived Tk::Text);
 
 Construct Tk::Widget 'ROText';
 
@@ -28,6 +28,13 @@ sub ClassInit
  return $val;
 }
 
+sub Populate {
+    my($self,$args) = @_;
+    $self->SUPER::Populate($args);
+    my $m = $self->menu->entrycget($self->menu->index('Search'), '-menu');
+    $m->delete($m->index('Replace'));
+}
+
 sub Tk::Widget::ScrlROText { shift->Scrolled('ROText' => @_) }
 
 1;
