@@ -5,7 +5,7 @@ package Tk::Table;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = sprintf '4.%03d', q$Revision: #12 $ =~ /\D(\d+)\s*$/;
+$VERSION = '4.014'; # was: sprintf '4.%03d', q$Revision: #12 $ =~ /\D(\d+)\s*$/;
 
 use Tk::Pretty;
 use AutoLoader;
@@ -170,6 +170,7 @@ sub Layout
   {
    $t->{xsb} = $t->Scrollbar(-orient => 'horizontal', -command => ['xview' => $t]) unless (defined $t->{xsb});
    $xsb   = $t->{xsb};
+   $t->Advertise('xscrollbar' => $xsb);
    $xs[3] = $xsb->ReqHeight;
    if ($sb =~ /n/)
     {
@@ -191,6 +192,7 @@ sub Layout
   {
    $t->{ysb} = $t->Scrollbar(-orient => 'vertical', -command => ['yview' => $t]) unless (defined $t->{ysb});
    $ysb    = $t->{ysb};
+   $t->Advertise('yscrollbar' => $ysb);
    $ys[2]  = $ysb->ReqWidth;
    if ($sb =~ /w/)
     {
@@ -594,5 +596,3 @@ sub see
  return $see;
 }
 
-=cut
-
