@@ -3,7 +3,7 @@
 # modify it under the same terms as Perl itself.
 package Tk::Widget;
 use vars qw($VERSION @DefaultMenuLabels);
-$VERSION = sprintf '4.%03d', q$Revision: #30 $ =~ /\D(\d+)\s*$/;
+$VERSION = '4.033'; # was: sprintf '4.%03d', q$Revision: #30 $ =~ /\D(\d+)\s*$/;
 
 require Tk;
 use AutoLoader;
@@ -616,6 +616,7 @@ sub Palette
    $Palette{'activeForeground'}    = ($c->configure('-activeforeground'))[3];
    $Palette{'background'}          = ($c->configure('-background'))[3];
    $Palette{'disabledForeground'}  = ($c->configure('-disabledforeground'))[3];
+
    $Palette{'foreground'}          = ($c->configure('-foreground'))[3];
    $Palette{'highlightBackground'} = ($c->configure('-highlightbackground'))[3];
    $Palette{'highlightColor'}      = ($c->configure('-highlightcolor'))[3];
@@ -643,9 +644,13 @@ sub Palette
 # be computed from this) or an even number of values consisting of
 # option names and values. The name for an option is the one used
 # for the option database, such as activeForeground, not -activeforeground.
+# Additional special option names are:
+#   priority: set the priority for the option database entries, see Tk::option
 sub setPalette
 {
  my $w = shift->MainWindow;
+ # Just return on monochrome displays, otherwise errors will occur
+ return if $w->depth == 1;
  my %new = (@_ == 1) ? (background => $_[0]) : @_;
  my $priority = delete($new{'priority'}) || 'widgetDefault';
 
@@ -653,8 +658,23 @@ sub setPalette
  # aren't specified, compute them from other colors that are specified.
 
  die 'must specify a background color' if (!exists $new{background});
- $new{'foreground'} = 'black' unless (exists $new{foreground});
  my @bg = $w->rgb($new{'background'});
+
+ if (!exists $new{foreground})
+  {
+   # Note that the range of each value in the triple returned by
+   # [winfo rgb] is 0-65535, and your eyes are more sensitive to
+   # green than to red, and more to red than to blue.
+   my($r,$g,$b) = @bg;
+   if ($r+1.5*$g+0.5*$b > 100000)
+    {
+     $new{'foreground'} = 'black';
+    }
+   else
+    {
+     $new{'foreground'} = 'white';
+    }
+  }
  my @fg = $w->rgb($new{'foreground'});
  my $darkerBg = sprintf('#%02x%02x%02x',9*$bg[0]/2560,9*$bg[1]/2560,9*$bg[2]/2560);
  foreach my $i ('activeForeground','insertBackground','selectForeground','highlightColor')
@@ -703,8 +723,30 @@ sub setPalette
  # doesn't exist, fill it in using the defaults from a few widgets.
  my $Palette = $w->Palette;
 
+ # let's make one of each of the widgets so we know what the 
+ # defaults are currently for this platform.
+ $Tk::___tk_set_palette = $w->Toplevel(Name => "___tk_set_palette");
+ $Tk::___tk_set_palette->withdraw;
+ foreach my $q (qw(Button Canvas Checkbutton Entry Frame Label Labelframe
+		   Listbox Menubutton Menu Message Radiobutton Scale Scrollbar
+		   Spinbox Text
+		 ))
+  {     
+   $Tk::___tk_set_palette->Component($q, $q);
+  }
+
  # Walk the widget hierarchy, recoloring all existing windows.
- $w->RecolorTree(\%new);
+ my $res = $w->RecolorTree(\%new);
+ if ($res->{addOptionDB})
+  {
+   for (@{ $res->{addOptionDB} })
+    {
+     $w->optionAdd(@$_);
+    }
+  }
+
+ $Tk::___tk_set_palette->destroy;
+
  # Change the option database so that future windows will get the
  # same colors.
  foreach my $option (keys %new)
@@ -733,24 +775,49 @@ sub RecolorTree
 {
  my ($w,$colors) = @_;
  local ($@);
- my $Palette = $w->Palette;
+ my @addOptionDB;
+ my $prototype = $Tk::___tk_set_palette->Subwidget($w->Class) || undef;
  foreach my $dbOption (keys %$colors)
   {
    my $option = "-\L$dbOption";
-   my $value;
-   eval {local $SIG{'__DIE__'}; $value = $w->cget($option) };
-   if (defined $value)
+   my $class = ucfirst($dbOption);
+   my @value;
+   eval {local $SIG{'__DIE__'}; @value = $w->configure($option) };
+   if (@value)
     {
-     if ($value eq $Palette->{$dbOption})
+     # if the option database has a preference for this
+     # dbOption, then use it, otherwise use the defaults
+     # for the widget.
+     my $defaultcolor = $w->optionGet($dbOption, $class);
+     no warnings 'uninitialized';
+     if ($defaultcolor eq '' ||
+	 ($prototype && $prototype->cget($option) ne $defaultcolor))
       {
-       $w->configure($option,$colors->{$dbOption});
+       $defaultcolor = $value[3];
+      }
+     if ($defaultcolor ne '' && $value[4] ne '') # XXX why this can be empty?
+      {
+       $defaultcolor = join ",", $w->rgb($defaultcolor);
+       my $chosencolor = join ",", $w->rgb($value[4]);
+       if ($defaultcolor eq $chosencolor)
+        {
+         # Change the option database so that future windows will get
+         # the same colors.
+         push @addOptionDB, ['*'.$w->Class.".$dbOption", $colors->{$dbOption}, 60];
+         $w->configure($option,$colors->{$dbOption});
+        }
       }
     }
   }
  foreach my $child ($w->children)
   {
-   $child->RecolorTree($colors);
+   my $res = $child->RecolorTree($colors);
+   if ($res->{addOptionDB})
+    {
+     push @addOptionDB, @{ $res->{addOptionDB} };
+    }
   }
+ return { addOptionDB => \@addOptionDB };
 }
 # tkDarken --
 # Given a color name, computes a new color value that darkens (or
@@ -888,9 +955,12 @@ sub Busy
    $g->grabRelease;
   }
  $w->update;
- eval {local $SIG{'__DIE__'};  $w->grab };
- $w->update;
- if ($sub)
+ if (Tk::Exists($w))
+  {
+   eval {local $SIG{'__DIE__'};  $w->grab };
+   $w->update;
+  }
+ if ($sub && Tk::Exists($w))
   {
    eval { $sub->() };
    my $err = $@;
@@ -1405,7 +1475,7 @@ sub bindDump {
                     } else {
                         push @out, sprintf( "%s '%s'\n", ' ' x $spc1, $callback->[0], "\n" );
                     }
-                    foreach my $arg (@$callback[1 .. $#{@$callback}]) {
+                    foreach my $arg (@$callback[1 .. $#$callback]) {
                         if (ref $arg) {
                             push @out, sprintf( "%s %-40s", ' ' x $spc2, $arg );
                         } else {
@@ -1508,3 +1578,193 @@ sub pathname
  my $x = $w->winfo('pathname',-displayof  => oct($id));
  return $x->PathName;
 }
+
+# ::tk::UnderlineAmpersand --
+# This procedure takes some text with ampersand and returns
+# text w/o ampersand and position of the ampersand.
+# Double ampersands are converted to single ones.
+# Position returned is -1 when there is no ampersand.
+#
+sub UnderlineAmpersand
+{
+ my (undef,$text) = @_;
+ if ($text =~ m{(?<!&)&(?!&)}g)
+  {
+   my $idx = pos $text;
+   $text =~ s{(?<!&)&(?!&)}{};
+   ($text, $idx);
+  }
+ else
+  {
+   ($text, -1);
+  }
+}
+
+# ::tk::SetAmpText -- 
+# Given widget path and text with "magic ampersands",
+# sets -text and -underline options for the widget
+#
+sub SetAmpText
+{
+ my ($w,$text) = @_;
+ my ($newtext,$under) =  $w->UnderlineAmpersand($text);
+ $w->configure(-text => $newtext, -underline => $under);
+}
+
+# ::tk::AmpWidget --
+# Creates new widget, turning -text option into -text and
+# -underline options, returned by ::tk::UnderlineAmpersand.
+#
+sub AmpWidget
+{
+ my ($w,$class,%args) = @_;
+ my @options;
+ while(my($opt,$val) = each %args)
+  {
+   if ($opt eq "-text")
+    {
+     my ($newtext,$under) = $w->UnderlineAmpersand($val);
+     push @options, -text => $newtext, -underline => $under;
+    }
+   else
+    {
+     push @options, $opt, $val;
+    }
+  }
+ my $result = $w->$class(@options);
+ if ($result->can('AmpWidgetPostHook'))
+  {
+   $result->AmpWidgetPostHook;
+  }
+ return $result;
+}
+
+# ::tk::FindAltKeyTarget --
+# search recursively through the hierarchy of visible widgets
+# to find button or label which has $char as underlined character
+#
+sub FindAltKeyTarget
+{
+ my ($w,$char) = @_;
+ $char = lc $char;
+ if ($w->isa('Tk::Button') || $w->isa('Tk::Label'))
+  {
+   if ($char eq lc substr($w->cget(-text), $w->cget(-underline), 1))
+    {
+     return $w;
+    }
+   else
+    {
+     return undef;
+    }
+  }
+ else
+  {
+   for my $cw ($w->gridSlaves, $w->packSlaves, $w->placeSlaves) # Cannot handle $w->formSlaves here?
+    {
+     my $target = $cw->FindAltKeyTarget($char);
+     return $target if ($target);
+    }
+  }
+ undef;
+}
+
+# ::tk::AltKeyInDialog --
+# <Alt-Key> event handler for standard dialogs. Sends <<AltUnderlined>>
+# to button or label which has appropriate underlined character
+#
+sub AltKeyInDialog
+{
+ my ($w, $key) = @_;
+ my $target = $w->FindAltKeyTarget($key);
+ return if !$target;
+ $target->eventGenerate('<<AltUnderlined>>');
+}
+
+# ::tk::SetFocusGrab --
+#   swap out current focus and grab temporarily (for dialogs)
+# Arguments:
+#   grab	new window to grab
+#   focus	window to give focus to
+# Results:
+#   Returns nothing
+#
+sub SetFocusGrab
+{
+ my ($grab,$focus) = @_;
+ my $index = "$grab,$focus";
+ $Tk::FocusGrab{$index} ||= [];
+ my $data = $Tk::FocusGrab{$index};
+ push @$data, $grab->focusCurrent;
+ my $oldGrab = $grab->grabCurrent;
+ push @$data, $oldGrab;
+ if (Tk::Exists($oldGrab))
+  {
+   push @$data, $oldGrab->grabStatus;
+  }
+ # The "grab" command will fail if another application
+ # already holds the grab.  So catch it.
+ Tk::catch { $grab->grab };
+ if (Tk::Exists($focus))
+  {
+   $focus->focus;
+  }
+}
+
+# ::tk::RestoreFocusGrab --
+#   restore old focus and grab (for dialogs)
+# Arguments:
+#   grab	window that had taken grab
+#   focus	window that had taken focus
+#   destroy	destroy|withdraw - how to handle the old grabbed window
+# Results:
+#   Returns nothing
+#
+sub RestoreFocusGrab
+{
+ my ($grab, $focus, $destroy) = @_;
+ $destroy = 'destroy' if !$destroy;
+ my $index = "$grab,$focus";
+ my ($oldFocus, $oldGrab, $oldStatus);
+ if (exists $Tk::FocusGrab{$index})
+  {
+   ($oldFocus, $oldGrab, $oldStatus) = $Tk::FocusGrab{$index};
+   delete $Tk::FocusGrab{$index};
+  }
+ else
+  {
+   $oldGrab = "";
+  }
+
+ Tk::catch { $oldFocus->focus };
+ if (Tk::Exists($grab))
+  {
+   $grab->grabRelease;
+   if ($destroy eq "withdraw")
+    {
+     $grab->withdraw;
+    }
+   else
+    {
+     $grab->destroy;
+    }
+  }
+ if (Tk::Exists($oldGrab) && $oldGrab->ismapped)
+  {
+   if ($oldStatus eq "global")
+    {
+     $oldGrab->grabGlobal;
+    }
+   else
+    {
+     $oldGrab->grab;
+    }
+  }
+}
+
+# See http://rt.cpan.org/Ticket/Display.html?id=30929 and
+# http://rt.cpan.org/Ticket/Display.html?id=31016
+sub MasterMenu
+ {
+ }
+
