@@ -10,7 +10,7 @@ sub bind {
     my($demo) = @_;
     $TOP = $MW->WidgetDemo(
         -name     => $demo,
-        -text     =>'',				      
+        -text     =>'',
         -title    => 'Text Demonstration - Tag Bindings',
         -iconname => 'bind',
     );
@@ -30,9 +30,7 @@ sub bind {
 	@normal = (-foreground => undef, -background => undef);
     }
 
-    $t->insert('0.0', 'The same tag mechanism that controls display styles in text widgets can also be used to associate Perl commands with regions of text, so that mouse or keyboard actions on the text cause particular Perl commands to be invoked.  For example, in the text below the descriptions of the canvas demonstrations have been tagged.  When you move the mouse over a demo description the description lights up, and when you press button 1 over a description then that particular demonstration is invoked.
-
-');
+    $t->insert('0.0', "The same tag mechanism that controls display styles in text widgets can also be used to associate Perl commands with regions of text, so that mouse or keyboard actions on the text cause particular Perl commands to be invoked.  For example, in the text below the descriptions of the canvas demonstrations have been tagged.  When you move the mouse over a demo description the description lights up, and when you press button 1 over a description then that particular demonstration is invoked.\n\n");
     $t->insert('end','1. Samples of all the different types of items that can be created in canvas widgets.', 'd1');
     $t->insert('end', "\n\n");
     $t->insert('end', '2. A simple two-dimensional plot that allows you to adjust the positions of the data points.', 'd2');
@@ -46,19 +44,19 @@ sub bind {
     $t->insert('end', '6. A grid that demonstrates how canvases can be scrolled.', 'd6');
 
     foreach $tag (qw(d1 d2 d3 d4 d5 d6)) {
-	$t->tag('bind', $tag, '<Any-Enter>' => 
-            sub {shift->tag('configure', $tag, @bold)}
+	$t->tagBind($tag, '<Any-Enter>' =>
+            sub {shift->tagConfigure($tag, @bold)}
         );
-	$t->tag('bind', $tag, '<Any-Leave>' =>
-            sub {shift->tag('configure', $tag, @normal)}
+	$t->tagBind($tag, '<Any-Leave>' =>
+            sub {shift->tagConfigure($tag, @normal)}
         );
     }
-    $t->tag(qw/bind d1 <1>/ => sub {&items('items')});
-    $t->tag(qw/bind d2 <1>/ => sub {&plot('plot')});
-    $t->tag(qw/bind d3 <1>/ => sub {&ctext('ctext')});
-    $t->tag(qw/bind d4 <1>/ => sub {&arrows('arrows')});
-    $t->tag(qw/bind d5 <1>/ => sub {&ruler('ruler')});
-    $t->tag(qw/bind d6 <1>/ => sub {&cscroll('cscroll')});
+    $t->tagBind(qw/d1 <1>/ => sub {&items('items')});
+    $t->tagBind(qw/d2 <1>/ => sub {&plot('plot')});
+    $t->tagBind(qw/d3 <1>/ => sub {&ctext('ctext')});
+    $t->tagBind(qw/d4 <1>/ => sub {&arrows('arrows')});
+    $t->tagBind(qw/d5 <1>/ => sub {&ruler('ruler')});
+    $t->tagBind(qw/d6 <1>/ => sub {&cscroll('cscroll')});
 
     $t->mark(qw/set insert 0.0/);
 
