@@ -18,10 +18,10 @@
  * The format record for the BMP file format:
  */
 
-static int ChnMatchBMP _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Channel chan, Tcl_Obj *fileName,
-	Tcl_Obj *format, int *widthPtr, int *heightPtr));
-static int ObjMatchBMP _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *dataObj,
-	Tcl_Obj *format, int *widthPtr, int *heightPtr));
+static int ChnMatchBMP _ANSI_ARGS_((Tcl_Channel chan, Tcl_Obj *fileName,
+	Tcl_Obj *format, int *widthPtr, int *heightPtr,Tcl_Interp *interp));
+static int ObjMatchBMP _ANSI_ARGS_((Tcl_Obj *dataObj,
+	Tcl_Obj *format, int *widthPtr, int *heightPtr, Tcl_Interp *interp));
 static int ChnReadBMP _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Channel chan,
 	Tcl_Obj *fileName, Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 	int destX, int destY, int width, int height, int srcX, int srcY));
@@ -31,8 +31,7 @@ static int ObjReadBMP _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *dataObj,
 static int ChnWriteBMP _ANSI_ARGS_((Tcl_Interp *interp, char *filename,
 	Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
 static int StringWriteBMP _ANSI_ARGS_((Tcl_Interp *interp,
-	Tcl_DString *data, Tcl_Obj *format,
-	Tk_PhotoImageBlock *blockPtr));
+	Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
 
 Tk_PhotoImageFormat imgFmtBMP = {
     "bmp",					/* name */
@@ -58,12 +57,12 @@ static int CommonWriteBMP _ANSI_ARGS_((Tcl_Interp *interp, MFile *handle,
 	Tk_PhotoImageBlock *blockPtr));
 static void putint _ANSI_ARGS_((MFile *handle, int i));
 
-static int ChnMatchBMP(interp, chan, fileName, format, widthPtr, heightPtr)
-    Tcl_Interp *interp;
+static int ChnMatchBMP(chan, fileName, format, widthPtr, heightPtr, interp)
     Tcl_Channel chan;
     Tcl_Obj *fileName;
     Tcl_Obj *format;
     int *widthPtr, *heightPtr;
+    Tcl_Interp *interp;
 {
     MFile handle;
 
@@ -75,11 +74,11 @@ static int ChnMatchBMP(interp, chan, fileName, format, widthPtr, heightPtr)
     return CommonMatchBMP(&handle, widthPtr, heightPtr, NULL, NULL, NULL, NULL);
 }
 
-static int ObjMatchBMP(interp, data, format, widthPtr, heightPtr)
-    Tcl_Interp *interp;
+static int ObjMatchBMP(data, format, widthPtr, heightPtr, interp)
     Tcl_Obj *data;
     Tcl_Obj *format;
     int *widthPtr, *heightPtr;
+    Tcl_Interp *interp;
 {
     MFile handle;
 
@@ -226,7 +225,7 @@ static int CommonReadBMP(interp, handle, imageHandle, destX, destY,
     CommonMatchBMP(handle, &fileWidth, &fileHeight, &colorMap, &numBits,
 	    &numCols, &comp);
 
-    /*printf("reading %d-bit BMP %dx%d\n", numBits, width, height);*/
+    /* printf("reading %d-bit BMP %dx%d\n", numBits, width, height); */
     if (comp != 0) {
 	Tcl_AppendResult(interp,
 		"Compressed BMP files not (yet) supported", (char *) NULL);
@@ -256,7 +255,7 @@ static int CommonReadBMP(interp, handle, imageHandle, destX, destY,
 	    for( y = height-1; y>=0; y--) {
 		ImgRead(handle, line, bytesPerLine);
 		Tk_PhotoPutBlock(imageHandle, &block, destX, destY+y,
-			width,1);
+			width,1, TK_PHOTO_COMPOSITE_SET);
 	    }
 	    break;
 	case 8:
@@ -268,7 +267,7 @@ static int CommonReadBMP(interp, handle, imageHandle, destX, destY,
 		    expline += 3;
 		}
 		Tk_PhotoPutBlock(imageHandle, &block, destX, destY+y,
-			width,1);
+			width,1, TK_PHOTO_COMPOSITE_SET);
 		expline = block.pixelPtr;
 	    }
 	    break;
@@ -287,7 +286,7 @@ static int CommonReadBMP(interp, handle, imageHandle, destX, destY,
 		    expline += 3;
 		}
 		Tk_PhotoPutBlock(imageHandle, &block, destX, destY+y,
-			width,1);
+			width,1, TK_PHOTO_COMPOSITE_SET);
 		expline = block.pixelPtr;
 	    }
 	    break;
@@ -302,7 +301,7 @@ static int CommonReadBMP(interp, handle, imageHandle, destX, destY,
 		    expline += 3;
 		}
 		Tk_PhotoPutBlock(imageHandle, &block, destX, destY+y,
-			width,1);
+			width,1, TK_PHOTO_COMPOSITE_SET);
 		expline = block.pixelPtr;
 	    }
 	    break;
@@ -362,15 +361,15 @@ static int ChnWriteBMP(interp, filename, format, blockPtr)
     return result;
 }
 
-static int StringWriteBMP(interp, dataPtr, format, blockPtr)
+static int StringWriteBMP(interp, format, blockPtr)
     Tcl_Interp *interp;
-    Tcl_DString *dataPtr;
     Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     MFile handle;
     int result;
     Tcl_DString data;
+    Tcl_DString *dataPtr;
 
     ImgFixStringWriteProc(&data, &interp, &dataPtr, &format, &blockPtr);
 
@@ -405,7 +404,6 @@ static int CommonWriteBMP(interp, handle, blockPtr)
     unsigned char *imagePtr, *pixelPtr;
     unsigned char buf[4];
     int colors[256];
-    int testnum = 10;
 
     greenOffset = blockPtr->offset[1] - blockPtr->offset[0];
     blueOffset = blockPtr->offset[2] - blockPtr->offset[0];
@@ -429,8 +427,12 @@ static int CommonWriteBMP(interp, handle, blockPtr)
 		else
 		    pixel = (pixelPtr[0]<<16) | (pixelPtr[greenOffset]<<8) | pixelPtr[blueOffset];
 		for (i = 0; i < ncolors && pixel != colors[i]; i++);
-		if ((i == ncolors) && (ncolors < 256))
-		    colors[ncolors++] = pixel;
+		if (i == ncolors) {
+		    if (ncolors < 256) {
+			colors[ncolors] = pixel;
+		    }
+		    ncolors++;
+		}
 		pixelPtr += blockPtr->pixelSize;
 	    }
 	}
@@ -472,17 +474,6 @@ static int CommonWriteBMP(interp, handle, blockPtr)
 
     imagePtr =  blockPtr->pixelPtr + blockPtr->offset[0]
 	    + blockPtr->height * blockPtr->pitch;
-    greenOffset = blockPtr->offset[1] - blockPtr->offset[0];
-    blueOffset = blockPtr->offset[2] - blockPtr->offset[0];
-    alphaOffset = blockPtr->offset[0];
-    if (alphaOffset < blockPtr->offset[2]) {
-	alphaOffset = blockPtr->offset[2];
-    }
-    if (++alphaOffset < blockPtr->pixelSize) {
-	alphaOffset -= blockPtr->offset[0];
-    } else {
-	alphaOffset = 0;
-    }
     for (y = 0; y < blockPtr->height; y++) {
 	pixelPtr = imagePtr -= blockPtr->pitch;
 	for (x=0; x<blockPtr->width; x++) {
@@ -502,9 +493,6 @@ static int CommonWriteBMP(interp, handle, blockPtr)
 		buf[2] = pixelPtr[0];
 	    }
 	    ImgWrite(handle, (char *) buf, nbytes);
-	    if (testnum >0) {
-		testnum--;
-	    }
 	    pixelPtr += blockPtr->pixelSize;
 	}
 	if (bperline) {
@@ -513,3 +501,4 @@ static int CommonWriteBMP(interp, handle, blockPtr)
     }
     return(TCL_OK);
 }
+
