@@ -302,6 +302,10 @@ Tk_CreateOptionTable(interp, templatePtr)
 		|| (specPtr->type == TK_OPTION_BORDER)
 		|| (specPtr->type == TK_OPTION_CURSOR)
 		|| (specPtr->type == TK_OPTION_OBJ)
+		|| (specPtr->type == TK_OPTION_CALLBACK)
+		|| (specPtr->type == TK_OPTION_SCALARVAR)
+		|| (specPtr->type == TK_OPTION_ARRAYVAR)
+		|| (specPtr->type == TK_OPTION_HASHVAR)
 		|| (specPtr->type == TK_OPTION_CUSTOM)) {
 	    optionPtr->flags |= OPTION_NEEDS_FREEING;
 	}
@@ -590,6 +594,11 @@ Tk_InitOptions(interp, recordPtr, optionTable, tkwin)
 	    return TCL_ERROR;
 	}
 	Tcl_DecrRefCount(valuePtr);
+
+	if (source == OPTION_DATABASE) {
+		Tcl_DecrRefCount(valuePtr);
+	}
+
     }
     return TCL_OK;
 }
@@ -951,19 +960,40 @@ DoObjConfig(interp, recordPtr, optionPtr, valuePtr, tkwin, savedOptionPtr)
 	    }
 	    break;
 	}
+	/* As we make a copy or increment REFCNT use internal slot
+	   to give us the hook to free it again
+	 */
+	case TK_OPTION_SCALARVAR:
+	case TK_OPTION_ARRAYVAR:
+	case TK_OPTION_HASHVAR:
+	case TK_OPTION_CALLBACK:
 	case TK_OPTION_OBJ: {
-	    Tcl_Obj *new;
-
-	    if (nullOK && ObjectIsEmpty(valuePtr)) {
+	    int code = TCL_OK;
+	    if (nullOK && (LangNull(valuePtr)||ObjectIsEmpty(valuePtr))) {
 		valuePtr = NULL;
-		new = NULL;
-	    } else {
-		new = LangCopyArg(valuePtr);
 	    }
 	    if (internalPtr != NULL) {
 		*((Tcl_Obj **) oldInternalPtr) = *((Tcl_Obj **) internalPtr);
-		*((Tcl_Obj **) internalPtr) = new;
+		code = LangConfigObj(interp,(Tcl_Obj **) internalPtr,
+					     valuePtr,optionPtr->specPtr->type);
+		if (code != TCL_OK) {
+		  return code;
+		}
+	    } else {
+		LangDebug("No attempt to save internal form of %s's %s\n",
+			   Tk_Class(tkwin), specPtr->optionName);
+		abort();
 	    }
+#if 0
+	if (optionPtr->specPtr->type == TK_OPTION_SCALARVAR) {
+		LangDebug("configure %s's %s\n",
+			   Tk_Class(tkwin), specPtr->optionName);
+		LangDumpVec("old",1,(Tcl_Obj **) oldInternalPtr);
+		LangDumpVec("src",1,&valuePtr);
+		LangDumpVec("new",1,(Tcl_Obj **) internalPtr);
+
+	}
+#endif
 	    break;
 	}
 	case TK_OPTION_CUSTOM: {
@@ -1045,8 +1075,12 @@ ObjectIsEmpty(objPtr)
 	return (objPtr->length == 0);
     }
 #endif
+#if 0
     Tcl_GetStringFromObj(objPtr, &length);
     return (length == 0);
+#else
+    return LangNull(objPtr);
+#endif
 }
 
 /*
@@ -1336,6 +1370,31 @@ Tk_SetOptions(interp, recordPtr, optionTable, objc, objv, tkwin, savePtr,
     Tk_SavedOptions *lastSavePtr, *newSavePtr;
     int mask;
 
+#ifdef _LANG
+    /*
+     * If -class Foo is the first option/value pair then
+     * reset widget class if necessary and re-set the defaults
+     * in the option table.
+     */
+    if (tkwin && objc >= 2 && strcmp(Tcl_GetString(objv[0]),"-class") == 0) {
+	char *new = Tcl_GetString(objv[1]);
+	if (strcmp(new,Tk_Class(tkwin)) != 0) {
+	    Tk_SetClass(tkwin, Tcl_GetString(objv[1]));
+	    if (Tk_InitOptions(interp, recordPtr, optionTable, tkwin) != TCL_OK) {
+		return TCL_ERROR;
+	    }
+	}
+	/* See if option was legal */
+	optionPtr = GetOptionFromObj(interp, objv[0], tablePtr);
+	if (optionPtr == NULL) {
+	    if (interp)
+		Tcl_ResetResult(interp);
+	    objc -= 2;
+	    objv += 2;
+	}
+    }
+#endif
+
     if (savePtr != NULL) {
 	savePtr->recordPtr = recordPtr;
 	savePtr->tkwin = tkwin;
@@ -1353,15 +1412,7 @@ Tk_SetOptions(interp, recordPtr, optionTable, objc, objv, tkwin, savePtr,
     for ( ; objc > 0; objc -= 2, objv += 2) {
 	optionPtr = GetOptionFromObj(interp, objv[0], tablePtr);
 	if (optionPtr == NULL) {
-	    if (tkwin != NULL && strcmp(Tcl_GetString(objv[0]),"-class") == 0) {
-		if (interp) {
-		    Tcl_ResetResult(interp); /* Not really an error */
-		}
-		Tk_SetClass(tkwin, Tcl_GetString(objv[1]));
-		continue;
-	    } else {
-	        goto error;
-	    }
+	    goto error;
 	}
 	if (optionPtr->specPtr->type == TK_OPTION_SYNONYM) {
 	    optionPtr = optionPtr->extra.synonymPtr;
@@ -1580,9 +1631,18 @@ Tk_RestoreSavedOptions(savePtr)
 			    = *((Tk_Window *) &savePtr->items[i].internalForm);
 		    break;
 		}
+		case TK_OPTION_SCALARVAR:
+		case TK_OPTION_ARRAYVAR:
+		case TK_OPTION_HASHVAR:
+		case TK_OPTION_CALLBACK:
 		case TK_OPTION_OBJ: {
 		    *((Tcl_Obj **) internalPtr)
 			    = *((Tcl_Obj **) &savePtr->items[i].internalForm);
+#if 0
+		    LangDebug("restore %s's %s\n",
+			Tk_Class(savePtr->tkwin), optionPtr->specPtr->optionName);
+		    LangDumpVec("Now",1,(Tcl_Obj **) internalPtr);
+#endif
 		    break;
 		}
 		case TK_OPTION_CUSTOM: {
@@ -1755,9 +1815,18 @@ FreeResources(optionPtr, objPtr, internalPtr, tkwin)
 		}
 	    }
 	    break;
+	case TK_OPTION_SCALARVAR:
+	case TK_OPTION_ARRAYVAR:
+	case TK_OPTION_HASHVAR:
+	case TK_OPTION_CALLBACK:
 	case TK_OPTION_OBJ:
 	    if (internalFormExists) {
 		if (*((Tcl_Obj **) internalPtr) != NULL) {
+#if 0
+		    LangDebug("free %s's %s\n",
+			Tk_Class(tkwin), optionPtr->specPtr->optionName);
+		    LangDumpVec("Was",1,(Tcl_Obj **) internalPtr);
+#endif
 		    Tcl_DecrRefCount(*((Tcl_Obj **) internalPtr));
 		    *((Tcl_Obj **) internalPtr) = NULL;
 		}
@@ -2100,6 +2169,13 @@ GetObjectForOption(interp, recordPtr, optionPtr, tkwin)
 	    if (cursor != None) {
 		objPtr = Tcl_NewStringObj(
 			Tk_NameOfCursor(Tk_Display(tkwin), cursor), -1);
+#ifdef _LANG
+	    {
+	     Tcl_Obj **elemv;
+	     int elemc;
+	     Tcl_ListObjGetElements(interp,objPtr,&elemc,&elemv);
+	    }
+#endif
 	    }
 	    break;
 	}
@@ -2124,11 +2200,24 @@ GetObjectForOption(interp, recordPtr, optionPtr, tkwin)
 	    }
 	    break;
 	}
+	case TK_OPTION_SCALARVAR:
+	case TK_OPTION_ARRAYVAR:
+	case TK_OPTION_HASHVAR: {
+	    LangSetVar(&objPtr,*((Tcl_Obj **) internalPtr));
+	    break;
+	}
+	
+	case TK_OPTION_CALLBACK:
 	case TK_OPTION_OBJ: {
 	    Tcl_Obj *obj = *((Tcl_Obj **) internalPtr);
 	    if (obj != NULL) {
 		Tcl_IncrRefCount(obj);
 		objPtr = obj;
+#if 0
+		LangDebug("return %s's %s\n",
+			Tk_Class(tkwin), optionPtr->specPtr->optionName);
+		LangDumpVec("Now",1,&obj);
+#endif
 	    }
 	    break;
 	}
