@@ -17,7 +17,7 @@
 
 static int		AddClause _ANSI_ARGS_((
 			    Tcl_Interp * interp, FileFilter * filterPtr,
-			    Arg patternsStr, Arg ostypesStr,
+			    Tcl_Obj * patternsStr, Tcl_Obj * ostypesStr,
 			    int isWindows));
 static void		FreeClauses _ANSI_ARGS_((FileFilter * filterPtr));
 static void		FreeGlobPatterns _ANSI_ARGS_((
@@ -82,12 +82,12 @@ int
 TkGetFileFilters(interp, flistPtr, arg, isWindows)
     Tcl_Interp *interp;		/* Interpreter to use for error reporting. */
     FileFilterList * flistPtr;	/* Stores the list of file filters. */
-    Arg arg;		/* Value of the -filetypes option. */
+    Tcl_Obj * arg;		/* Value of the -filetypes option. */
     int isWindows;		/* True if we are running on Windows. */
 {
     int listArgc;
-    Arg * listArgv = NULL;
-    Arg * typeInfo = NULL;
+    Tcl_Obj * * listArgv = NULL;
+    Tcl_Obj * * typeInfo = NULL;
     int typeCount;
     int code = TCL_OK;
     int i;
@@ -201,8 +201,8 @@ TkFreeFileFilters(flistPtr)
 static int AddClause(interp, filterPtr, patternsStr, ostypesStr, isWindows)
     Tcl_Interp * interp;	/* Interpreter to use for error reporting. */
     FileFilter * filterPtr;	/* Stores the new filter clause */
-    Arg patternsStr;		/* A TCL list of glob patterns. */
-    Arg ostypesStr;		/* A TCL list of Mac OSType strings. */
+    Tcl_Obj * patternsStr;		/* A TCL list of glob patterns. */
+    Tcl_Obj * ostypesStr;		/* A TCL list of Mac OSType strings. */
     int isWindows;		/* True if we are running on Windows; False
 				 * if we are running on the Mac; Glob
 				 * patterns need to be processed differently
