@@ -1,9 +1,9 @@
 /*
  * tkImage.c --
  *
- *	This module implements the image protocol, which allows lots
- *	of different kinds of images to be used in lots of different
- *	widgets.
+ *      This module implements the image protocol, which allows lots
+ *      of different kinds of images to be used in lots of different
+ *      widgets.
  *
  * Copyright (c) 1994 The Regents of the University of California.
  * Copyright (c) 1994-1997 Sun Microsystems, Inc.
@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkImage.c,v 1.17 2002/08/05 04:30:39 dgp Exp $
+ * RCS: @(#) $Id: tkImage.c,v 1.19.2.2 2003/09/17 23:45:05 dgp Exp $
  */
 
 #include "tkInt.h"
@@ -24,10 +24,10 @@
  */
 
 typedef struct Image {
-    Tk_Window tkwin;		/* Window passed to Tk_GetImage (needed to
+    Tk_Window tkwin;            /* Window passed to Tk_GetImage (needed to
 				 * "re-get" the image later if the manager
 				 * changes). */
-    Display *display;		/* Display for tkwin.  Needed because when
+    Display *display;           /* Display for tkwin.  Needed because when
 				 * the image is eventually freed tkwin may
 				 * not exist anymore. */
     struct ImageMaster *masterPtr;
@@ -41,7 +41,7 @@ typedef struct Image {
 				 * in a way that affects redisplay. */
     ClientData widgetClientData;
 				/* Argument to pass to changeProc. */
-    struct Image *nextPtr;	/* Next in list of all image instances
+    struct Image *nextPtr;      /* Next in list of all image instances
 				 * associated with the same name. */
 
 } Image;
@@ -54,23 +54,23 @@ typedef struct Image {
  */
 
 typedef struct ImageMaster {
-    Tk_ImageType *typePtr;	/* Information about image type.  NULL means
+    Tk_ImageType *typePtr;      /* Information about image type.  NULL means
 				 * that no image manager owns this image:  the
 				 * image was deleted. */
-    ClientData masterData;	/* One-word argument to pass to image mgr
+    ClientData masterData;      /* One-word argument to pass to image mgr
 				 * when dealing with the master, as opposed
 				 * to instances. */
-    int width, height;		/* Last known dimensions for image. */
-    Tcl_HashTable *tablePtr;	/* Pointer to hash table containing image
+    int width, height;          /* Last known dimensions for image. */
+    Tcl_HashTable *tablePtr;    /* Pointer to hash table containing image
 				 * (the imageTable field in some TkMainInfo
 				 * structure). */
-    Tcl_HashEntry *hPtr;	/* Hash entry in mainPtr->imageTable for
+    Tcl_HashEntry *hPtr;        /* Hash entry in mainPtr->imageTable for
 				 * this structure (used to delete the hash
 				 * entry). */
-    Image *instancePtr;		/* Pointer to first in list of instances
+    Image *instancePtr;         /* Pointer to first in list of instances
 				 * derived from this name. */
-    int deleted;		/* Flag set when image is being deleted. */
-    TkWindow *winPtr;		/* Main window of interpreter (used to
+    int deleted;                /* Flag set when image is being deleted. */
+    TkWindow *winPtr;           /* Main window of interpreter (used to
 				 * detect when the world is falling apart.) */
 } ImageMaster;
 
@@ -86,37 +86,38 @@ static Tcl_ThreadDataKey dataKey;
  * Prototypes for local procedures:
  */
 
-static void	DeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
-static void	EventuallyDeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
+static void     DeleteImage _ANSI_ARGS_((ImageMaster *masterPtr));
+static void     EventuallyDeleteImage _ANSI_ARGS_((ImageMaster *masterPtr,
+						   int forgetHashEntryNow));
 
 /*
  *----------------------------------------------------------------------
  *
  * Tk_CreateOldImageType, Tk_CreateImageType --
  *
- *	This procedure is invoked by an image manager to tell Tk about
- *	a new kind of image and the procedures that manage the new type.
- *	The procedure is typically invoked during Tcl_AppInit.
+ *      This procedure is invoked by an image manager to tell Tk about
+ *      a new kind of image and the procedures that manage the new type.
+ *      The procedure is typically invoked during Tcl_AppInit.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The new image type is entered into a table used in the "image
- *	create" command.
+ *      The new image type is entered into a table used in the "image
+ *      create" command.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tk_CreateOldImageType(typePtr)
-    Tk_ImageType *typePtr;	/* Structure describing the type.  All of
+    Tk_ImageType *typePtr;      /* Structure describing the type.  All of
 				 * the fields except "nextPtr" must be filled
 				 * in by caller.  Must not have been passed
 				 * to Tk_CreateImageType previously. */
 {
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
-            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     typePtr->nextPtr = tsdPtr->oldImageTypeList;
     tsdPtr->oldImageTypeList = typePtr;
@@ -124,13 +125,13 @@ Tk_CreateOldImageType(typePtr)
 
 void
 Tk_CreateImageType(typePtr)
-    Tk_ImageType *typePtr;	/* Structure describing the type.  All of
+    Tk_ImageType *typePtr;      /* Structure describing the type.  All of
 				 * the fields except "nextPtr" must be filled
 				 * in by caller.  Must not have been passed
 				 * to Tk_CreateImageType previously. */
 {
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
-            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     typePtr->nextPtr = tsdPtr->imageTypeList;
     tsdPtr->imageTypeList = typePtr;
@@ -141,24 +142,24 @@ Tk_CreateImageType(typePtr)
  *
  * Tk_ImageObjCmd --
  *
- *	This procedure is invoked to process the "image" Tcl command.
- *	See the user documentation for details on what it does.
+ *      This procedure is invoked to process the "image" Tcl command.
+ *      See the user documentation for details on what it does.
  *
  * Results:
- *	A standard Tcl result.
+ *      A standard Tcl result.
  *
  * Side effects:
- *	See the user documentation.
+ *      See the user documentation.
  *
  *----------------------------------------------------------------------
  */
 
 int
 Tk_ImageObjCmd(clientData, interp, objc, objv)
-    ClientData clientData;	/* Main window associated with interpreter. */
-    Tcl_Interp *interp;		/* Current interpreter. */
-    int objc;			/* Number of arguments. */
-    Tcl_Obj *CONST objv[];	/* Argument strings. */
+    ClientData clientData;      /* Main window associated with interpreter. */
+    Tcl_Interp *interp;         /* Current interpreter. */
+    int objc;                   /* Number of arguments. */
+    Tcl_Obj *CONST objv[];      /* Argument strings. */
 {
     static CONST char *imageOptions[] = {
 	"create", "delete", "height", "inuse", "names", "type", "types",
@@ -178,7 +179,7 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
     char idString[16 + TCL_INTEGER_SPACE], *name;
     TkDisplay *dispPtr = winPtr->dispPtr;
     ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
-            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (objc < 2) {
 	Tcl_WrongNumArgs(interp, 1, objv, "option ?args?");
@@ -232,13 +233,31 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 	     */
 
 	    if ((objc == 3) || (*(arg = Tcl_GetString(objv[3])) == '-')) {
-		dispPtr->imageId++;
-		sprintf(idString, "image%d", dispPtr->imageId);
-		name = idString;
+                Tcl_CmdInfo dummy;
+                do {
+		    dispPtr->imageId++;
+		    sprintf(idString, "image%d", dispPtr->imageId);
+		    name = idString;
+                } while (Tcl_GetCommandInfo(interp, name, &dummy) != 0);
 		firstOption = 3;
 	    } else {
+		TkWindow *topWin;
+
 		name = arg;
 		firstOption = 4;
+		/*
+		 * Need to check if the _command_ that we are about to
+		 * create is the name of the current master widget
+		 * command (normally "." but could have been renamed)
+		 * and fail in that case before a really nasty and
+		 * hard to stop crash happens.
+		 */
+		topWin = (TkWindow *) TkToplevelWindowForCommand(interp, name);
+		if (topWin != NULL && winPtr->mainPtr->winPtr == topWin) {
+		    Tcl_AppendResult(interp, "images may not be named the ",
+			    "same as the main window", (char *) NULL);
+		    return TCL_ERROR;
+		}
 	    }
 
 	    /*
@@ -301,7 +320,7 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 	    if ((*typePtr->createProc)(interp, name, objc,
 		    args, typePtr, (Tk_ImageMaster) masterPtr,
 		    &masterPtr->masterData) != TCL_OK) {
-		EventuallyDeleteImage(masterPtr);
+		EventuallyDeleteImage(masterPtr, 0);
 		Tcl_Release((ClientData) masterPtr);
 		if (oldimage) {
 		    ckfree((char *) args);
@@ -318,9 +337,7 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 		imagePtr->instanceData = (*typePtr->getProc)(
 		    imagePtr->tkwin, masterPtr->masterData);
 	    }
-	    Tcl_SetResult(interp,
-		    Tcl_GetHashKey(&winPtr->mainPtr->imageTable, hPtr),
-		    TCL_STATIC);
+            Tcl_SetObjResult(interp, LangObjectObj( interp, Tcl_GetHashKey(&winPtr->mainPtr->imageTable, hPtr)));
 	    break;
 	}
 	case IMAGE_DELETE: {
@@ -383,8 +400,8 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 	    }
 	    hPtr = Tcl_FirstHashEntry(&winPtr->mainPtr->imageTable, &search);
 	    for ( ; hPtr != NULL; hPtr = Tcl_NextHashEntry(&search)) {
-		Tcl_AppendElement(interp, Tcl_GetHashKey(
-		    &winPtr->mainPtr->imageTable, hPtr));
+		Tcl_ListObjAppendElement(interp, Tcl_GetObjResult(interp),
+			LangObjectObj(interp,Tcl_GetHashKey(&winPtr->mainPtr->imageTable, hPtr)));
 	    }
 	    break;
 	}
@@ -449,16 +466,16 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
  *
  * Tk_ImageChanged --
  *
- *	This procedure is called by an image manager whenever something
- *	has happened that requires the image to be redrawn (some of its
- *	pixels have changed, or its size has changed).
+ *      This procedure is called by an image manager whenever something
+ *      has happened that requires the image to be redrawn (some of its
+ *      pixels have changed, or its size has changed).
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Any widgets that display the image are notified so that they
- *	can redisplay themselves as appropriate.
+ *      Any widgets that display the image are notified so that they
+ *      can redisplay themselves as appropriate.
  *
  *----------------------------------------------------------------------
  */
@@ -466,11 +483,11 @@ Tk_ImageObjCmd(clientData, interp, objc, objv)
 void
 Tk_ImageChanged(imageMaster, x, y, width, height, imageWidth,
 	imageHeight)
-    Tk_ImageMaster imageMaster;	/* Image that needs redisplay. */
-    int x, y;			/* Coordinates of upper-left pixel of
+    Tk_ImageMaster imageMaster; /* Image that needs redisplay. */
+    int x, y;                   /* Coordinates of upper-left pixel of
 				 * region of image that needs to be
 				 * redrawn. */
-    int width, height;		/* Dimensions (in pixels) of region of
+    int width, height;          /* Dimensions (in pixels) of region of
 				 * image to redraw.  If either dimension
 				 * is zero then the image doesn't need to
 				 * be redrawn (perhaps all that happened is
@@ -494,24 +511,27 @@ Tk_ImageChanged(imageMaster, x, y, width, height, imageWidth,
  *
  * Tk_NameOfImage --
  *
- *	Given a token for an image master, this procedure returns
- *	the name of the image.
+ *      Given a token for an image master, this procedure returns
+ *      the name of the image.
  *
  * Results:
- *	The return value is the string name for imageMaster.
+ *      The return value is the string name for imageMaster.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 CONST char *
 Tk_NameOfImage(imageMaster)
-    Tk_ImageMaster imageMaster;		/* Token for image. */
+    Tk_ImageMaster imageMaster;         /* Token for image. */
 {
     ImageMaster *masterPtr = (ImageMaster *) imageMaster;
 
+    if (masterPtr->hPtr == NULL) {
+	return NULL;
+    }
     return Tcl_GetHashKey(masterPtr->tablePtr, masterPtr->hPtr);
 }
 
@@ -520,36 +540,36 @@ Tk_NameOfImage(imageMaster)
  *
  * Tk_GetImage --
  *
- *	This procedure is invoked by a widget when it wants to use
- *	a particular image in a particular window.
+ *      This procedure is invoked by a widget when it wants to use
+ *      a particular image in a particular window.
  *
  * Results:
- *	The return value is a token for the image.  If there is no image
- *	by the given name, then NULL is returned and an error message is
- *	left in the interp's result.
+ *      The return value is a token for the image.  If there is no image
+ *      by the given name, then NULL is returned and an error message is
+ *      left in the interp's result.
  *
  * Side effects:
- *	Tk records the fact that the widget is using the image, and
- *	it will invoke changeProc later if the widget needs redisplay
- *	(i.e. its size changes or some of its pixels change).  The
- *	caller must eventually invoke Tk_FreeImage when it no longer
- *	needs the image.
+ *      Tk records the fact that the widget is using the image, and
+ *      it will invoke changeProc later if the widget needs redisplay
+ *      (i.e. its size changes or some of its pixels change).  The
+ *      caller must eventually invoke Tk_FreeImage when it no longer
+ *      needs the image.
  *
  *----------------------------------------------------------------------
  */
 
 Tk_Image
 Tk_GetImage(interp, tkwin, name, changeProc, clientData)
-    Tcl_Interp *interp;		/* Place to leave error message if image
+    Tcl_Interp *interp;         /* Place to leave error message if image
 				 * can't be found. */
-    Tk_Window tkwin;		/* Token for window in which image will
+    Tk_Window tkwin;            /* Token for window in which image will
 				 * be used. */
-    CONST char *name;		/* Name of desired image. */
+    CONST char *name;           /* Name of desired image. */
     Tk_ImageChangedProc *changeProc;
 				/* Procedure to invoke when redisplay is
 				 * needed because image's pixels or size
 				 * changed. */
-    ClientData clientData;	/* One-word argument to pass to damageProc. */
+    ClientData clientData;      /* One-word argument to pass to damageProc. */
 {
     Tcl_HashEntry *hPtr;
     ImageMaster *masterPtr;
@@ -586,22 +606,22 @@ Tk_GetImage(interp, tkwin, name, changeProc, clientData)
  *
  * Tk_FreeImage --
  *
- *	This procedure is invoked by a widget when it no longer needs
- *	an image acquired by a previous call to Tk_GetImage.  For each
- *	call to Tk_GetImage there must be exactly one call to Tk_FreeImage.
+ *      This procedure is invoked by a widget when it no longer needs
+ *      an image acquired by a previous call to Tk_GetImage.  For each
+ *      call to Tk_GetImage there must be exactly one call to Tk_FreeImage.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The association between the image and the widget is removed.
+ *      The association between the image and the widget is removed.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tk_FreeImage(image)
-    Tk_Image image;		/* Token for image that is no longer
+    Tk_Image image;             /* Token for image that is no longer
 				 * needed by a widget. */
 {
     Image *imagePtr = (Image *) image;
@@ -633,9 +653,10 @@ Tk_FreeImage(image)
      */
 
     if ((masterPtr->typePtr == NULL) && (masterPtr->instancePtr == NULL)) {
-        if (masterPtr->hPtr != NULL) {
+	if (masterPtr->hPtr != NULL) {
 	    Tcl_DeleteHashEntry(masterPtr->hPtr);
-        }
+	}
+	Tcl_Release(masterPtr->winPtr);
 	ckfree((char *) masterPtr);
     }
 }
@@ -645,28 +666,28 @@ Tk_FreeImage(image)
  *
  * Tk_PostscriptImage --
  *
- *	This procedure is called by widgets that contain images in order
- *	to redisplay an image on the screen or an off-screen pixmap.
+ *      This procedure is called by widgets that contain images in order
+ *      to redisplay an image on the screen or an off-screen pixmap.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The image's manager is notified, and it redraws the desired
- *	portion of the image before returning.
+ *      The image's manager is notified, and it redraws the desired
+ *      portion of the image before returning.
  *
  *----------------------------------------------------------------------
  */
 
 int
 Tk_PostscriptImage(image, interp, tkwin, psinfo, x, y, width, height, prepass)
-    Tk_Image image;		/* Token for image to redisplay. */
+    Tk_Image image;             /* Token for image to redisplay. */
     Tcl_Interp *interp;
     Tk_Window tkwin;
-    Tk_PostscriptInfo psinfo;	/* postscript info */
-    int x, y;			/* Upper-left pixel of region in image that
+    Tk_PostscriptInfo psinfo;   /* postscript info */
+    int x, y;                   /* Upper-left pixel of region in image that
 				 * needs to be redisplayed. */
-    int width, height;		/* Dimensions of region to redraw. */
+    int width, height;          /* Dimensions of region to redraw. */
     int prepass;
 {
     Image *imagePtr = (Image *) image;
@@ -705,7 +726,7 @@ Tk_PostscriptImage(image, interp, tkwin, psinfo, x, y, width, height, prepass)
      */
 
     pmap = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
-                        width, height, Tk_Depth(tkwin));
+			width, height, Tk_Depth(tkwin));
 
     gcValues.foreground = WhitePixelOfScreen(Tk_Screen(tkwin));
     newGC = Tk_GetGC(tkwin, GCForeground, &gcValues);
@@ -740,15 +761,15 @@ Tk_PostscriptImage(image, interp, tkwin, psinfo, x, y, width, height, prepass)
  *
  * Tk_RedrawImage --
  *
- *	This procedure is called by widgets that contain images in order
- *	to redisplay an image on the screen or an off-screen pixmap.
+ *      This procedure is called by widgets that contain images in order
+ *      to redisplay an image on the screen or an off-screen pixmap.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The image's manager is notified, and it redraws the desired
- *	portion of the image before returning.
+ *      The image's manager is notified, and it redraws the desired
+ *      portion of the image before returning.
  *
  *----------------------------------------------------------------------
  */
@@ -756,16 +777,16 @@ Tk_PostscriptImage(image, interp, tkwin, psinfo, x, y, width, height, prepass)
 void
 Tk_RedrawImage(image, imageX, imageY, width, height, drawable,
 	drawableX, drawableY)
-    Tk_Image image;		/* Token for image to redisplay. */
-    int imageX, imageY;		/* Upper-left pixel of region in image that
+    Tk_Image image;             /* Token for image to redisplay. */
+    int imageX, imageY;         /* Upper-left pixel of region in image that
 				 * needs to be redisplayed. */
-    int width, height;		/* Dimensions of region to redraw. */
-    Drawable drawable;		/* Drawable in which to display image
+    int width, height;          /* Dimensions of region to redraw. */
+    Drawable drawable;          /* Drawable in which to display image
 				 * (window or pixmap).  If this is a pixmap,
 				 * it must have the same depth as the window
 				 * used in the Tk_GetImage call for the
 				 * image. */
-    int drawableX, drawableY;	/* Coordinates in drawable that correspond
+    int drawableX, drawableY;   /* Coordinates in drawable that correspond
 				 * to imageX and imageY. */
 {
     Image *imagePtr = (Image *) image;
@@ -808,23 +829,23 @@ Tk_RedrawImage(image, imageX, imageY, width, height, drawable,
  *
  * Tk_SizeOfImage --
  *
- *	This procedure returns the current dimensions of an image.
+ *      This procedure returns the current dimensions of an image.
  *
  * Results:
- *	The width and height of the image are returned in *widthPtr
- *	and *heightPtr.
+ *      The width and height of the image are returned in *widthPtr
+ *      and *heightPtr.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tk_SizeOfImage(image, widthPtr, heightPtr)
-    Tk_Image image;		/* Token for image whose size is wanted. */
-    int *widthPtr;		/* Return width of image here. */
-    int *heightPtr;		/* Return height of image here. */
+    Tk_Image image;             /* Token for image whose size is wanted. */
+    int *widthPtr;              /* Return width of image here. */
+    int *heightPtr;             /* Return height of image here. */
 {
     Image *imagePtr = (Image *) image;
 
@@ -837,25 +858,25 @@ Tk_SizeOfImage(image, widthPtr, heightPtr)
  *
  * Tk_DeleteImage --
  *
- *	Given the name of an image, this procedure destroys the
- *	image.
+ *      Given the name of an image, this procedure destroys the
+ *      image.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The image is destroyed; existing instances will display as
- *	blank areas.  If no such image exists then the procedure does
- *	nothing.
+ *      The image is destroyed; existing instances will display as
+ *      blank areas.  If no such image exists then the procedure does
+ *      nothing.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tk_DeleteImage(interp, name)
-    Tcl_Interp *interp;		/* Interpreter in which the image was
+    Tcl_Interp *interp;         /* Interpreter in which the image was
 				 * created. */
-    CONST char *name;		/* Name of image. */
+    CONST char *name;           /* Name of image. */
 {
     Tcl_HashEntry *hPtr;
     TkWindow *winPtr;
@@ -876,22 +897,22 @@ Tk_DeleteImage(interp, name)
  *
  * DeleteImage --
  *
- *	This procedure is responsible for deleting an image.
+ *      This procedure is responsible for deleting an image.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	The connection is dropped between instances of this image and
- *	an image master.  Image instances will redisplay themselves
- *	as empty areas, but existing instances will not be deleted.
+ *      The connection is dropped between instances of this image and
+ *      an image master.  Image instances will redisplay themselves
+ *      as empty areas, but existing instances will not be deleted.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 DeleteImage(masterPtr)
-    ImageMaster *masterPtr;	/* Pointer to main data structure for image. */
+    ImageMaster *masterPtr;     /* Pointer to main data structure for image. */
 {
     Image *imagePtr;
     Tk_ImageType *typePtr;
@@ -910,7 +931,7 @@ DeleteImage(masterPtr)
 	(*typePtr->deleteProc)(masterPtr->masterData);
     }
     if (masterPtr->instancePtr == NULL) {
-        if ((masterPtr->winPtr->flags & TK_ALREADY_DEAD) == 0) {
+	if (masterPtr->hPtr != NULL) {
 	    Tcl_DeleteHashEntry(masterPtr->hPtr);
 	}
 	Tcl_Release((ClientData) masterPtr->winPtr);
@@ -923,23 +944,27 @@ DeleteImage(masterPtr)
  *
  * EventuallyDeleteImage --
  *
- *	Arrange for an image to be deleted when it is safe to do so.
+ *      Arrange for an image to be deleted when it is safe to do so.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Image will get freed, though not until it is no longer
- *	Tcl_Preserve()d by anything.  May be called multiple times on
- *	the same image without ill effects.
+ *      Image will get freed, though not until it is no longer
+ *      Tcl_Preserve()d by anything.  May be called multiple times on
+ *      the same image without ill effects.
  *
  *----------------------------------------------------------------------
  */
 
 static void
-EventuallyDeleteImage(masterPtr)
-    ImageMaster *masterPtr;	/* Pointer to main data structure for image. */
+EventuallyDeleteImage(masterPtr, forgetHashEntryNow)
+    ImageMaster *masterPtr;     /* Pointer to main data structure for image. */
+    int forgetHashEntryNow;
 {
+    if (forgetHashEntryNow) {
+	masterPtr->hPtr = NULL;
+    }
     if (!masterPtr->deleted) {
 	masterPtr->deleted = 1;
 	Tcl_EventuallyFree((ClientData) masterPtr,
@@ -952,23 +977,23 @@ EventuallyDeleteImage(masterPtr)
  *
  * TkDeleteAllImages --
  *
- *	This procedure is called when an application is deleted.  It
- *	calls back all of the managers for all images so that they
- *	can cleanup, then it deletes all of Tk's internal information
- *	about images.
+ *      This procedure is called when an application is deleted.  It
+ *      calls back all of the managers for all images so that they
+ *      can cleanup, then it deletes all of Tk's internal information
+ *      about images.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	All information for all images gets deleted.
+ *      All information for all images gets deleted.
  *
  *----------------------------------------------------------------------
  */
 
 void
 TkDeleteAllImages(mainPtr)
-    TkMainInfo *mainPtr;	/* Structure describing application that is
+    TkMainInfo *mainPtr;        /* Structure describing application that is
 				 * going away. */
 {
     Tcl_HashSearch search;
@@ -976,7 +1001,7 @@ TkDeleteAllImages(mainPtr)
 
     for (hPtr = Tcl_FirstHashEntry(&mainPtr->imageTable, &search);
 	    hPtr != NULL; hPtr = Tcl_NextHashEntry(&search)) {
-	EventuallyDeleteImage((ImageMaster *) Tcl_GetHashValue(hPtr));
+	EventuallyDeleteImage((ImageMaster *) Tcl_GetHashValue(hPtr), 1);
     }
     Tcl_DeleteHashTable(&mainPtr->imageTable);
 }
@@ -986,28 +1011,28 @@ TkDeleteAllImages(mainPtr)
  *
  * Tk_GetImageMasterData --
  *
- *	Given the name of an image, this procedure returns the type
- *	of the image and the clientData associated with its master.
+ *      Given the name of an image, this procedure returns the type
+ *      of the image and the clientData associated with its master.
  *
  * Results:
- *	If there is no image by the given name, then NULL is returned
- *	and a NULL value is stored at *typePtrPtr.  Otherwise the return
- *	value is the clientData returned by the createProc when the
- *	image was created and a pointer to the type structure for the
- *	image is stored at *typePtrPtr.
+ *      If there is no image by the given name, then NULL is returned
+ *      and a NULL value is stored at *typePtrPtr.  Otherwise the return
+ *      value is the clientData returned by the createProc when the
+ *      image was created and a pointer to the type structure for the
+ *      image is stored at *typePtrPtr.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 ClientData
 Tk_GetImageMasterData(interp, name, typePtrPtr)
-    Tcl_Interp *interp;		/* Interpreter in which the image was
+    Tcl_Interp *interp;         /* Interpreter in which the image was
 				 * created. */
-    CONST char *name;		/* Name of image. */
-    Tk_ImageType **typePtrPtr;	/* Points to location to fill in with
+    CONST char *name;           /* Name of image. */
+    Tk_ImageType **typePtrPtr;  /* Points to location to fill in with
 				 * pointer to type information for image. */
 {
     Tcl_HashEntry *hPtr;
@@ -1025,19 +1050,18 @@ Tk_GetImageMasterData(interp, name, typePtrPtr)
     return masterPtr->masterData;
 }
 
-/* ARGSUSED */
-static void
-TileChangedProc(clientData, x, y, width, height, imageWidth, imageHeight)
-    ClientData clientData;
-    int x, y, width, height;	/* Not used */
-    int imageWidth, imageHeight;
-{
-}
 
 /*========================================================================
  * Tile support - reimplemented avoiding BLTisms
  *========================================================================*/
 
+typedef struct Tk_TileChange_
+{
+ struct Tk_TileChange_ *next;
+ Tk_TileChangedProc *changeProc;
+ ClientData clientData;
+} Tk_TileChange;
+
 struct Tk_Tile_
 {
  Tk_Image image;
@@ -1045,9 +1069,10 @@ struct Tk_Tile_
  int height;
  Tk_Window tkwin;
  Pixmap   pixmap;
+ Tk_TileChange *handlers;
 };
 
-static void		TileImageChanged _ANSI_ARGS_((ClientData clientData,
+static void             TileImageChanged _ANSI_ARGS_((ClientData clientData,
 			    int x, int y, int width, int height,
 			    int imgWidth, int imgHeight));
 
@@ -1059,27 +1084,34 @@ int y;
 int width;
 int height;
 {
-#ifdef NOT_YET
     Tk_Tile tile = (Tk_Tile) clientData;
-    if (tile->width < imgWidth || tile->height < imgHeight) {
+    Tk_TileChange *handler;
+    if (tile->pixmap == None ||
+        tile->width != imgWidth || tile->height != imgHeight) {
 	Tk_Window tkwin = tile->tkwin;
 	if (tile->pixmap) {
 	    Tk_FreePixmap(Tk_Display(tkwin),tile->pixmap);
 	    tile->width  = 0;
 	    tile->height = 0;
+	    tile->pixmap = None;
 	}
-	tile->pixmap = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
+	if (imgWidth >= 0 && imgHeight >= 0) {
+	    tile->pixmap = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
 				imgWidth, imgHeight, Tk_Depth(tkwin));
-	if (tile->pixmap) {
-	    tile->width  = imgWidth;
-	    tile->height = imgHeight;
+	    if (tile->pixmap) {
+		tile->width  = imgWidth;
+		tile->height = imgHeight;
+	    }
 	}
     }
-    if (tile->pixmap) {
+    if (tile->pixmap != None) {
 	Tk_RedrawImage(tile->image, 0, 0, tile->width, tile->height,
 		       tile->pixmap, 0, 0);
     }
-#endif
+    for (handler = tile->handlers; handler != NULL; handler = handler->next) {
+	(*handler->changeProc)(handler->clientData, tile);
+
+    }
 }
 
 /*
@@ -1087,24 +1119,24 @@ int height;
  *
  * Tk_GetTile
  *
- *	Convert the named image into a tile.
+ *      Convert the named image into a tile.
  *
  * Results:
- *	If the image is valid, a new tile is returned.  If the name
- *	does not represent a proper image, an error message is left in
- *	interp->result.
+ *      If the image is valid, a new tile is returned.  If the name
+ *      does not represent a proper image, an error message is left in
+ *      interp->result.
  *
  * Side Effects:
- *	Memory and X resources are allocated.  Bookkeeping is
- *	maintained on the tile (i.e. width, height, and name).
+ *      Memory and X resources are allocated.  Bookkeeping is
+ *      maintained on the tile (i.e. width, height, and name).
  *
  *----------------------------------------------------------------------
  */
 Tk_Tile
 Tk_GetTile(interp, tkwin, imageName)
-    Tcl_Interp *interp;		/* Interpreter to report results back to */
-    Tk_Window tkwin;		/* Window on the same display as tile */
-    CONST char *imageName;	/* Name of image */
+    Tcl_Interp *interp;         /* Interpreter to report results back to */
+    Tk_Window tkwin;            /* Window on the same display as tile */
+    CONST char *imageName;      /* Name of image */
 {
     Tk_Tile tile = (Tk_Tile) ckalloc(sizeof(struct Tk_Tile_));
     memset(tile,0,sizeof(*tile));
@@ -1123,24 +1155,29 @@ Tk_GetTile(interp, tkwin, imageName)
  *
  * Tk_FreeTile
  *
- *	Release the resources associated with the tile.
+ *      Release the resources associated with the tile.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side Effects:
- *	Memory and X resources are freed.  Bookkeeping information
- *	about the tile (i.e. width, height, and name) is discarded.
+ *      Memory and X resources are freed.  Bookkeeping information
+ *      about the tile (i.e. width, height, and name) is discarded.
  *
  */
 void
 Tk_FreeTile(tile)
-    Tk_Tile tile;		/* Tile to be deleted */
+    Tk_Tile tile;               /* Tile to be deleted */
 {
  if (tile->image)
   Tk_FreeImage(tile->image);
  if (tile->pixmap)
   Tk_FreePixmap(Tk_Display(tile->tkwin),tile->pixmap);
+ while (tile->handlers) {
+    Tk_TileChange *handler = tile->handlers;
+    tile->handlers = handler->next;
+    ckfree((char *) handler);
+ }
  ckfree((char *) tile);
 }
 
@@ -1149,18 +1186,18 @@ Tk_FreeTile(tile)
  *
  * Tk_NameOfTile
  *
- *	Returns the name of the image from which the tile was
- *	generated.
+ *      Returns the name of the image from which the tile was
+ *      generated.
  *
  * Results:
- *	The name of the image is returned.  The name is not unique.
- *	Many tiles may use the same image.
+ *      The name of the image is returned.  The name is not unique.
+ *      Many tiles may use the same image.
  *
  *----------------------------------------------------------------------
  */
 CONST char *
 Tk_NameOfTile(tile)
-    Tk_Tile tile;		/* Tile to query */
+    Tk_Tile tile;               /* Tile to query */
 {
  if (tile)
   {
@@ -1175,18 +1212,37 @@ Tk_NameOfTile(tile)
  *
  * Tk_PixmapOfTile
  *
- *	Returns the pixmap of the tile.
+ *      Returns the pixmap of the tile.
  *
  * Results:
- *	The X pixmap used as the tile is returned.
+ *      The X pixmap used as the tile is returned.
  *
  *----------------------------------------------------------------------
  */
 Pixmap
 Tk_PixmapOfTile(tile)
-    Tk_Tile tile;		/* Tile to query */
+    Tk_Tile tile;               /* Tile to query */
 {
-    return tile->pixmap;
+    if (tile) {
+	if (tile->image && tile->pixmap == None) {
+	    Tk_Window tkwin = tile->tkwin;
+	    int width  = 0;
+	    int height = 0;
+	    Tk_SizeOfImage(tile->image,&width,&height);
+	    if (width >= 0 && height >= 0) {
+		Tk_MakeWindowExist(tkwin);
+		tile->pixmap = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
+				width, height, Tk_Depth(tkwin));
+		if (tile->pixmap != None) {
+		    tile->width  = width;
+		    tile->height = height;
+		    Tk_RedrawImage(tile->image, 0, 0, width, height,  tile->pixmap, 0, 0);
+		}
+	    }
+	}
+	return tile->pixmap;
+    }
+    return None;
 }
 
 /*
@@ -1194,19 +1250,31 @@ Tk_PixmapOfTile(tile)
  *
  * Tk_SizeOfTile
  *
- *	Returns the width and height of the tile.
+ *      Returns the width and height of the tile.
  *
  * Results:
- *	The width and height of the tile are returned.
+ *      The width and height of the tile are returned.
  *
  *----------------------------------------------------------------------
  */
 void
 Tk_SizeOfTile(tile, widthPtr, heightPtr)
-    Tk_Tile tile;		/* Tile to query */
-    int *widthPtr, *heightPtr;	/* Returned dimensions of the tile (out) */
+    Tk_Tile tile;               /* Tile to query */
+    int *widthPtr, *heightPtr;  /* Returned dimensions of the tile (out) */
 {
-    Tk_SizeOfImage(tile->image,widthPtr,heightPtr);
+    if (tile) {
+	if (tile->image && tile->pixmap == None) {
+	    Tk_SizeOfImage(tile->image,widthPtr,heightPtr);
+	}
+	else {
+	    *widthPtr  = tile->height;
+	    *heightPtr = tile->width;
+	}
+    }
+    else {
+	*widthPtr  = 0;
+	*heightPtr = 0;
+    }
 }
 
 /*
@@ -1214,44 +1282,71 @@ Tk_SizeOfTile(tile, widthPtr, heightPtr)
  *
  * Tk_SetTileChangedProc
  *
- *	Sets the routine to called when an image changes.  If
- *	*changeProc* is NULL, no callback will be performed.
+ *      Sets the routine to called when an image changes.  If
+ *      *changeProc* is NULL, no callback will be performed.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side Effects:
- *	The designated routine will be called the next time the
- *	image associated with the tile changes.
+ *      The designated routine will be called the next time the
+ *      image associated with the tile changes.
  *
  *----------------------------------------------------------------------
  */
 void
-Tk_SetTileChangedProc(tile, changeProc, clientData, itemPtr)
-    Tk_Tile tile;		/* Tile to query */
+Tk_SetTileChangedProc(tile, changeProc, clientData)
+    Tk_Tile tile;               /* Tile to query */
     Tk_TileChangedProc *changeProc;
     ClientData clientData;
-    Tk_Item *itemPtr;
 {
+    if (tile) {
+	Tk_TileChange **link = &tile->handlers;
+	Tk_TileChange *handler;
+
+	while ((handler = *link)) {
+	    if (handler->clientData == clientData) {
+		break;
+	    }
+	    link = &handler->next;
+	}
+	if (changeProc == NULL ) {
+	    if (handler) {
+		*link = handler->next;
+		ckfree((char *) handler);
+	    }
+	}
+	else {
+	    if (handler == NULL) {
+		handler = (Tk_TileChange *) ckalloc(sizeof(Tk_TileChange));
+		memset(handler,0,sizeof(*handler));
+		handler->next = NULL;
+		*link = handler;
+		handler->clientData = clientData;
+	    }
+	    handler->changeProc = changeProc;
+	}
+    }
 }
 
+
 /*
  *----------------------------------------------------------------------
  *
  * Tk_SetTSOrigin --
  *
- *	Set the pattern origin of the tile to a common point (i.e. the
- *	origin (0,0) of the top level window) so that tiles from two
- *	different widgets will match up.  This done by setting the
- *	GCTileStipOrigin field is set to the translated origin of the
- *	toplevel window in the hierarchy.
+ *      Set the pattern origin of the tile to a common point (i.e. the
+ *      origin (0,0) of the top level window) so that tiles from two
+ *      different widgets will match up.  This done by setting the
+ *      GCTileStipOrigin field is set to the translated origin of the
+ *      toplevel window in the hierarchy.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side Effects:
- *	The GCTileStipOrigin is reset in the GC.  This will cause the
- *	tile origin to change when the GC is used for drawing.
+ *      The GCTileStipOrigin is reset in the GC.  This will cause the
+ *      tile origin to change when the GC is used for drawing.
  *
  *----------------------------------------------------------------------
  */
