@@ -29,7 +29,7 @@
  * |   provided "as is" without express or implied warranty.		|
  * +-------------------------------------------------------------------+
  *
- * RCS: @(#) $Id: tkImgGIF.c,v 1.22 2002/08/08 09:35:08 hobbs Exp $
+ * RCS: @(#) $Id: tkImgGIF.c,v 1.24 2003/02/20 15:28:40 dkf Exp $
  */
 
 #include "tkPort.h"
@@ -103,20 +103,20 @@ static Tcl_ThreadDataKey dataKey;
  * The format record for the GIF file format:
  */
 
-static int      FileMatchGIF _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Channel chan, char *fileName,
-		    Tcl_Obj *format, int *widthPtr, int *heightPtr));
+static int      FileMatchGIF _ANSI_ARGS_((Tcl_Channel chan, Tcl_Obj *fileName,
+		    Tcl_Obj *format, int *widthPtr, int *heightPtr, Tcl_Interp *interp));
 static int      FileReadGIF  _ANSI_ARGS_((Tcl_Interp *interp,
 		    Tcl_Channel chan, Tcl_Obj *fileName, Tcl_Obj *format,
 		    Tk_PhotoHandle imageHandle, int destX, int destY,
 		    int width, int height, int srcX, int srcY));
-static int	StringMatchGIF _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *dataObj,
-		    Tcl_Obj *format, int *widthPtr, int *heightPtr));
+static int	StringMatchGIF _ANSI_ARGS_((Tcl_Obj *dataObj,
+		    Tcl_Obj *format, int *widthPtr, int *heightPtr, Tcl_Interp *interp));
 static int	StringReadGIF _ANSI_ARGS_((Tcl_Interp *interp, Tcl_Obj *dataObj,
 		    Tcl_Obj *format, Tk_PhotoHandle imageHandle,
 		    int destX, int destY, int width, int height,
 		    int srcX, int srcY));
 static int 	FileWriteGIF _ANSI_ARGS_((Tcl_Interp *interp,
-		    CONST char *filename, Tcl_Obj *format,
+		    char *filename, Tcl_Obj *format,
 		    Tk_PhotoImageBlock *blockPtr));
 static int	CommonWriteGIF _ANSI_ARGS_((Tcl_Interp *interp,
 		    Tcl_Channel handle, Tcl_Obj *format,
@@ -199,15 +199,14 @@ static void		mInit _ANSI_ARGS_((unsigned char *string,
  */
 
 static int
-FileMatchGIF(interp, chan, fileName, format, widthPtr, heightPtr)
-    Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
+FileMatchGIF(chan, fileName, format, widthPtr, heightPtr, interp)
     Tcl_Channel chan;		/* The image file, open for reading. */
     Tcl_Obj *fileName;		/* The name of the image file. */
     Tcl_Obj *format;		/* User-specified format object, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * raw GIF file. */
-    Tcl_Interp *interp;		/* not used */
+    Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
 {
 	return ReadGIFHeader(chan, widthPtr, heightPtr);
 }
@@ -429,13 +428,6 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	    continue;
 	}
 
-	/*
-	 * If a trash buffer has been allocated, free it now.
-	 */
-	if (trashBuffer != NULL) {
-	    ckfree((char *)trashBuffer);
-	    trashBuffer = NULL;
-	}
 	if (BitSet(buf[8], LOCALCOLORMAP)) {
 	    if (!ReadColorMap(chan, bitPixel, colorMap)) {
 		    Tcl_AppendResult(interp, "error reading color map",
@@ -490,6 +482,12 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
 	    TK_PHOTO_COMPOSITE_SET);
 
     noerror:
+    /*
+     * If a trash buffer has been allocated, free it now.
+     */
+    if (trashBuffer != NULL) {
+	ckfree((char *)trashBuffer);
+    }
     if (block.pixelPtr) {
 	ckfree((char *) block.pixelPtr);
     }
@@ -497,11 +495,16 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
     return TCL_OK;
 
     error:
+    /*
+     * If a trash buffer has been allocated, free it now.
+     */
+    if (trashBuffer != NULL) {
+	ckfree((char *)trashBuffer);
+    }
     if (block.pixelPtr) {
 	ckfree((char *) block.pixelPtr);
     }
     return TCL_ERROR;
-
 }
 
 /*
@@ -523,7 +526,7 @@ FileReadGIF(interp, chan, fileName, format, imageHandle, destX, destY,
  */
 
 static int
-StringMatchGIF(inter, pdataObj, format, widthPtr, heightPtr)
+StringMatchGIF(dataObj, format, widthPtr, heightPtr, interp)
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
     Tcl_Obj *dataObj;		/* the object containing the image data */
     Tcl_Obj *format;		/* the image format object, or NULL */
@@ -605,6 +608,7 @@ StringReadGIF(interp, dataObj, format, imageHandle,
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
     Tcl_Channel dataSrc;
     char *data;
+    Tcl_Obj *name;
 
     /*
      * Check whether the data is Base64 encoded
@@ -619,8 +623,10 @@ StringReadGIF(interp, dataObj, format, imageHandle,
 	mInit((unsigned char *)data, &handle);
 	dataSrc = (Tcl_Channel) &handle;
     }
-    result = FileReadGIF(interp, dataSrc, "inline data",
+    name = Tcl_NewStringObj("inline data",0);
+    result = FileReadGIF(interp, dataSrc, name,
 	    format, imageHandle, destX, destY, width, height, srcX, srcY);
+    Tcl_DecrRefCount(name);
     tsdPtr->fromData = 0;
     return result;
 }
@@ -744,8 +750,6 @@ DoExtension(chan, label, transparent)
     return count;
 }
 
-static int ZeroDataBlock = 0;
-
 static int
 GetDataBlock(chan, buf)
     Tcl_Channel chan;
@@ -757,8 +761,6 @@ GetDataBlock(chan, buf)
 	return -1;
     }
 
-    ZeroDataBlock = count == 0;
-
     if ((count != 0) && (! ReadOK(chan, buf, count))) {
 	return -1;
     }
@@ -1396,7 +1398,7 @@ static int ReadValue _ANSI_ARGS_((void));
 static int
 FileWriteGIF(interp, filename, format, blockPtr)
     Tcl_Interp *interp;		/* Interpreter to use for reporting errors. */
-    CONST char	*filename;
+    char	*filename;
     Tcl_Obj	*format;
     Tk_PhotoImageBlock *blockPtr;
 {
