@@ -10,12 +10,16 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkPort.h 1.7 96/02/11 16:42:10
+ * RCS: @(#) $Id: tkPort.h,v 1.3 2002/08/31 06:12:26 das Exp $
  */
 
 #ifndef _TKPORT
 #define _TKPORT
 
+#if defined(__WIN32__) || defined(_WIN32)
+#define NEED_REAL_STDIO
+#endif
+
 #ifndef _Lang
 #include "Lang.h"
 #endif
@@ -25,9 +29,14 @@
 #   ifndef strcasecmp
 #       define strcasecmp(a,b) stricmp(a,b)
 #   endif
+#   ifdef __CYGWIN__
+#       undef strcasecmp
+#   endif
 #else
 #   if defined(MAC_TCL)
 #	include "tkMacPort.h"
+#   elif defined(MAC_OSX_TK)
+#	include "../macosx/tkMacOSXPort.h"
 #   else
 #       ifdef __PM__
 #           include "tkOS2Port.h"
