@@ -54,7 +54,7 @@ Tk_StateParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
     ClientData clientData;		/* some flags.*/
     Tcl_Interp *interp;			/* Used for reporting errors. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
-    Arg ovalue;				/* Value of option. */
+    Tcl_Obj * ovalue;				/* Value of option. */
     char *widgRec;			/* Pointer to record for item. */
     int offset;				/* Offset into item. */
 {
@@ -129,7 +129,7 @@ Tk_StateParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
  *--------------------------------------------------------------
  */
 
-Arg
+Tcl_Obj *
 Tk_StatePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;		/* Ignored. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
@@ -177,7 +177,7 @@ TkOrientParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
     ClientData clientData;		/* some flags.*/
     Tcl_Interp *interp;			/* Used for reporting errors. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
-    Arg ovalue;				/* Value of option. */
+    Tcl_Obj * ovalue;				/* Value of option. */
     char *widgRec;			/* Pointer to record for item. */
     int offset;				/* Offset into item. */
 {
@@ -232,7 +232,7 @@ TkOrientParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
  *--------------------------------------------------------------
  */
 
-Arg
+Tcl_Obj *
 TkOrientPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;		/* Ignored. */
     Tk_Window tkwin;			/* Window containing canvas widget. */
@@ -266,7 +266,7 @@ TkTileParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* Window on same display as tile */
-    Arg ovalue;			/* Name of image */
+    Tcl_Obj * ovalue;			/* Name of image */
     char *widgRec;		/* Widget structure record */
     int offset;			/* Offset of tile in record */
 {
@@ -303,7 +303,7 @@ TkTileParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
  *----------------------------------------------------------------------
  */
 
-Arg
+Tcl_Obj *
 TkTilePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
@@ -330,7 +330,7 @@ TkOffsetParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
     ClientData clientData;	/* not used */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* Window on same display as tile */
-    Arg ovalue;			/* Name of image */
+    Tcl_Obj * ovalue;			/* Name of image */
     char *widgRec;		/* Widget structure record */
     int offset;			/* Offset of tile in record */
 {
@@ -471,7 +471,7 @@ badTSOffset:
  *----------------------------------------------------------------------
  */
 
-Arg
+Tcl_Obj *
 TkOffsetPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
@@ -541,7 +541,7 @@ TkPixelParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
 				 * allowed as well */
     Tcl_Interp *interp;		/* Interpreter to send results back to */
     Tk_Window tkwin;		/* Window on same display as tile */
-    Arg ovalue;			/* Name of image */
+    Tcl_Obj * ovalue;			/* Name of image */
     char *widgRec;		/* Widget structure record */
     int offset;			/* Offset of tile in record */
 {
@@ -571,7 +571,7 @@ TkPixelParseProc(clientData, interp, tkwin, ovalue, widgRec, offset)
  *----------------------------------------------------------------------
  */
 
-Arg
+Tcl_Obj *
 TkPixelPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;	/* not used */
     Tk_Window tkwin;		/* not used */
