@@ -11,7 +11,7 @@
  *	of properly formed UTF-8 characters.  There is a one-to-one map between
  *	Unicode and UTF characters.  Because Unicode characters have a fixed
  *	width, operations such as indexing operate on Unicode data.  The String
- *	ojbect is opitmized for the case where each UTF char in a string is
+ *	object is optimized for the case where each UTF char in a string is
  *	only one byte.  In this case, we store the value of numChars, but we
  *	don't store the Unicode data (unless Tcl_GetUnicode is explicitly
  *	called).
@@ -33,7 +33,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclStringObj.c,v 1.24 2002/01/26 01:10:08 dgp Exp $ */
+ * RCS: @(#) $Id: tclStringObj.c,v 1.32 2003/02/19 16:43:28 das Exp $ */
 
 #include "tclInt.h"
 
@@ -94,15 +94,19 @@ typedef struct String {
 				 * for the UTF string (minus 1 byte for
 				 * the termination char). */
     size_t uallocated;		/* The amount of space actually allocated
-				 * for the Unicode string. 0 means the
-				 * Unicode string rep is invalid. */
+				 * for the Unicode string (minus 2 bytes for
+				 * the termination char). */
+    int hasUnicode;		/* Boolean determining whether the string
+				 * has a Unicode representation. */
     Tcl_UniChar unicode[2];	/* The array of Unicode chars.  The actual
 				 * size of this field depends on the
 				 * 'uallocated' field above. */
 } String;
 
-#define STRING_SIZE(len)	\
-		((unsigned) (sizeof(String) + ((len-1) * sizeof(Tcl_UniChar))))
+#define STRING_UALLOC(numChars)	\
+		(numChars * sizeof(Tcl_UniChar))
+#define STRING_SIZE(ualloc)	\
+		((unsigned) (sizeof(String) - sizeof(Tcl_UniChar) + ualloc))
 #define GET_STRING(objPtr) \
 		((String *) (objPtr)->internalRep.otherValuePtr)
 #define SET_STRING(objPtr, stringPtr) \
@@ -321,7 +325,7 @@ Tcl_NewUnicodeObj(unicode, numChars)
 	    while (unicode[numChars] != 0) { numChars++; }
 	}
     }
-    uallocated = (numChars + 1) * sizeof(Tcl_UniChar);
+    uallocated = STRING_UALLOC(numChars);
 
     /*
      * Create a new obj with an invalid string rep.
@@ -334,6 +338,7 @@ Tcl_NewUnicodeObj(unicode, numChars)
     stringPtr = (String *) ckalloc(STRING_SIZE(uallocated));
     stringPtr->numChars = numChars;
     stringPtr->uallocated = uallocated;
+    stringPtr->hasUnicode = (numChars > 0);
     stringPtr->allocated = 0;
     memcpy((VOID *) stringPtr->unicode, (VOID *) unicode, uallocated);
     stringPtr->unicode[numChars] = 0;
@@ -373,8 +378,22 @@ Tcl_GetCharLength(objPtr)
      */
 
     if (stringPtr->numChars == -1) {
+	register int i = objPtr->length;
+	register unsigned char *str = (unsigned char *) objPtr->bytes;
 
-	stringPtr->numChars = Tcl_NumUtfChars(objPtr->bytes, objPtr->length);
+	/*
+	 * This is a speed sensitive function, so run specially over the
+	 * string to count continuous ascii characters before resorting
+	 * to the Tcl_NumUtfChars call.  This is a long form of:
+	 stringPtr->numChars = Tcl_NumUtfChars(objPtr->bytes, objPtr->length);
+	*/
+
+	while (i && (*str < 0xC0)) { i--; str++; }
+	stringPtr->numChars = objPtr->length - i;
+	if (i) {
+	    stringPtr->numChars += Tcl_NumUtfChars(objPtr->bytes
+		    + (objPtr->length - i), i);
+	}
 
  	if (stringPtr->numChars == objPtr->length) {
 
@@ -384,7 +403,7 @@ Tcl_GetCharLength(objPtr)
 	     * unicode string.
 	     */
 
-	    stringPtr->uallocated = 0;
+	    stringPtr->hasUnicode = 0;
 
 	} else {
 
@@ -452,7 +471,7 @@ Tcl_GetUniChar(objPtr, index)
 
 	stringPtr = GET_STRING(objPtr);
     }
-    if (stringPtr->uallocated == 0) {
+    if (stringPtr->hasUnicode == 0) {
 
 	/*
 	 * All of the characters in the Utf string are 1 byte chars,
@@ -460,7 +479,7 @@ Tcl_GetUniChar(objPtr, index)
 	 * and convert the index'th byte to a Unicode character.
 	 */
 
-	Tcl_UtfToUniChar(&objPtr->bytes[index], &unichar);
+	unichar = (Tcl_UniChar) objPtr->bytes[index];
     } else {
 	unichar = stringPtr->unicode[index];
     }
@@ -495,7 +514,7 @@ Tcl_GetUnicode(objPtr)
     SetStringFromAny(NULL, objPtr);
     stringPtr = GET_STRING(objPtr);
 
-    if ((stringPtr->numChars == -1) || (stringPtr->uallocated == 0)) {
+    if ((stringPtr->numChars == -1) || (stringPtr->hasUnicode == 0)) {
 
 	/*
 	 * We haven't yet calculated the length, or all of the characters
@@ -548,7 +567,7 @@ Tcl_GetUnicodeFromObj(objPtr, lengthPtr)
     SetStringFromAny(NULL, objPtr);
     stringPtr = GET_STRING(objPtr);
 
-    if ((stringPtr->numChars == -1) || (stringPtr->uallocated == 0)) {
+    if ((stringPtr->numChars == -1) || (stringPtr->hasUnicode == 0)) {
 
 	/*
 	 * We haven't yet calculated the length, or all of the characters
@@ -739,7 +758,6 @@ Tcl_SetObjLength(objPtr, length)
 				 * representation of object, not including
 				 * terminating null byte. */
 {
-    char *new;
     String *stringPtr;
 
     if (Tcl_IsShared(objPtr)) {
@@ -747,38 +765,59 @@ Tcl_SetObjLength(objPtr, length)
     }
     SetStringFromAny(NULL, objPtr);
 
-    /*
-     * Invalidate the unicode data.
-     */
-
     stringPtr = GET_STRING(objPtr);
-    stringPtr->numChars = -1;
-    stringPtr->uallocated = 0;
 
-    if (length > (int) stringPtr->allocated) {
+    /* Check that we're not extending a pure unicode string */
+
+    if (length > (int) stringPtr->allocated &&
+	    (objPtr->bytes != NULL || stringPtr->hasUnicode == 0)) {
+	char *new;
 
 	/*
 	 * Not enough space in current string. Reallocate the string
 	 * space and free the old string.
 	 */
- 	if (objPtr->bytes != tclEmptyStringRep) {
+	if (objPtr->bytes != tclEmptyStringRep && objPtr->bytes != NULL) {
 	    new = (char *) ckrealloc((char *)objPtr->bytes,
 		    (unsigned)(length+1));
 	} else {
 	    new = (char *) ckalloc((unsigned) (length+1));
 	    if (objPtr->bytes != NULL && objPtr->length != 0) {
- 	    	memcpy((VOID *) new, (VOID *) objPtr->bytes,
- 		    	(size_t) objPtr->length);
- 	    	Tcl_InvalidateStringRep(objPtr);
+		memcpy((VOID *) new, (VOID *) objPtr->bytes,
+			(size_t) objPtr->length);
+		Tcl_InvalidateStringRep(objPtr);
 	    }
 	}
 	objPtr->bytes = new;
 	stringPtr->allocated = length;
+	/* Invalidate the unicode data. */
+	stringPtr->hasUnicode = 0;
     }
 
-    objPtr->length = length;
-    if ((objPtr->bytes != NULL) && (objPtr->bytes != tclEmptyStringRep)) {
-	objPtr->bytes[length] = 0;
+    if (objPtr->bytes != NULL) {
+        objPtr->length = length;
+        if (objPtr->bytes != tclEmptyStringRep) {
+            /* Ensure the string is NULL-terminated */
+            objPtr->bytes[length] = 0;
+        }
+        /* Invalidate the unicode data. */
+        stringPtr->numChars = -1;
+        stringPtr->hasUnicode = 0;
+    } else {
+        /* Changing length of pure unicode string */
+        size_t uallocated = STRING_UALLOC(length);
+        if (uallocated > stringPtr->uallocated) {
+            stringPtr = (String *) ckrealloc((char*) stringPtr,
+                    STRING_SIZE(uallocated));
+            SET_STRING(objPtr, stringPtr);
+            stringPtr->uallocated = uallocated;
+        }
+        stringPtr->numChars = length;
+        stringPtr->hasUnicode = (length > 0);
+        /* Ensure the string is NULL-terminated */
+        stringPtr->unicode[length] = 0;
+        stringPtr->allocated = 0;
+        objPtr->length = 0;
     }
 }
 
@@ -814,7 +853,6 @@ Tcl_AttemptSetObjLength(objPtr, length)
 				 * representation of object, not including
 				 * terminating null byte. */
 {
-    char *new;
     String *stringPtr;
 
     if (Tcl_IsShared(objPtr)) {
@@ -822,27 +860,25 @@ Tcl_AttemptSetObjLength(objPtr, length)
     }
     SetStringFromAny(NULL, objPtr);
 
-    /*
-     * Invalidate the unicode data.
-     */
-
     stringPtr = GET_STRING(objPtr);
-    stringPtr->numChars = -1;
-    stringPtr->uallocated = 0;
 
-    if (length > (int) stringPtr->allocated) {
+    /* Check that we're not extending a pure unicode string */
+
+    if (length > (int) stringPtr->allocated &&
+	    (objPtr->bytes != NULL || stringPtr->hasUnicode == 0)) {
+	char *new;
 
 	/*
 	 * Not enough space in current string. Reallocate the string
 	 * space and free the old string.
 	 */
- 	if (objPtr->bytes != tclEmptyStringRep) {
+	if (objPtr->bytes != tclEmptyStringRep && objPtr->bytes != NULL) {
 	    new = (char *) attemptckrealloc((char *)objPtr->bytes,
 		    (unsigned)(length+1));
 	    if (new == NULL) {
 		return 0;
 	    }
- 	} else {
+	} else {
 	    new = (char *) attemptckalloc((unsigned) (length+1));
 	    if (new == NULL) {
 		return 0;
@@ -855,11 +891,37 @@ Tcl_AttemptSetObjLength(objPtr, length)
 	}
 	objPtr->bytes = new;
 	stringPtr->allocated = length;
+	/* Invalidate the unicode data. */
+	stringPtr->hasUnicode = 0;
     }
 
-    objPtr->length = length;
-    if ((objPtr->bytes != NULL) && (objPtr->bytes != tclEmptyStringRep)) {
-	objPtr->bytes[length] = 0;
+    if (objPtr->bytes != NULL) {
+	objPtr->length = length;
+	if (objPtr->bytes != tclEmptyStringRep) {
+	    /* Ensure the string is NULL-terminated */
+	    objPtr->bytes[length] = 0;
+	}
+	/* Invalidate the unicode data. */
+	stringPtr->numChars = -1;
+	stringPtr->hasUnicode = 0;
+    } else {
+	/* Changing length of pure unicode string */
+	size_t uallocated = STRING_UALLOC(length);
+	if (uallocated > stringPtr->uallocated) {
+	    stringPtr = (String *) attemptckrealloc((char*) stringPtr,
+		    STRING_SIZE(uallocated));
+	    if (stringPtr == NULL) {
+	        return 0;
+	    }
+	    SET_STRING(objPtr, stringPtr);
+	    stringPtr->uallocated = uallocated;
+	}
+	stringPtr->numChars = length;
+	stringPtr->hasUnicode = (length > 0);
+	/* Ensure the string is NULL-terminated */
+	stringPtr->unicode[length] = 0;
+	stringPtr->allocated = 0;
+	objPtr->length = 0;
     }
     return 1;
 }
@@ -898,7 +960,7 @@ Tcl_SetUnicodeObj(objPtr, unicode, numChars)
 	    while (unicode[numChars] != 0) { numChars++; }
 	}
     }
-    uallocated = (numChars + 1) * sizeof(Tcl_UniChar);
+    uallocated = STRING_UALLOC(numChars);
 
     /*
      * Free the internal rep if one exists, and invalidate the string rep.
@@ -917,6 +979,7 @@ Tcl_SetUnicodeObj(objPtr, unicode, numChars)
     stringPtr = (String *) ckalloc(STRING_SIZE(uallocated));
     stringPtr->numChars = numChars;
     stringPtr->uallocated = uallocated;
+    stringPtr->hasUnicode = (numChars > 0);
     stringPtr->allocated = 0;
     memcpy((VOID *) stringPtr->unicode, (VOID *) unicode, uallocated);
     stringPtr->unicode[numChars] = 0;
@@ -973,7 +1036,7 @@ Tcl_AppendToObj(objPtr, bytes, length)
      */
 
     stringPtr = GET_STRING(objPtr);
-    if (stringPtr->uallocated > 0) {
+    if (stringPtr->hasUnicode != 0) {
 	AppendUtfToUnicodeRep(objPtr, bytes, length);
 
 	stringPtr = GET_STRING(objPtr);
@@ -1025,7 +1088,7 @@ Tcl_AppendUnicodeToObj(objPtr, unicode, length)
      * "unicode" to objPtr's string rep.
      */
 
-    if (stringPtr->uallocated > 0) {
+    if (stringPtr->hasUnicode != 0) {
 	AppendUnicodeToUnicodeRep(objPtr, unicode, length);
     } else {
 	AppendUnicodeToUtfRep(objPtr, unicode, length);
@@ -1067,7 +1130,7 @@ Tcl_AppendObjToObj(objPtr, appendObjPtr)
      */
 
     stringPtr = GET_STRING(objPtr);
-    if (stringPtr->uallocated > 0) {
+    if (stringPtr->hasUnicode != 0) {
 
 	/*
 	 * If appendObjPtr is not of the "String" type, don't convert it.
@@ -1076,10 +1139,10 @@ Tcl_AppendObjToObj(objPtr, appendObjPtr)
 	if (appendObjPtr->typePtr == &tclStringType) {
 	    stringPtr = GET_STRING(appendObjPtr);
 	    if ((stringPtr->numChars == -1)
-		    || (stringPtr->uallocated == 0)) {
+		    || (stringPtr->hasUnicode == 0)) {
 
 		/*
-		 * If appendObjPtr is a string obj with no valide Unicode
+		 * If appendObjPtr is a string obj with no valid Unicode
 		 * rep, then fill its unicode rep.
 		 */
 
@@ -1162,21 +1225,22 @@ AppendUnicodeToUnicodeRep(objPtr, unicode, appendNumChars)
 
     /*
      * If not enough space has been allocated for the unicode rep,
-     * reallocate the internal rep object with additional space.  First try to
-     * double the required allocation; if that fails, try a more modest
-     * increase.  See the "TCL STRING GROWTH ALGORITHM" comment at the top of
-     * this file for an explanation of this growth algorithm.
+     * reallocate the internal rep object with additional space.  First
+     * try to double the required allocation; if that fails, try a more
+     * modest increase.  See the "TCL STRING GROWTH ALGORITHM" comment at
+     * the top of this file for an explanation of this growth algorithm.
      */
 
     numChars = stringPtr->numChars + appendNumChars;
 
-    if (numChars >= stringPtr->uallocated) {
- 	stringPtr->uallocated = 2 * numChars;
+    if (STRING_UALLOC(numChars) >= stringPtr->uallocated) {
+     	stringPtr->uallocated = STRING_UALLOC(2 * numChars);
 	tmpString = (String *) attemptckrealloc((char *)stringPtr,
 		STRING_SIZE(stringPtr->uallocated));
 	if (tmpString == NULL) {
 	    stringPtr->uallocated =
-		numChars + appendNumChars + TCL_GROWTH_MIN_ALLOC;
+	        STRING_UALLOC(numChars + appendNumChars)
+		+ TCL_GROWTH_MIN_ALLOC;
 	    tmpString = (String *) ckrealloc((char *)stringPtr,
 		    STRING_SIZE(stringPtr->uallocated));
 	}
@@ -1337,16 +1401,15 @@ AppendUtfToUtfRep(objPtr, bytes, numBytes)
 	    Tcl_SetObjLength(objPtr,
 		    newLength + numBytes + TCL_GROWTH_MIN_ALLOC);
 	}
-    } else {
-
-	/*
-	 * Invalidate the unicode data.
-	 */
-
-	stringPtr->numChars = -1;
-	stringPtr->uallocated = 0;
     }
 
+    /*
+     * Invalidate the unicode data.
+     */
+
+    stringPtr->numChars = -1;
+    stringPtr->hasUnicode = 0;
+
     memcpy((VOID *) (objPtr->bytes + oldLength), (VOID *) bytes,
 	    (size_t) numBytes);
     objPtr->bytes[newLength] = 0;
@@ -1557,8 +1620,9 @@ FillUnicodeRep(objPtr)
     if (stringPtr->numChars == -1) {
 	stringPtr->numChars = Tcl_NumUtfChars(src, objPtr->length);
     }
+    stringPtr->hasUnicode = (stringPtr->numChars > 0);
 
-    uallocated = stringPtr->numChars * sizeof(Tcl_UniChar);
+    uallocated = STRING_UALLOC(stringPtr->numChars);
     if (uallocated > stringPtr->uallocated) {
 
 	/*
@@ -1588,7 +1652,7 @@ FillUnicodeRep(objPtr)
 
     srcEnd = src + objPtr->length;
     for (dst = stringPtr->unicode; src < srcEnd; dst++) {
-	src += Tcl_UtfToUniChar(src, dst);
+	src += TclUtfToUniChar(src, dst);
     }
     *dst = 0;
 
@@ -1630,9 +1694,9 @@ DupStringInternalRep(srcPtr, copyPtr)
      * internal rep, and invalidate the string rep of the new object.
      */
 
-    if (srcStringPtr->uallocated == 0) {
-    	copyStringPtr = (String *) ckalloc(sizeof(String));
-	copyStringPtr->uallocated = 0;
+    if (srcStringPtr->hasUnicode == 0) {
+    	copyStringPtr = (String *) ckalloc(STRING_SIZE(STRING_UALLOC(0)));
+	copyStringPtr->uallocated = STRING_UALLOC(0);
     } else {
 	copyStringPtr = (String *) ckalloc(
 	    STRING_SIZE(srcStringPtr->uallocated));
@@ -1644,6 +1708,7 @@ DupStringInternalRep(srcPtr, copyPtr)
 	copyStringPtr->unicode[srcStringPtr->numChars] = 0;
     }
     copyStringPtr->numChars = srcStringPtr->numChars;
+    copyStringPtr->hasUnicode = srcStringPtr->hasUnicode;
     copyStringPtr->allocated = srcStringPtr->allocated;
 
     /*
@@ -1681,7 +1746,7 @@ SetStringFromAny(interp, objPtr)
     register Tcl_Obj *objPtr;	/* The object to convert. */
 {
     /*
-     * The Unicode object is opitmized for the case where each UTF char
+     * The Unicode object is optimized for the case where each UTF char
      * in a string is only one byte.  In this case, we store the value of
      * numChars, but we don't copy the bytes to the unicodeObj->unicode.
      */
@@ -1703,9 +1768,10 @@ SetStringFromAny(interp, objPtr)
 	 * Allocate enough space for the basic String structure.
 	 */
 
-	stringPtr = (String *) ckalloc(sizeof(String));
+	stringPtr = (String *) ckalloc(STRING_SIZE(STRING_UALLOC(0)));
 	stringPtr->numChars = -1;
-	stringPtr->uallocated = 0;
+	stringPtr->uallocated = STRING_UALLOC(0);
+	stringPtr->hasUnicode = 0;
 
 	if (objPtr->bytes != NULL) {
 	    stringPtr->allocated = objPtr->length;
@@ -1740,7 +1806,7 @@ static void
 UpdateStringOfString(objPtr)
     Tcl_Obj *objPtr;		/* Object with string rep to update. */
 {
-    int i, length, size;
+    int i, size;
     Tcl_UniChar *unicode;
     char dummy[TCL_UTF_MAX];
     char *dst;
@@ -1762,7 +1828,6 @@ UpdateStringOfString(objPtr)
 	}
 
 	unicode = stringPtr->unicode;
-	length = stringPtr->numChars * sizeof(Tcl_UniChar);
 
 	/*
 	 * Translate the Unicode string to UTF.  "size" will hold the
