@@ -1,17 +1,17 @@
 /*
  * tclUnixNotify.c --
  *
- *	This file contains the implementation of the select-based
- *	Unix-specific notifier, which is the lowest-level part of the
- *	Tcl event loop.  This file works together with
- *	../generic/tclNotify.c.
+ *      This file contains the implementation of the select-based
+ *      Unix-specific notifier, which is the lowest-level part of the
+ *      Tcl event loop.  This file works together with
+ *      ../generic/tclNotify.c.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixNotfy.c,v 1.11 2002/08/31 06:09:46 das Exp $
+ * RCS: @(#) $Id: tclUnixNotfy.c,v 1.11.2.2 2003/07/16 22:09:48 hobbs Exp $
  */
 
 #if 0
@@ -22,6 +22,9 @@
 #include "Lang.h"
 #endif
 #include <signal.h>
+#ifdef _LANG
+#include <errno.h>
+#endif
 
 #ifdef TCL_EVENT_IMPLEMENT
 
@@ -30,6 +33,7 @@ extern TclStubs tclStubs;
 #else
 #include "tkEvent_f.h"
 #endif
+extern Tcl_NotifierProcs tclOriginalNotifier;
 
 /*
  * This structure is used to keep track of the notifier info for a
@@ -38,14 +42,14 @@ extern TclStubs tclStubs;
 
 typedef struct FileHandler {
     int fd;
-    int mask;			/* Mask of desired events: TCL_READABLE,
+    int mask;                   /* Mask of desired events: TCL_READABLE,
 				 * etc. */
-    int readyMask;		/* Mask of events that have been seen since the
+    int readyMask;              /* Mask of events that have been seen since the
 				 * last time file handlers were invoked for
 				 * this file. */
-    Tcl_FileProc *proc;		/* Procedure to call, in the style of
+    Tcl_FileProc *proc;         /* Procedure to call, in the style of
 				 * Tcl_CreateFileHandler. */
-    ClientData clientData;	/* Argument to pass to proc. */
+    ClientData clientData;      /* Argument to pass to proc. */
     struct FileHandler *nextPtr;/* Next in list of all files we care about. */
 } FileHandler;
 
@@ -55,9 +59,9 @@ typedef struct FileHandler {
  */
 
 typedef struct FileHandlerEvent {
-    Tcl_Event header;		/* Information that is standard for
+    Tcl_Event header;           /* Information that is standard for
 				 * all events. */
-    int fd;			/* File descriptor that is ready.  Used
+    int fd;                     /* File descriptor that is ready.  Used
 				 * to find the FileHandler structure for
 				 * the file (can't point directly to the
 				 * FileHandler structure because it could
@@ -82,27 +86,27 @@ typedef struct ThreadSpecificData {
 				/* This array reflects the readable/writable
 				 * conditions that were found to exist by the
 				 * last call to select. */
-    int numFdBits;		/* Number of valid bits in checkMasks
+    int numFdBits;              /* Number of valid bits in checkMasks
 				 * (one more than highest fd for which
 				 * Tcl_WatchFile has been called). */
 #ifdef TCL_THREADS
-    int onList;			/* True if it is in this list */
-    unsigned int pollState;	/* pollState is used to implement a polling
+    int onList;                 /* True if it is in this list */
+    unsigned int pollState;     /* pollState is used to implement a polling
 				 * handshake between each thread and the
 				 * notifier thread. Bits defined below. */
     struct ThreadSpecificData *nextPtr, *prevPtr;
-                                /* All threads that are currently waiting on
-                                 * an event have their ThreadSpecificData
-                                 * structure on a doubly-linked listed formed
-                                 * from these pointers.  You must hold the
-                                 * notifierMutex lock before accessing these
-                                 * fields. */
+				/* All threads that are currently waiting on
+				 * an event have their ThreadSpecificData
+				 * structure on a doubly-linked listed formed
+				 * from these pointers.  You must hold the
+				 * notifierMutex lock before accessing these
+				 * fields. */
     Tcl_Condition waitCV;     /* Any other thread alerts a notifier
 				 * that an event is ready to be processed
 				 * by signaling this condition variable. */
     int eventReady;           /* True if an event is ready to be processed.
-                               * Used as condition flag together with
-                               * waitCV above. */
+			       * Used as condition flag together with
+			       * waitCV above. */
 #endif
 } ThreadSpecificData;
 
@@ -161,15 +165,15 @@ static Tcl_Condition notifierCV;
 
 /*
  * The pollState bits
- *	POLL_WANT is set by each thread before it waits on its condition
- *		variable.  It is checked by the notifier before it does
- *		select.
- *	POLL_DONE is set by the notifier if it goes into select after
- *		seeing POLL_WANT.  The idea is to ensure it tries a select
- *		with the same bits the initial thread had set.
+ *      POLL_WANT is set by each thread before it waits on its condition
+ *              variable.  It is checked by the notifier before it does
+ *              select.
+ *      POLL_DONE is set by the notifier if it goes into select after
+ *              seeing POLL_WANT.  The idea is to ensure it tries a select
+ *              with the same bits the initial thread had set.
  */
-#define POLL_WANT	0x1
-#define POLL_DONE	0x2
+#define POLL_WANT       0x1
+#define POLL_DONE       0x2
 
 /*
  * This is the thread ID of the notifier thread that does select.
@@ -183,9 +187,9 @@ static Tcl_ThreadId notifierThread;
  */
 
 #ifdef TCL_THREADS
-static void	NotifierThreadProc _ANSI_ARGS_((ClientData clientData));
+static void     NotifierThreadProc _ANSI_ARGS_((ClientData clientData));
 #endif
-static int	FileHandlerEventProc _ANSI_ARGS_((Tcl_Event *evPtr,
+static int      FileHandlerEventProc _ANSI_ARGS_((Tcl_Event *evPtr,
 		    int flags));
 
 /*
@@ -193,13 +197,13 @@ static int	FileHandlerEventProc _ANSI_ARGS_((Tcl_Event *evPtr,
  *
  * Tcl_InitNotifier --
  *
- *	Initializes the platform specific notifier state.
+ *      Initializes the platform specific notifier state.
  *
  * Results:
- *	Returns a handle to the notifier state for this thread..
+ *      Returns a handle to the notifier state for this thread..
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -243,22 +247,22 @@ Tcl_InitNotifier()
  *
  * Tcl_FinalizeNotifier --
  *
- *	This function is called to cleanup the notifier state before
- *	a thread is terminated.
+ *      This function is called to cleanup the notifier state before
+ *      a thread is terminated.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	May terminate the background notifier thread if this is the
- *	last notifier instance.
+ *      May terminate the background notifier thread if this is the
+ *      last notifier instance.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_FinalizeNotifier(clientData)
-    ClientData clientData;		/* Not used. */
+    ClientData clientData;              /* Not used. */
 {
 #ifdef TCL_THREADS
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
@@ -276,7 +280,7 @@ Tcl_FinalizeNotifier(clientData)
 	    panic("Tcl_FinalizeNotifier: notifier pipe not initialized");
 	}
 
-        /*
+	/*
 	 * Send "q" message to the notifier thread so that it will
 	 * terminate.  The notifier will return from its call to select()
 	 * and notice that a "q" message has arrived, it will then close
@@ -307,18 +311,18 @@ Tcl_FinalizeNotifier(clientData)
  *
  * Tcl_AlertNotifier --
  *
- *	Wake up the specified notifier from any thread. This routine
- *	is called by the platform independent notifier code whenever
- *	the Tcl_ThreadAlert routine is called.  This routine is
- *	guaranteed not to be called on a given notifier after
- *	Tcl_FinalizeNotifier is called for that notifier.
+ *      Wake up the specified notifier from any thread. This routine
+ *      is called by the platform independent notifier code whenever
+ *      the Tcl_ThreadAlert routine is called.  This routine is
+ *      guaranteed not to be called on a given notifier after
+ *      Tcl_FinalizeNotifier is called for that notifier.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Signals the notifier condition variable for the specified
- *	notifier.
+ *      Signals the notifier condition variable for the specified
+ *      notifier.
  *
  *----------------------------------------------------------------------
  */
@@ -341,22 +345,22 @@ Tcl_AlertNotifier(clientData)
  *
  * Tcl_SetTimer --
  *
- *	This procedure sets the current notifier timer value.  This
- *	interface is not implemented in this notifier because we are
- *	always running inside of Tcl_DoOneEvent.
+ *      This procedure sets the current notifier timer value.  This
+ *      interface is not implemented in this notifier because we are
+ *      always running inside of Tcl_DoOneEvent.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_SetTimer(timePtr)
-    Tcl_Time *timePtr;		/* Timeout value, may be NULL. */
+    Tcl_Time *timePtr;          /* Timeout value, may be NULL. */
 {
     /*
      * The interval timer doesn't do anything in this implementation,
@@ -364,7 +368,7 @@ Tcl_SetTimer(timePtr)
      * timeout values to Tcl_WaitForEvent.
      */
 
-    if (tclStubs.tcl_SetTimer != Tcl_SetTimer) {
+    if (tclStubs.tcl_SetTimer != tclOriginalNotifier.setTimerProc) {
 	tclStubs.tcl_SetTimer(timePtr);
     }
 }
@@ -374,20 +378,20 @@ Tcl_SetTimer(timePtr)
  *
  * Tcl_ServiceModeHook --
  *
- *	This function is invoked whenever the service mode changes.
+ *      This function is invoked whenever the service mode changes.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_ServiceModeHook(mode)
-    int mode;			/* Either TCL_SERVICE_ALL, or
+    int mode;                   /* Either TCL_SERVICE_ALL, or
 				 * TCL_SERVICE_NONE. */
 {
 }
@@ -397,33 +401,33 @@ Tcl_ServiceModeHook(mode)
  *
  * Tcl_CreateFileHandler --
  *
- *	This procedure registers a file handler with the select notifier.
+ *      This procedure registers a file handler with the select notifier.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Creates a new file handler structure.
+ *      Creates a new file handler structure.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_CreateFileHandler(fd, mask, proc, clientData)
-    int fd;			/* Handle of stream to watch. */
-    int mask;			/* OR'ed combination of TCL_READABLE,
+    int fd;                     /* Handle of stream to watch. */
+    int mask;                   /* OR'ed combination of TCL_READABLE,
 				 * TCL_WRITABLE, and TCL_EXCEPTION:
 				 * indicates conditions under which
 				 * proc should be called. */
-    Tcl_FileProc *proc;		/* Procedure to call for each
+    Tcl_FileProc *proc;         /* Procedure to call for each
 				 * selected event. */
-    ClientData clientData;	/* Arbitrary data to pass to proc. */
+    ClientData clientData;      /* Arbitrary data to pass to proc. */
 {
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
     FileHandler *filePtr;
     int index, bit;
 
-    if (tclStubs.tcl_CreateFileHandler != Tcl_CreateFileHandler) {
+    if (tclStubs.tcl_CreateFileHandler != tclOriginalNotifier.createFileHandlerProc) {
 	tclStubs.tcl_CreateFileHandler(fd, mask, proc, clientData);
 	return;
     }
@@ -476,28 +480,28 @@ Tcl_CreateFileHandler(fd, mask, proc, clientData)
  *
  * Tcl_DeleteFileHandler --
  *
- *	Cancel a previously-arranged callback arrangement for
- *	a file.
+ *      Cancel a previously-arranged callback arrangement for
+ *      a file.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	If a callback was previously registered on file, remove it.
+ *      If a callback was previously registered on file, remove it.
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_DeleteFileHandler(fd)
-    int fd;		/* Stream id for which to remove callback procedure. */
+    int fd;             /* Stream id for which to remove callback procedure. */
 {
     FileHandler *filePtr, *prevPtr;
     int index, bit, i;
     unsigned long flags;
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
-    if (tclStubs.tcl_DeleteFileHandler != Tcl_DeleteFileHandler) {
+    if (tclStubs.tcl_DeleteFileHandler != tclOriginalNotifier.deleteFileHandlerProc) {
 	tclStubs.tcl_DeleteFileHandler(fd);
 	return;
     }
@@ -576,27 +580,27 @@ int (*Tcl_WaitForEventProc)(int, fd_mask *, Tcl_Time *) = NULL;
  *
  * FileHandlerEventProc --
  *
- *	This procedure is called by Tcl_ServiceEvent when a file event
- *	reaches the front of the event queue.  This procedure is
- *	responsible for actually handling the event by invoking the
- *	callback for the file handler.
+ *      This procedure is called by Tcl_ServiceEvent when a file event
+ *      reaches the front of the event queue.  This procedure is
+ *      responsible for actually handling the event by invoking the
+ *      callback for the file handler.
  *
  * Results:
- *	Returns 1 if the event was handled, meaning it should be removed
- *	from the queue.  Returns 0 if the event was not handled, meaning
- *	it should stay on the queue.  The only time the event isn't
- *	handled is if the TCL_FILE_EVENTS flag bit isn't set.
+ *      Returns 1 if the event was handled, meaning it should be removed
+ *      from the queue.  Returns 0 if the event was not handled, meaning
+ *      it should stay on the queue.  The only time the event isn't
+ *      handled is if the TCL_FILE_EVENTS flag bit isn't set.
  *
  * Side effects:
- *	Whatever the file handler's callback procedure does.
+ *      Whatever the file handler's callback procedure does.
  *
  *----------------------------------------------------------------------
  */
 
 static int
 FileHandlerEventProc(evPtr, flags)
-    Tcl_Event *evPtr;		/* Event to service. */
-    int flags;			/* Flags that indicate what events to
+    Tcl_Event *evPtr;           /* Event to service. */
+    int flags;                  /* Flags that indicate what events to
 				 * handle, such as TCL_FILE_EVENTS. */
 {
     int mask;
@@ -649,23 +653,23 @@ FileHandlerEventProc(evPtr, flags)
  *
  * Tcl_WaitForEvent --
  *
- *	This function is called by Tcl_DoOneEvent to wait for new
- *	events on the message queue.  If the block time is 0, then
- *	Tcl_WaitForEvent just polls without blocking.
+ *      This function is called by Tcl_DoOneEvent to wait for new
+ *      events on the message queue.  If the block time is 0, then
+ *      Tcl_WaitForEvent just polls without blocking.
  *
  * Results:
- *	Returns -1 if the select would block forever, otherwise
- *	returns 0.
+ *      Returns -1 if the select would block forever, otherwise
+ *      returns 0.
  *
  * Side effects:
- *	Queues file events that are detected by the select.
+ *      Queues file events that are detected by the select.
  *
  *----------------------------------------------------------------------
  */
 
 int
 Tcl_WaitForEvent(timePtr)
-    Tcl_Time *timePtr;		/* Maximum block time, or NULL. */
+    Tcl_Time *timePtr;          /* Maximum block time, or NULL. */
 {
     FileHandler *filePtr;
     FileHandlerEvent *fileEvPtr;
@@ -678,7 +682,7 @@ Tcl_WaitForEvent(timePtr)
 #endif
     ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
 
-    if (tclStubs.tcl_WaitForEvent != Tcl_WaitForEvent) {
+    if (tclStubs.tcl_WaitForEvent != tclOriginalNotifier.waitForEventProc) {
 	return tclStubs.tcl_WaitForEvent(timePtr);
     }
 
@@ -734,19 +738,19 @@ Tcl_WaitForEvent(timePtr)
     }
 
     if (waitForFiles) {
-        /*
-         * Add the ThreadSpecificData structure of this thread to the list
-         * of ThreadSpecificData structures of all threads that are waiting
-         * on file events.
-         */
+	/*
+	 * Add the ThreadSpecificData structure of this thread to the list
+	 * of ThreadSpecificData structures of all threads that are waiting
+	 * on file events.
+	 */
 
 
-        tsdPtr->nextPtr = waitingListPtr;
-        if (waitingListPtr) {
-            waitingListPtr->prevPtr = tsdPtr;
-        }
-        tsdPtr->prevPtr = 0;
-        waitingListPtr = tsdPtr;
+	tsdPtr->nextPtr = waitingListPtr;
+	if (waitingListPtr) {
+	    waitingListPtr->prevPtr = tsdPtr;
+	}
+	tsdPtr->prevPtr = 0;
+	waitingListPtr = tsdPtr;
 	tsdPtr->onList = 1;
 
 	write(triggerPipe, "", 1);
@@ -755,7 +759,7 @@ Tcl_WaitForEvent(timePtr)
     memset((VOID *) tsdPtr->readyMasks, 0, 3*MASK_SIZE*sizeof(fd_mask));
 
     if (!tsdPtr->eventReady) {
-        Tcl_ConditionWait(&tsdPtr->waitCV, &notifierMutex, timePtr);
+	Tcl_ConditionWait(&tsdPtr->waitCV, &notifierMutex, timePtr);
     }
     tsdPtr->eventReady = 0;
 
@@ -767,15 +771,15 @@ Tcl_WaitForEvent(timePtr)
 	 * which the notifier thread was still doing a select on.
 	 */
 
-        if (tsdPtr->prevPtr) {
-            tsdPtr->prevPtr->nextPtr = tsdPtr->nextPtr;
-        } else {
-            waitingListPtr = tsdPtr->nextPtr;
-        }
-        if (tsdPtr->nextPtr) {
-            tsdPtr->nextPtr->prevPtr = tsdPtr->prevPtr;
-        }
-        tsdPtr->nextPtr = tsdPtr->prevPtr = NULL;
+	if (tsdPtr->prevPtr) {
+	    tsdPtr->prevPtr->nextPtr = tsdPtr->nextPtr;
+	} else {
+	    waitingListPtr = tsdPtr->nextPtr;
+	}
+	if (tsdPtr->nextPtr) {
+	    tsdPtr->nextPtr->prevPtr = tsdPtr->prevPtr;
+	}
+	tsdPtr->nextPtr = tsdPtr->prevPtr = NULL;
 	tsdPtr->onList = 0;
 	write(triggerPipe, "", 1);
     }
@@ -797,6 +801,16 @@ Tcl_WaitForEvent(timePtr)
     if (numFound == -1) {
 	memset((VOID *) tsdPtr->readyMasks, 0, 3*MASK_SIZE*sizeof(fd_mask));
     }
+
+#ifdef _LANG
+    /*
+     * Language-specific check for signals
+     */
+    if (numFound == -1 && errno == EINTR) {
+	LangAsyncCheck();
+    }
+#endif
+
 #endif
 
     /*
@@ -849,35 +863,36 @@ Tcl_WaitForEvent(timePtr)
  *
  * NotifierThreadProc --
  *
- *	This routine is the initial (and only) function executed by the
- *	special notifier thread.  Its job is to wait for file descriptors
- *	to become readable or writable or to have an exception condition
- *	and then to notify other threads who are interested in this
- *	information by signalling a condition variable.  Other threads
- *	can signal this notifier thread of a change in their interests
- *	by writing a single byte to a special pipe that the notifier
- *	thread is monitoring.
+ *      This routine is the initial (and only) function executed by the
+ *      special notifier thread.  Its job is to wait for file descriptors
+ *      to become readable or writable or to have an exception condition
+ *      and then to notify other threads who are interested in this
+ *      information by signalling a condition variable.  Other threads
+ *      can signal this notifier thread of a change in their interests
+ *      by writing a single byte to a special pipe that the notifier
+ *      thread is monitoring.
  *
  * Result:
- *	None.  Once started, this routine never exits.  It dies with
- *	the overall process.
+ *      None.  Once started, this routine never exits.  It dies with
+ *      the overall process.
  *
  * Side effects:
- *	The trigger pipe used to signal the notifier thread is created
- *	when the notifier thread first starts.
+ *      The trigger pipe used to signal the notifier thread is created
+ *      when the notifier thread first starts.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 NotifierThreadProc(clientData)
-    ClientData clientData;	/* Not used. */
+    ClientData clientData;      /* Not used. */
 {
     ThreadSpecificData *tsdPtr;
     fd_mask masks[3*MASK_SIZE];
-    long *maskPtr = (long *)masks;	/* masks[] cast to type long[] */
+    long *maskPtr = (long *)masks;      /* masks[] cast to type long[] */
     int fds[2];
-    int i, status, index, bit, numFdBits, found, receivePipe, word;
+    int i, status, index, bit, numFdBits, receivePipe;
+    long found, word;
     struct timeval poll = {0., 0.}, *timePtr;
     int maskSize = 3 * ((MASK_SIZE) / sizeof(long)) * sizeof(fd_mask);
     char buf[2];
@@ -932,7 +947,7 @@ NotifierThreadProc(clientData)
 	 */
 
 	memset((VOID *)masks, 0, 3*MASK_SIZE*sizeof(fd_mask));
-        numFdBits = receivePipe + 1;
+	numFdBits = receivePipe + 1;
 	index = receivePipe / (NBBY*sizeof(fd_mask));
 	bit = 1 << (receivePipe % (NBBY*sizeof(fd_mask)));
 	masks[index] |= bit;
@@ -943,13 +958,13 @@ NotifierThreadProc(clientData)
 
 	Tcl_MutexLock(&notifierMutex);
 	timePtr = NULL;
-        for (tsdPtr = waitingListPtr; tsdPtr; tsdPtr = tsdPtr->nextPtr) {
-            for (i = 0; i < maskSize; i++) {
-                maskPtr[i] |= ((long*)tsdPtr->checkMasks)[i];
-            }
-            if (tsdPtr->numFdBits > numFdBits) {
-                numFdBits = tsdPtr->numFdBits;
-            }
+	for (tsdPtr = waitingListPtr; tsdPtr; tsdPtr = tsdPtr->nextPtr) {
+	    for (i = 0; i < maskSize; i++) {
+		maskPtr[i] |= ((long*)tsdPtr->checkMasks)[i];
+	    }
+	    if (tsdPtr->numFdBits > numFdBits) {
+		numFdBits = tsdPtr->numFdBits;
+	    }
 	    if (tsdPtr->pollState & POLL_WANT) {
 		/*
 		 * Here we make sure we go through select() with the same
@@ -959,7 +974,7 @@ NotifierThreadProc(clientData)
 		tsdPtr->pollState |= POLL_DONE;
 		timePtr = &poll;
 	    }
-        }
+	}
 	Tcl_MutexUnlock(&notifierMutex);
 
 	maskSize = 3 * ((MASK_SIZE) / sizeof(long)) * sizeof(fd_mask);
@@ -972,23 +987,23 @@ NotifierThreadProc(clientData)
 	     */
 
 	    continue;
-        }
+	}
 
 	/*
 	 * Alert any threads that are waiting on a ready file descriptor.
 	 */
 
 	Tcl_MutexLock(&notifierMutex);
-        for (tsdPtr = waitingListPtr; tsdPtr; tsdPtr = tsdPtr->nextPtr) {
+	for (tsdPtr = waitingListPtr; tsdPtr; tsdPtr = tsdPtr->nextPtr) {
 	    found = 0;
 
-            for (i = 0; i < maskSize; i++) {
-                word = maskPtr[i] & ((long*)tsdPtr->checkMasks)[i];
-                found |= word;
-                (((long*)(tsdPtr->readyMasks))[i]) = word;
+	    for (i = 0; i < maskSize; i++) {
+		word = maskPtr[i] & ((long*)tsdPtr->checkMasks)[i];
+		found |= word;
+		(((long*)(tsdPtr->readyMasks))[i]) = word;
 	    }
-            if (found || (tsdPtr->pollState & POLL_DONE)) {
-                tsdPtr->eventReady = 1;
+	    if (found || (tsdPtr->pollState & POLL_DONE)) {
+		tsdPtr->eventReady = 1;
 		if (tsdPtr->onList) {
 		    /*
 		     * Remove the ThreadSpecificData structure of this
@@ -1010,8 +1025,8 @@ NotifierThreadProc(clientData)
 		    tsdPtr->pollState = 0;
 		}
 		Tcl_ConditionNotify(&tsdPtr->waitCV);
-            }
-        }
+	    }
+	}
 	Tcl_MutexUnlock(&notifierMutex);
 
 	/*
