@@ -4,10 +4,10 @@
 #include "tkVMacro.h"
 
 static int   DItemParseProc _ANSI_ARGS_((ClientData clientData,
-		Tcl_Interp *interp, Tk_Window tkwin, Arg value,
+		Tcl_Interp *interp, Tk_Window tkwin, Tcl_Obj * value,
 		char *widRec, int offset));
 
-static Arg   DItemPrintProc _ANSI_ARGS_((
+static Tcl_Obj *   DItemPrintProc _ANSI_ARGS_((
 		ClientData clientData, Tk_Window tkwin, char *widRec,
 		int offset, Tcl_FreeProc **freeProcPtr));
 
@@ -16,14 +16,14 @@ DItemParseProc(clientData, interp, tkwin, value, widRec,offset)
     ClientData clientData;
     Tcl_Interp *interp;
     Tk_Window tkwin;
-    Arg value;
+    Tcl_Obj * value;
     char *widRec;
     int offset;
 {
  return TixDItemParseProc(clientData, interp, tkwin, value, widRec,offset);
 }
 
-static Arg
+static Tcl_Obj *
 DItemPrintProc(clientData, tkwin, widRec,offset, freeProcPtr)
     ClientData clientData;
     Tk_Window tkwin;
