@@ -47,8 +47,8 @@ typedef struct StyleInfo {
 
 FORWARD int   		DItemStyleParseProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, Tk_Window tkwin,
-			    Arg value,char *widRec, int offset));
-FORWARD Arg 		DItemStylePrintProc _ANSI_ARGS_((
+			    Tcl_Obj * value,char *widRec, int offset));
+FORWARD Tcl_Obj * 		DItemStylePrintProc _ANSI_ARGS_((
 			    ClientData clientData, Tk_Window tkwin,
 			    char *widRec, int offset,
 			    Tcl_FreeProc **freeProcPtr));
@@ -251,7 +251,7 @@ Tix_ItemStyleCmd(clientData, interp, argc, argv)
 #else
 		Tcl_SetStringObj(objv[n],Tcl_GetString(objv[i]),-1);
 		Tcl_SetStringObj(objv[n+1],Tcl_GetString(objv[i+1]),-1);
-		return TCL_ERROR;
+		/* XXX ??? return TCL_ERROR; */
 #endif
 	    }
 	    n+=2;
@@ -934,7 +934,7 @@ LINKAGE int DItemStyleParseProc(clientData, interp, tkwin, value, widRec,offset)
     ClientData clientData;
     Tcl_Interp *interp;
     Tk_Window tkwin;
-    Arg value;
+    Tcl_Obj * value;
     char *widRec;		/* Must point to a valid Tix_DItem struct */
     int offset;
 {
@@ -990,7 +990,7 @@ not_found:
     return TCL_ERROR;
 }
 
-LINKAGE Arg
+LINKAGE Tcl_Obj *
 DItemStylePrintProc(clientData, tkwin, widRec,offset, freeProcPtr)
     ClientData clientData;
     Tk_Window tkwin;
@@ -999,7 +999,7 @@ DItemStylePrintProc(clientData, tkwin, widRec,offset, freeProcPtr)
     Tcl_FreeProc **freeProcPtr;
 {
     Tix_DItemStyle *stylePtr = *((Tix_DItemStyle**)(widRec+offset));
-    Arg result = NULL;
+    Tcl_Obj * result = NULL;
     if (stylePtr != NULL && !(stylePtr->base.flags & TIX_STYLE_DEFAULT)) {
         LangSetObj(&result,LangObjectObj(stylePtr->base.interp,
                                          stylePtr->base.name));
