@@ -40,6 +40,14 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_PIXELS, "-borderwidth", "borderWidth", "BorderWidth",
        DEF_GRID_BORDER_WIDTH, Tk_Offset(WidgetRecord, borderWidth), 0},
 
+    {TK_CONFIG_CALLBACK, "-browsecmd", "browseCmd", "BrowseCmd",
+       "", Tk_Offset(WidgetRecord, browseCmd),
+       TK_CONFIG_NULL_OK},
+
+    {TK_CONFIG_CALLBACK, "-command", "command", "Command",
+       "", Tk_Offset(WidgetRecord, command),
+       TK_CONFIG_NULL_OK},
+
     {TK_CONFIG_ACTIVE_CURSOR, "-cursor", "cursor", "Cursor",
        DEF_GRID_CURSOR, Tk_Offset(WidgetRecord, cursor),
        TK_CONFIG_NULL_OK},
@@ -266,6 +274,7 @@ static TIX_DECLARE_SUBCMD(Tix_GrGeometryInfo);
 static TIX_DECLARE_SUBCMD(Tix_GrInfo);
 static TIX_DECLARE_SUBCMD(Tix_GrIndex);
 static TIX_DECLARE_SUBCMD(Tix_GrMove);
+static TIX_DECLARE_SUBCMD(Tix_GrSee);
 static TIX_DECLARE_SUBCMD(Tix_GrNearest);
 EXTERN TIX_DECLARE_SUBCMD(Tix_GrSelection);
 static TIX_DECLARE_SUBCMD(Tix_GrSet);
@@ -650,10 +659,8 @@ WidgetCommand(clientData, interp, argc, argv)
 	   "option from to by"},
 	{TIX_DEFAULT_LEN, "nearest", 2, 2, Tix_GrNearest,
 	   "x y"},
-#if 0
-	{TIX_DEFAULT_LEN, "see", 1, 1, Tix_GrSee,
+	{TIX_DEFAULT_LEN, "see", 2, 2, Tix_GrSee,
 	   "x y"},
-#endif
 	{TIX_DEFAULT_LEN, "selection", 3, 5, Tix_GrSelection,
 	   "option x1 y1 ?x2 y2?"},
 	{TIX_DEFAULT_LEN, "set", 2, TIX_VAR_ARGS, Tix_GrSet,
@@ -1683,11 +1690,8 @@ Tix_GrBdType(clientData, interp, argc, argv)
 	Tcl_AppendElement(interp,"y");
 	Tcl_IntResults(interp,2,1, bd[0], bd[1]);
     } else {
-	buf[0] = '\0';
+	/* return nothing */
     }
-
-    Tcl_AppendResult(interp, buf, NULL);
-
     return TCL_OK;
 }
 
@@ -2050,7 +2054,6 @@ Tix_GrIndex(clientData, interp, argc, argv)
 {
     WidgetPtr wPtr = (WidgetPtr) clientData;
     int x, y;
-    char buff[100];
 
     if (TixGridDataGetIndex(interp, wPtr, objv[0], objv[1], &x, &y)!=TCL_OK) {
 	return TCL_ERROR;
@@ -2129,7 +2132,7 @@ Tix_GrMove(clientData, interp, argc, argv)
 	return TCL_ERROR;
     }
 
-TixGridDataMoveRange(wPtr, wPtr->dataSet, which, from, to, by);
+    TixGridDataMoveRange(wPtr, wPtr->dataSet, which, from, to, by);
 
     return TCL_OK;
 }
@@ -2428,6 +2431,65 @@ void Tix_GrScrollPage(wPtr, count, axis)
     }
 }
 
+/*----------------------------------------------------------------------
+ * "see" sub command
+ *----------------------------------------------------------------------
+ */
+static int
+Tix_GrSee(clientData, interp, argc, argv)
+    ClientData clientData;
+    Tcl_Interp *interp;		/* Current interpreter. */
+    int argc;			/* Number of arguments. */
+    char **argv;		/* Argument strings. */
+{
+    WidgetPtr wPtr = (WidgetPtr) clientData;
+    int x, y;
+    int code = TCL_OK;
+    int oldXOff, oldYOff, lim;
+
+    oldXOff = wPtr->scrollInfo[0].offset;
+    oldYOff = wPtr->scrollInfo[1].offset;
+
+    if (TixGridDataGetIndex(interp, wPtr, objv[0], objv[1], &x, &y)!=TCL_OK) {
+	return TCL_ERROR;
+    }
+
+    x -= wPtr->hdrSize[0];
+    if (x >= wPtr->scrollInfo[0].max) {
+	x = wPtr->scrollInfo[0].max -1;
+    }
+    if (x < 0) {
+	x = 0;
+    }
+    lim = wPtr->scrollInfo[0].offset + (int)(wPtr->scrollInfo[0].window * wPtr->scrollInfo[0].max);
+
+    if (x < wPtr->scrollInfo[0].offset || (x+1) > lim) {
+	wPtr->scrollInfo[0].offset = x;
+    }
+
+    y -= wPtr->hdrSize[1];
+    if (y >= wPtr->scrollInfo[1].max) {
+	y = wPtr->scrollInfo[1].max -1;
+    }
+    if (y < 0) {
+	y = 0;
+    }
+    lim = wPtr->scrollInfo[1].offset + (int)(wPtr->scrollInfo[1].window * wPtr->scrollInfo[1].max);
+
+    if (y < wPtr->scrollInfo[1].offset || (y+1) > lim) {
+	wPtr->scrollInfo[1].offset = y;
+    }
+
+    if (oldXOff != wPtr->scrollInfo[0].offset ||
+	oldYOff != wPtr->scrollInfo[1].offset) {
+	wPtr->toResetRB    = 1;
+	wPtr->toComputeSel = 1;
+	Tix_GrDoWhenIdle(wPtr, TIX_GR_REDRAW);
+    }
+
+    return code;
+}
+
 /*----------------------------------------------------------------------
  * "xview" and "yview" sub command
  *----------------------------------------------------------------------
@@ -2562,9 +2624,9 @@ ConfigElement(wPtr, chPtr, argc, argv, flags, forced)
 }
 
 static char * areaNames[4] = {
-    "s-margin",
-    "x-margin",
-    "y-margin",
+    "s_margin",
+    "x_margin",
+    "y_margin",
     "main"
 };
 
