@@ -1,4 +1,7 @@
-/* 
+
+/*	$Id: tixNBFrame.c,v 1.2 2000/10/12 02:56:38 idiscovery Exp $	*/
+
+/*
  * tixNBFrame.c --
  *
  *	This module implements "tixNoteBookFrame" widgets.
@@ -221,7 +224,7 @@ static Tk_ConfigSpec configSpecs[] = {
 
     {TK_CONFIG_PIXELS, "-width", "width", "Width",
        DEF_NOTEBOOKFRAME_WIDTH, Tk_Offset(WidgetRecord, desiredWidth), 0},
-    
+
     {TK_CONFIG_END, (char *) NULL, (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, 0}
 };
@@ -309,7 +312,7 @@ static void		RedrawWhenIdle _ANSI_ARGS_((WidgetPtr wPtr));
 static int		TabConfigure _ANSI_ARGS_((WidgetPtr wPtr,
 			    Tab *tPtr, char ** argv, int argc));
 
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -388,7 +391,7 @@ Tix_NoteBookFrameCmd(clientData, interp, argc, argv)
     wPtr->redrawing	 	= 0;
     wPtr->gotFocus	 	= 0;
 
-    Tk_CreateEventHandler(wPtr->tkwin, 
+    Tk_CreateEventHandler(wPtr->tkwin,
 	ExposureMask|StructureNotifyMask|FocusChangeMask,
 	WidgetEventProc, (ClientData) wPtr);
     wPtr->widgetCmd = Tcl_CreateCommand(interp, Tk_PathName(wPtr->tkwin),
@@ -401,7 +404,7 @@ Tix_NoteBookFrameCmd(clientData, interp, argc, argv)
     interp->result = Tk_PathName(wPtr->tkwin);
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -532,7 +535,7 @@ WidgetCommand(clientData, interp, argc, argv)
 	    }
 	}
 	if (tPtr == NULL) {
-	    Tcl_AppendResult(wPtr->interp, 
+	    Tcl_AppendResult(wPtr->interp,
 		"Unknown tab \"", argv[2], "\"", (char*) NULL);
 	    goto error;
 	}
@@ -554,12 +557,8 @@ WidgetCommand(clientData, interp, argc, argv)
 	RedrawWhenIdle(wPtr);
     }
     else if ((c == 'g') && (strncmp(argv[1], "geometryinfo", length) == 0)) {
-	char buff[20];
-
 	ComputeGeometry(wPtr);
-	sprintf(buff, "%d %d", wPtr->width, wPtr->height);
-
-	Tcl_AppendResult(interp, buff, NULL);
+	Tcl_IntResults(interp, 2, 0, wPtr->width, wPtr->height);
     }
     else if ((c == 'i') && (strncmp(argv[1], "identify", length) == 0)) {
 	if (argc != 4) {
@@ -571,10 +570,10 @@ WidgetCommand(clientData, interp, argc, argv)
 	    int x, y, left, right;
 	    Tab * tPtr;
 
-	    if (Tcl_GetInt(interp, argv[2], &x) != TCL_OK) {
+	    if (Tcl_GetIntFromObj(interp, argv[2], &x) != TCL_OK) {
 		goto error;
 	    }
-	    if (Tcl_GetInt(interp, argv[3], &y) != TCL_OK) {
+	    if (Tcl_GetIntFromObj(interp, argv[3], &y) != TCL_OK) {
 		goto error;
 	    }
 
@@ -673,7 +672,7 @@ WidgetCommand(clientData, interp, argc, argv)
 	Tab * tPtr;
 
 	if (argc < 3) {
-	    Tix_ArgcError(interp, argc, argv, 2, 
+	    Tix_ArgcError(interp, argc, argv, 2,
 		"page ?option value ...?");
 	    goto error;
 	}
@@ -706,7 +705,7 @@ WidgetCommand(clientData, interp, argc, argv)
     Tcl_Release((ClientData) wPtr);
     return TCL_ERROR;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -747,10 +746,10 @@ WidgetConfigure(interp, wPtr, argc, argv, flags)
 
     if (wPtr->tabPadx < 3) {
 	wPtr->tabPadx = 3;
-    } 
+    }
     if (wPtr->tabPady < 2) {
 	wPtr->tabPady = 2;
-    } 
+    }
 
     Tk_SetBackgroundFromBorder(wPtr->tkwin, wPtr->bgBorder);
 
@@ -829,7 +828,7 @@ WidgetConfigure(interp, wPtr, argc, argv, flags)
 
     return TCL_OK;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -859,7 +858,7 @@ WidgetEventProc(clientData, eventPtr)
       case DestroyNotify:
 	if (wPtr->tkwin != NULL) {
 	    wPtr->tkwin = NULL;
-	    Tcl_DeleteCommand(wPtr->interp, 
+	    Tcl_DeleteCommand(wPtr->interp,
 	        Tcl_GetCommandName(wPtr->interp, wPtr->widgetCmd));
 	}
 	CancelRedrawWhenIdle(wPtr);
@@ -889,7 +888,7 @@ WidgetEventProc(clientData, eventPtr)
 	break;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -942,7 +941,7 @@ WidgetDestroy(clientData)
     Tk_FreeOptions(configSpecs, (char *) wPtr, wPtr->display, 0);
     ckfree((char *) wPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -978,7 +977,7 @@ WidgetCmdDeletedProc(clientData)
 	Tk_DestroyWindow(tkwin);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -993,7 +992,7 @@ static Tab * FindTab(interp, wPtr, name)
     char * name;
 {
     Tab *tPtr;
- 
+
     for (tPtr=wPtr->tabHead; tPtr; tPtr=tPtr->next) {
 	if (strcmp(tPtr->name, name) == 0) {
 	    return tPtr;
@@ -1033,7 +1032,7 @@ static int TabConfigure(wPtr, tPtr, argv, argc)
     }
 
     if (tPtr->text != NULL) {
-	tPtr->numChars = strlen(tPtr->text);
+	tPtr->numChars = -1;
 	TixComputeTextGeometry(wPtr->font, tPtr->text, tPtr->numChars,
 	    tPtr->wrapLength, &tPtr->width, &tPtr->height);
     }
@@ -1054,7 +1053,7 @@ static int TabConfigure(wPtr, tPtr, argv, argc)
 
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1117,7 +1116,7 @@ AddTab(wPtr, name, argv, argc)
 
     return TCL_OK;
 }
-
+
 static void DeleteTab(tPtr)
     Tab * tPtr;
 {
@@ -1135,12 +1134,12 @@ static void DeleteTab(tPtr)
     }
 
     if (tPtr->wPtr->tkwin) {
-	Tk_FreeOptions(tabConfigSpecs, (char *)tPtr, 
+	Tk_FreeOptions(tabConfigSpecs, (char *)tPtr,
 		       Tk_Display(tPtr->wPtr->tkwin), 0);
     }
     ckfree((char*)tPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1163,7 +1162,7 @@ RedrawWhenIdle(wPtr)
 	Tcl_DoWhenIdle(WidgetDisplay, (ClientData)wPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1207,7 +1206,7 @@ static void GetTabPoints(wPtr, tPtr, x, points)
     points[5].x = points[4].x;
     points[5].y = points[0].y;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1241,7 +1240,7 @@ static void DrawTab(wPtr, tPtr, x, isActive, drawable)
     GetTabPoints(wPtr, tPtr, x, points);
     drawX = x + wPtr->borderWidth + wPtr->tabPadx;
     drawY = wPtr->borderWidth + wPtr->tabPady;
-    extraH = wPtr->tabsHeight - tPtr->height - 
+    extraH = wPtr->tabsHeight - tPtr->height -
       wPtr->borderWidth - wPtr->tabPady *2;
 
     if (extraH > 0) {
@@ -1306,7 +1305,7 @@ static void DrawTab(wPtr, tPtr, x, isActive, drawable)
     if (wPtr->gotFocus && tPtr == wPtr->focus) {
 	XDrawLine(Tk_Display(wPtr->tkwin), drawable, wPtr->focusGC,
             drawX,
-	    drawY + tPtr->height + 1, 
+	    drawY + tPtr->height + 1,
 	    drawX + tPtr->width,
 	    drawY + tPtr->height + 1);
     }
@@ -1352,7 +1351,7 @@ static void FocusTab(wPtr, tPtr, x, drawable)
 	    wPtr->borderWidth/2, TK_RELIEF_SUNKEN);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1398,7 +1397,7 @@ WidgetDisplay(clientData)
 
 	Tk_Fill3DRectangle(wPtr->tkwin, buffer,
 		wPtr->bgBorder, 0, wPtr->tabsHeight,
-		Tk_Width(wPtr->tkwin), 
+		Tk_Width(wPtr->tkwin),
 		Tk_Height(wPtr->tkwin) - wPtr->tabsHeight,
 		wPtr->borderWidth, wPtr->relief);
 
@@ -1423,7 +1422,7 @@ WidgetDisplay(clientData)
 	/* Draw the box */
 	Tk_Draw3DRectangle(wPtr->tkwin, buffer,
 		wPtr->bgBorder, 0, wPtr->tabsHeight,
-		Tk_Width(wPtr->tkwin), 
+		Tk_Width(wPtr->tkwin),
 		Tk_Height(wPtr->tkwin) - wPtr->tabsHeight,
 		wPtr->borderWidth, wPtr->relief);
 
@@ -1451,7 +1450,7 @@ WidgetDisplay(clientData)
 
     wPtr->redrawing = 0;
 }
-
+
 /*
  *--------------------------------------------------------------
  *
@@ -1483,11 +1482,11 @@ ComputeGeometry(wPtr)
 	wPtr->tabsWidth  = 0;
 	wPtr->tabsHeight = 0;
 	for (tPtr=wPtr->tabHead; tPtr; tPtr=tPtr->next) {
-	    
+
 	    if (tPtr->text != NULL) {
-		tPtr->numChars = strlen(tPtr->text);
+		tPtr->numChars = -1;
 		TixComputeTextGeometry(wPtr->font, tPtr->text,
-		    tPtr->numChars, tPtr->wrapLength, 
+		    tPtr->numChars, tPtr->wrapLength,
 		    &tPtr->width, &tPtr->height);
 	    }
 	    else if (tPtr->image != NULL) {
@@ -1515,7 +1514,7 @@ ComputeGeometry(wPtr)
 	wPtr->height = wPtr->tabsHeight + wPtr->borderWidth*2;
     }
 }
-
+
 /*
  *--------------------------------------------------------------
  *
