@@ -237,7 +237,7 @@ static int		Tix_TLGetNeighbor _ANSI_ARGS_((
 			    WidgetPtr wPtr, Tcl_Interp *interp,
 			    int type, int argc, char **argv));
 static int		Tix_TranslateIndex _ANSI_ARGS_((
-			    WidgetPtr wPtr, Tcl_Interp *interp, Arg string,
+			    WidgetPtr wPtr, Tcl_Interp *interp, Tcl_Obj * string,
 			    int * index, int isInsert));
 static int		Tix_TLDeleteRange _ANSI_ARGS_((
 			    WidgetPtr wPtr, ListEntry * fromPtr,
@@ -1343,7 +1343,7 @@ static int
 Tix_TranslateIndex(wPtr, interp, string, index, isInsert)
     WidgetPtr wPtr;		/* TList widget record. */
     Tcl_Interp *interp;		/* Current interpreter. */
-    Arg string;		/* String representation of the index. */
+    Tcl_Obj * string;		/* String representation of the index. */
     int * index;		/* Returns the index value (0 = 1st element).*/
     int isInsert;		/* Is this function called by an "insert"
 				 * operation? */
