@@ -1,4 +1,4 @@
-/* 
+/*
  * tkUnixEmbed.c --
  *
  *	This file contains platform-specific procedures for UNIX to provide
@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkUnixEmbed.c,v 1.2 1998/09/14 18:23:56 stanton Exp $
+ * RCS: @(#) $Id: tkUnixEmbed.c,v 1.6 2002/08/05 04:30:41 dgp Exp $
  */
 
 #include "tkInt.h"
@@ -46,9 +46,11 @@ typedef struct Container {
 					 * this process. */
 } Container;
 
-static Container *firstContainerPtr = NULL;
-					/* First in list of all containers
+typedef struct ThreadSpecificData {
+    Container *firstContainerPtr;       /* First in list of all containers
 					 * managed by this process.  */
+} ThreadSpecificData;
+static Tcl_ThreadDataKey dataKey;
 
 /*
  * Prototypes for static procedures defined in this file:
@@ -69,7 +71,7 @@ static void		EmbedSendConfigure _ANSI_ARGS_((
 static void		EmbedStructureProc _ANSI_ARGS_((ClientData clientData,
 			    XEvent *eventPtr));
 static void		EmbedWindowDeleted _ANSI_ARGS_((TkWindow *winPtr));
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -83,7 +85,7 @@ static void		EmbedWindowDeleted _ANSI_ARGS_((TkWindow *winPtr));
  * Results:
  *	The return value is normally TCL_OK.  If an error occurs (such
  *	as string not being a valid window spec), then the return value
- *	is TCL_ERROR and an error message is left in interp->result if
+ *	is TCL_ERROR and an error message is left in the interp's result if
  *	interp is non-NULL.
  *
  * Side effects:
@@ -99,21 +101,24 @@ TkpUseWindow(interp, tkwin, string)
 				 * if string is bogus. */
     Tk_Window tkwin;		/* Tk window that does not yet have an
 				 * associated X window. */
-    Arg string;		/* String identifying an X window to use
+    Tcl_Obj *string;		/* String identifying an X window to use
 				 * for tkwin;  must be an integer value. */
 {
     TkWindow *winPtr = (TkWindow *) tkwin;
+    TkWindow *usePtr;
     int id, anyError;
     Window parent;
     Tk_ErrorHandler handler;
     Container *containerPtr;
     XWindowAttributes parentAtts;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (winPtr->window != None) {
 	panic("TkUseWindow: X window already assigned");
     }
-    if (Tcl_GetInt(interp, string, &id) != TCL_OK) {
-        char *s = LangString(string);
+    if (Tcl_GetIntFromObj(interp, string, &id) != TCL_OK) {
+        char *s = Tcl_GetString(string);
         char *e = NULL;
 	id = strtoul(s,&e,0);
         if (!e || *e)
@@ -121,6 +126,15 @@ TkpUseWindow(interp, tkwin, string)
     }
     parent = (Window) id;
 
+    usePtr = (TkWindow *) Tk_IdToWindow(winPtr->display, parent);
+    if (usePtr != NULL) {
+	if (!(usePtr->flags & TK_CONTAINER)) {
+	    Tcl_AppendResult(interp, "window \"", usePtr->pathName,
+                    "\" doesn't have -container option set", (char *) NULL);
+	    return TCL_ERROR;
+	}
+    }
+
     /*
      * Tk sets the window colormap to the screen default colormap in
      * tkWindow.c:AllocWindow. This doesn't work well for embedded
@@ -161,7 +175,7 @@ TkpUseWindow(interp, tkwin, string)
      * app. are in the same process.
      */
 
-    for (containerPtr = firstContainerPtr; containerPtr != NULL;
+    for (containerPtr = tsdPtr->firstContainerPtr; containerPtr != NULL;
 	    containerPtr = containerPtr->nextPtr) {
 	if (containerPtr->parent == parent) {
 	    winPtr->flags |= TK_BOTH_HALVES;
@@ -175,14 +189,14 @@ TkpUseWindow(interp, tkwin, string)
 	containerPtr->parentRoot = parentAtts.root;
 	containerPtr->parentPtr = NULL;
 	containerPtr->wrapper = None;
-	containerPtr->nextPtr = firstContainerPtr;
-	firstContainerPtr = containerPtr;
+	containerPtr->nextPtr = tsdPtr->firstContainerPtr;
+	tsdPtr->firstContainerPtr = containerPtr;
     }
     containerPtr->embeddedPtr = winPtr;
     winPtr->flags |= TK_EMBEDDED;
     return TCL_OK;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -208,6 +222,8 @@ TkpMakeWindow(winPtr, parent)
 				 * which the window is to be created. */
 {
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (winPtr->flags & TK_EMBEDDED) {
 	/*
@@ -217,7 +233,7 @@ TkpMakeWindow(winPtr, parent)
 	 * into a wrapper window later.
 	 */
 
-	for (containerPtr = firstContainerPtr; ;
+	for (containerPtr = tsdPtr->firstContainerPtr; ;
 		containerPtr = containerPtr->nextPtr) {
 	    if (containerPtr == NULL) {
 		panic("TkMakeWindow couldn't find container for window");
@@ -240,7 +256,7 @@ TkpMakeWindow(winPtr, parent)
 	    &winPtr->atts);
 #endif
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -267,6 +283,8 @@ TkpMakeContainer(tkwin)
 {
     TkWindow *winPtr = (TkWindow *) tkwin;
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * Register the window as a container so that, for example, we can
@@ -280,8 +298,8 @@ TkpMakeContainer(tkwin)
     containerPtr->parentPtr = winPtr;
     containerPtr->wrapper = None;
     containerPtr->embeddedPtr = NULL;
-    containerPtr->nextPtr = firstContainerPtr;
-    firstContainerPtr = containerPtr;
+    containerPtr->nextPtr = tsdPtr->firstContainerPtr;
+    tsdPtr->firstContainerPtr = containerPtr;
     winPtr->flags |= TK_CONTAINER;
 
     /*
@@ -301,7 +319,7 @@ TkpMakeContainer(tkwin)
     Tk_CreateEventHandler(tkwin, FocusChangeMask, EmbedFocusProc,
 	    (ClientData) containerPtr);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -331,7 +349,7 @@ EmbedErrorProc(clientData, errEventPtr)
     *iPtr = 1;
     return 0;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -362,7 +380,7 @@ EmbeddedEventProc(clientData, eventPtr)
 	EmbedWindowDeleted(winPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -391,6 +409,8 @@ ContainerEventProc(clientData, eventPtr)
     TkWindow *winPtr = (TkWindow *) clientData;
     Container *containerPtr;
     Tk_ErrorHandler errHandler;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * Ignore any X protocol errors that happen in this procedure
@@ -405,7 +425,7 @@ ContainerEventProc(clientData, eventPtr)
      * Find the Container structure associated with the parent window.
      */
 
-    for (containerPtr = firstContainerPtr;
+    for (containerPtr = tsdPtr->firstContainerPtr;
 	    containerPtr->parent != eventPtr->xmaprequest.parent;
 	    containerPtr = containerPtr->nextPtr) {
 	if (containerPtr == NULL) {
@@ -469,7 +489,7 @@ ContainerEventProc(clientData, eventPtr)
     }
     Tk_DeleteErrorHandler(errHandler);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -519,7 +539,7 @@ EmbedStructureProc(clientData, eventPtr)
 	EmbedWindowDeleted(containerPtr->parentPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -553,11 +573,11 @@ EmbedFocusProc(clientData, eventPtr)
     if (eventPtr->type == FocusIn) {
 	/*
 	 * The focus just arrived at the container.  Change the X focus
-	 * to move it to the embedded application, if there is one. 
+	 * to move it to the embedded application, if there is one.
 	 * Ignore X errors that occur during this operation (it's
 	 * possible that the new focus window isn't mapped).
 	 */
-    
+
 	if (containerPtr->wrapper != None) {
 	    errHandler = Tk_CreateErrorHandler(eventPtr->xfocus.display, -1,
 		    -1, -1, (Tk_ErrorProc *) NULL, (ClientData) NULL);
@@ -567,7 +587,7 @@ EmbedFocusProc(clientData, eventPtr)
 	}
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -616,7 +636,7 @@ EmbedGeometryRequest(containerPtr, width, height)
 	EmbedSendConfigure(containerPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -679,7 +699,7 @@ EmbedSendConfigure(containerPtr)
 	    (unsigned int) winPtr->changes.height);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -705,8 +725,11 @@ TkpGetOtherWindow(winPtr)
 				 * embedded window. */
 {
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    for (containerPtr = firstContainerPtr; containerPtr != NULL;
+    for (containerPtr = tsdPtr->firstContainerPtr;
+            containerPtr != NULL;
 	    containerPtr = containerPtr->nextPtr) {
 	if (containerPtr->embeddedPtr == winPtr) {
 	    return containerPtr->parentPtr;
@@ -717,7 +740,7 @@ TkpGetOtherWindow(winPtr)
     panic("TkpGetOtherWindow couldn't find window");
     return NULL;
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -749,6 +772,8 @@ TkpRedirectKeyEvent(winPtr, eventPtr)
 {
     Container *containerPtr;
     Window saved;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * First, find the top-level window corresponding to winPtr.
@@ -763,7 +788,7 @@ TkpRedirectKeyEvent(winPtr, eventPtr)
 
 	    return;
 	}
-	if (winPtr->flags & TK_TOP_LEVEL) {
+	if (winPtr->flags & TK_TOP_HIERARCHY) {
 	    break;
 	}
 	winPtr = winPtr->parentPtr;
@@ -777,7 +802,7 @@ TkpRedirectKeyEvent(winPtr, eventPtr)
 	 * application.  Send the event back to the container.
 	 */
 
-	for (containerPtr = firstContainerPtr;
+	for (containerPtr = tsdPtr->firstContainerPtr;
 		containerPtr->embeddedPtr != winPtr;
 		containerPtr = containerPtr->nextPtr) {
 	    /* Empty loop body. */
@@ -789,7 +814,7 @@ TkpRedirectKeyEvent(winPtr, eventPtr)
 	eventPtr->xkey.window = saved;
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -819,12 +844,14 @@ TkpClaimFocus(topLevelPtr, force)
 {
     XEvent event;
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (!(topLevelPtr->flags & TK_EMBEDDED)) {
 	return;
     }
 
-    for (containerPtr = firstContainerPtr;
+    for (containerPtr = tsdPtr->firstContainerPtr;
 	    containerPtr->embeddedPtr != topLevelPtr;
 	    containerPtr = containerPtr->nextPtr) {
 	/* Empty loop body. */
@@ -839,7 +866,7 @@ TkpClaimFocus(topLevelPtr, force)
     event.xfocus.detail = force;
     XSendEvent(event.xfocus.display, event.xfocus.window, False, 0, &event);
 }
-           
+
 #if 0
 /*
  *----------------------------------------------------------------------
@@ -864,12 +891,14 @@ TkpTestembedCmd(clientData, interp, argc, argv)
     ClientData clientData;		/* Main window for application. */
     Tcl_Interp *interp;			/* Current interpreter. */
     int argc;				/* Number of arguments. */
-    char **argv;			/* Argument strings. */
+    CONST char **argv;			/* Argument strings. */
 {
     int all;
     Container *containerPtr;
     Tcl_DString dString;
     char buffer[50];
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if ((argc > 1) && (strcmp(argv[1], "all") == 0)) {
 	all = 1;
@@ -877,7 +906,7 @@ TkpTestembedCmd(clientData, interp, argc, argv)
 	all = 0;
     }
     Tcl_DStringInit(&dString);
-    for (containerPtr = firstContainerPtr; containerPtr != NULL;
+    for (containerPtr = tsdPtr->firstContainerPtr; containerPtr != NULL;
 	    containerPtr = containerPtr->nextPtr) {
 	Tcl_DStringStartSublist(&dString);
 	if (containerPtr->parent == None) {
@@ -917,8 +946,9 @@ TkpTestembedCmd(clientData, interp, argc, argv)
     Tcl_DStringResult(interp, &dString);
     return TCL_OK;
 }
+
 #endif
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -943,6 +973,8 @@ EmbedWindowDeleted(winPtr)
 				 * was deleted. */
 {
     Container *containerPtr, *prevPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     /*
      * Find the Container structure for this window work.  Delete the
@@ -951,7 +983,7 @@ EmbedWindowDeleted(winPtr)
      */
 
     prevPtr = NULL;
-    containerPtr = firstContainerPtr;
+    containerPtr = tsdPtr->firstContainerPtr;
     while (1) {
 	if (containerPtr->embeddedPtr == winPtr) {
 	    containerPtr->wrapper = None;
@@ -968,14 +1000,14 @@ EmbedWindowDeleted(winPtr)
     if ((containerPtr->embeddedPtr == NULL)
 	    && (containerPtr->parentPtr == NULL)) {
 	if (prevPtr == NULL) {
-	    firstContainerPtr = containerPtr->nextPtr;
+	    tsdPtr->firstContainerPtr = containerPtr->nextPtr;
 	} else {
 	    prevPtr->nextPtr = containerPtr->nextPtr;
 	}
 	ckfree((char *) containerPtr);
     }
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -999,9 +1031,11 @@ TkUnixContainerId(winPtr)
     TkWindow *winPtr;		/* Tk's structure for an embedded window. */
 {
     Container *containerPtr;
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *)
+            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
-    for (containerPtr = firstContainerPtr; containerPtr != NULL;
-	    containerPtr = containerPtr->nextPtr) {
+    for (containerPtr = tsdPtr->firstContainerPtr;
+            containerPtr != NULL; containerPtr = containerPtr->nextPtr) {
 	if (containerPtr->embeddedPtr == winPtr) {
 	    return containerPtr->parent;
 	}
@@ -1009,3 +1043,6 @@ TkUnixContainerId(winPtr)
     panic("TkUnixContainerId couldn't find window");
     return None;
 }
+
+
+
