@@ -1,7 +1,7 @@
 /*
  * tkWinX.c --
  *
- *	This file contains Windows emulation procedures for X routines.
+ *      This file contains Windows emulation procedures for X routines.
  *
  * Copyright (c) 1995-1996 Sun Microsystems, Inc.
  * Copyright (c) 1994 Software Research Associates, Inc.
@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinX.c,v 1.23 2002/06/19 19:37:55 mdejong Exp $
+ * RCS: @(#) $Id: tkWinX.c,v 1.25 2002/12/08 00:46:51 hobbs Exp $
  */
 
 #include "tkWinInt.h"
@@ -36,7 +36,7 @@
 #endif
 #ifndef WM_MOUSEWHEEL
 #define WM_MOUSEWHEEL (WM_MOUSELAST+1)  // message that will be supported
-                                        // by the OS
+					// by the OS
 #endif
 
 
@@ -83,10 +83,11 @@ TkWinProcs *tkWinProcs;
  */
 
 static char winScreenName[] = ":0"; /* Default name of windows display. */
-static HINSTANCE tkInstance;        /* Application instance handle. */
+static HINSTANCE tkInstance = NULL; /* Application instance handle. */
 static int childClassInitialized;   /* Registered child class? */
-static WNDCLASS childClass;	    /* Window class for child windows. */
-static int tkPlatformId = 0;	    /* version of Windows platform */
+static WNDCLASS childClass;         /* Window class for child windows. */
+static WNDCLASS ownDCClass;         /* Window class for child windows with private DC. */
+static int tkPlatformId = 0;        /* version of Windows platform */
 static Tcl_Encoding keyInputEncoding = NULL;/* The current character
 				     * encoding for keyboard input */
 static int keyInputCharset = -1;    /* The Win32 CHARSET for the keyboard
@@ -101,7 +102,7 @@ static Tcl_Encoding unicodeEncoding = NULL; /* unicode encoding */
 typedef struct ThreadSpecificData {
     TkDisplay *winDisplay;       /* TkDisplay structure that *
 				  *  represents Windows screen. */
-    int updatingClipboard;	/* If 1, we are updating the clipboard */
+    int updatingClipboard;      /* If 1, we are updating the clipboard */
 } ThreadSpecificData;
 static Tcl_ThreadDataKey dataKey;
 
@@ -110,14 +111,14 @@ static Tcl_ThreadDataKey dataKey;
  */
 
 #ifdef __CYGWIN__
-static void		DisplayFileProc _ANSI_ARGS_((ClientData clientData,
+static void             DisplayFileProc _ANSI_ARGS_((ClientData clientData,
 			    int flags));
 #endif
-static void		GenerateXEvent _ANSI_ARGS_((HWND hwnd, UINT message,
+static void             GenerateXEvent _ANSI_ARGS_((HWND hwnd, UINT message,
 			    WPARAM wParam, LPARAM lParam));
-static unsigned int	GetState _ANSI_ARGS_((UINT message, WPARAM wParam,
+static unsigned int     GetState _ANSI_ARGS_((UINT message, WPARAM wParam,
 			    LPARAM lParam));
-static void 		GetTranslatedKey _ANSI_ARGS_((XKeyEvent *xkey));
+static void             GetTranslatedKey _ANSI_ARGS_((XKeyEvent *xkey));
 static void             UpdateInputLanguage _ANSI_ARGS_((int charset));
 static int              HandleIMEComposition _ANSI_ARGS_((HWND hwnd,
 			    LPARAM lParam));
@@ -127,24 +128,24 @@ static int              HandleIMEComposition _ANSI_ARGS_((HWND hwnd,
  *
  * TkGetServerInfo --
  *
- *	Given a window, this procedure returns information about
- *	the window server for that window.  This procedure provides
- *	the guts of the "winfo server" command.
+ *      Given a window, this procedure returns information about
+ *      the window server for that window.  This procedure provides
+ *      the guts of the "winfo server" command.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 void
 TkGetServerInfo(interp, tkwin)
-    Tcl_Interp *interp;		/* The server information is returned in
+    Tcl_Interp *interp;         /* The server information is returned in
 				 * this interpreter's result. */
-    Tk_Window tkwin;		/* Token for window;  this selects a
+    Tk_Window tkwin;            /* Token for window;  this selects a
 				 * particular display and server. */
 {
     char buffer[60];
@@ -168,13 +169,13 @@ TkGetServerInfo(interp, tkwin)
  *
  * Tk_GetHINSTANCE --
  *
- *	Retrieves the global instance handle used by the Tk library.
+ *      Retrieves the global instance handle used by the Tk library.
  *
  * Results:
- *	Returns the global instance handle.
+ *      Returns the global instance handle.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -196,21 +197,51 @@ FindMyConsole( HWND win, LPARAM arg )
 HINSTANCE
 Tk_GetHINSTANCE()
 {
+    if (tkInstance == NULL) {
+        /* In dll case this is likely to be (perl or tcl) it is 
+           here for static liked schemes 
+         */
+	tkInstance = GetModuleHandle(NULL);
+    }
     return tkInstance;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkWinSetHINSTANCE --
+ *
+ *      Sets the global instance handle used by the Tk library.
+ *      This should be called by DllMain.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkWinSetHINSTANCE(hInstance)
+    HINSTANCE hInstance;
+{
+    tkInstance = hInstance;
+}
+
 /*
  *----------------------------------------------------------------------
  *
  * TkWinXInit --
  *
- *	Initialize Xlib emulation layer.
+ *      Initialize Xlib emulation layer.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Sets up various data structures.
+ *      Sets up various data structures.
  *
  *----------------------------------------------------------------------
  */
@@ -219,6 +250,9 @@ void
 TkWinXInit(hInstance)
     HINSTANCE hInstance;
 {
+    CHARSETINFO lpCs;
+    DWORD lpCP;
+
     if (childClassInitialized != 0) {
 	return;
     }
@@ -240,7 +274,7 @@ TkWinXInit(hInstance)
     }
 
 #ifdef _LANG
-	TclWinSetInterfaces(TkWinGetPlatformId() == VER_PLATFORM_WIN32_NT);
+    TclWinSetInterfaces(TkWinGetPlatformId() == VER_PLATFORM_WIN32_NT);
 #endif
 
     tkInstance = hInstance;
@@ -278,6 +312,30 @@ TkWinXInit(hInstance)
     if (!RegisterClass(&childClass)) {
 	panic("Unable to register TkChild class");
     }
+
+	ownDCClass = childClass;
+	ownDCClass.lpszClassName = TK_WIN_OWNDC_CLASS_NAME;
+	ownDCClass.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
+    if (!RegisterClass(&ownDCClass)) {
+	panic("Unable to register TkOwnDC class");
+    }
+
+    /*
+     * Initialize input language info
+     */
+    if(GetLocaleInfo(LANGIDFROMLCID(GetKeyboardLayout(0)),
+		     LOCALE_IDEFAULTANSICODEPAGE | LOCALE_RETURN_NUMBER,
+		     (LPWSTR) &lpCP,sizeof(lpCP)/sizeof(TCHAR))==0)
+	return CP_ACP;
+    if (TranslateCharsetInfo((DWORD *)lpCP, &lpCs, TCI_SRCCODEPAGE)==0)
+	return CP_ACP;
+    UpdateInputLanguage(lpCs.ciCharset);
+
+    /*
+     * Make sure we cleanup on finalize.
+     */
+    Tcl_CreateExitHandler((Tcl_ExitProc *) TkWinXCleanup,
+	    (ClientData) hInstance);
 }
 
 /*
@@ -285,13 +343,13 @@ TkWinXInit(hInstance)
  *
  * TkWinXCleanup --
  *
- *	Removes the registered classes for Tk.
+ *      Removes the registered classes for Tk.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Removes window classes from the system.
+ *      Removes window classes from the system.
  *
  *----------------------------------------------------------------------
  */
@@ -305,8 +363,9 @@ TkWinXCleanup(hInstance)
      */
 
     if (childClassInitialized) {
-        childClassInitialized = 0;
-        UnregisterClass(TK_WIN_CHILD_CLASS_NAME, hInstance);
+	childClassInitialized = 0;
+	UnregisterClass(TK_WIN_CHILD_CLASS_NAME, hInstance);
+	UnregisterClass(TK_WIN_OWNDC_CLASS_NAME, hInstance);
     }
 
     if (unicodeEncoding != NULL) {
@@ -326,17 +385,17 @@ TkWinXCleanup(hInstance)
  *
  * TkWinGetPlatformId --
  *
- *	Determines whether running under NT, 95, or Win32s, to allow
- *	runtime conditional code.  Win32s is no longer supported.
+ *      Determines whether running under NT, 95, or Win32s, to allow
+ *      runtime conditional code.  Win32s is no longer supported.
  *
  * Results:
- *	The return value is one of:
- *	    VER_PLATFORM_WIN32s		Win32s on Windows 3.1.
- *	    VER_PLATFORM_WIN32_WINDOWS	Win32 on Windows 95.
- *	    VER_PLATFORM_WIN32_NT	Win32 on Windows NT
+ *      The return value is one of:
+ *          VER_PLATFORM_WIN32s         Win32s on Windows 3.1.
+ *          VER_PLATFORM_WIN32_WINDOWS  Win32 on Windows 95.
+ *          VER_PLATFORM_WIN32_NT       Win32 on Windows NT
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -359,22 +418,22 @@ TkWinGetPlatformId()
  *
  * TkGetDefaultScreenName --
  *
- *	Returns the name of the screen that Tk should use during
- *	initialization.
+ *      Returns the name of the screen that Tk should use during
+ *      initialization.
  *
  * Results:
- *	Returns a statically allocated string.
+ *      Returns a statically allocated string.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 CONST char *
 TkGetDefaultScreenName(interp, screenName)
-    Tcl_Interp *interp;		/* Not used. */
-    CONST char *screenName;	/* If NULL, use default string. */
+    Tcl_Interp *interp;         /* Not used. */
+    CONST char *screenName;     /* If NULL, use default string. */
 {
     if ((screenName == NULL) || (screenName[0] == '\0')) {
 	screenName = winScreenName;
@@ -387,14 +446,14 @@ TkGetDefaultScreenName(interp, screenName)
  *
  * TkpOpenDisplay --
  *
- *	Create the Display structure and fill it with device
- *	specific information.
+ *      Create the Display structure and fill it with device
+ *      specific information.
  *
  * Results:
- *	Returns a TkDisplay structure on success or NULL on failure.
+ *      Returns a TkDisplay structure on success or NULL on failure.
  *
  * Side effects:
- *	Allocates a new TkDisplay structure.
+ *      Allocates a new TkDisplay structure.
  *
  *----------------------------------------------------------------------
  */
@@ -412,7 +471,7 @@ TkpOpenDisplay(display_name)
 
     if (tsdPtr->winDisplay != NULL) {
 	if (strcmp(tsdPtr->winDisplay->display->display_name, display_name)
-                == 0) {
+		== 0) {
 	    return tsdPtr->winDisplay;
 	} else {
 	    return NULL;
@@ -512,9 +571,9 @@ TkpOpenDisplay(display_name)
     screen->white_pixel = RGB(255, 255, 255);
     screen->black_pixel = RGB(0, 0, 0);
 
-    display->screens		= screen;
-    display->nscreens		= 1;
-    display->default_screen	= 0;
+    display->screens            = screen;
+    display->nscreens           = 1;
+    display->default_screen     = 0;
     screen->cmap = XCreateColormap(display, None, screen->root_visual,
 	    AllocNone);
     tsdPtr->winDisplay = (TkDisplay *) ckalloc(sizeof(TkDisplay));
@@ -529,14 +588,14 @@ TkpOpenDisplay(display_name)
  *
  * TkpCloseDisplay --
  *
- *	Closes and deallocates a Display structure created with the
- *	TkpOpenDisplay function.
+ *      Closes and deallocates a Display structure created with the
+ *      TkpOpenDisplay function.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Frees up memory.
+ *      Frees up memory.
  *
  *----------------------------------------------------------------------
  */
@@ -551,8 +610,8 @@ TkpCloseDisplay(dispPtr)
 	    Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
 
     if (dispPtr != tsdPtr->winDisplay) {
-        panic("TkpCloseDisplay: tried to call TkpCloseDisplay on another display");
-        return;
+	panic("TkpCloseDisplay: tried to call TkpCloseDisplay on another display");
+	return;
     }
 
     /*
@@ -572,23 +631,23 @@ TkpCloseDisplay(dispPtr)
     tsdPtr->winDisplay = NULL;
 
     if (display->display_name != (char *) NULL) {
-        ckfree(display->display_name);
+	ckfree(display->display_name);
     }
     if (display->screens != (Screen *) NULL) {
-        if (display->screens->root_visual != NULL) {
-            ckfree((char *) display->screens->root_visual);
-        }
-        if (display->screens->root != None) {
-            ckfree((char *) display->screens->root);
-        }
-        if (display->screens->cmap != None) {
-            XFreeColormap(display, display->screens->cmap);
-        }
-        ckfree((char *) display->screens);
+	if (display->screens->root_visual != NULL) {
+	    ckfree((char *) display->screens->root_visual);
+	}
+	if (display->screens->root != None) {
+	    ckfree((char *) display->screens->root);
+	}
+	if (display->screens->cmap != None) {
+	    XFreeColormap(display, display->screens->cmap);
+	}
+	ckfree((char *) display->screens);
     }
 #ifdef __CYGWIN__
     if (dispPtr->display != 0) {
-        Tcl_DeleteFileHandler(ConnectionNumber(dispPtr->display));
+	Tcl_DeleteFileHandler(ConnectionNumber(dispPtr->display));
 	close(ConnectionNumber(dispPtr->display));
     }
 #endif
@@ -602,23 +661,23 @@ TkpCloseDisplay(dispPtr)
  *
  * DisplayFileProc --
  *
- *	This procedure implements the file handler for the /dev/windows
- *	connection.
+ *      This procedure implements the file handler for the /dev/windows
+ *      connection.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Process Win32 message queue.  Compare to tclWin/tclWinNotify.c
- *	Tcl_WaitForEvent() event loop.
+ *      Process Win32 message queue.  Compare to tclWin/tclWinNotify.c
+ *      Tcl_WaitForEvent() event loop.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 DisplayFileProc(clientData, flags)
-    ClientData clientData;		/* The display pointer. */
-    int flags;				/* Should be TCL_READABLE. */
+    ClientData clientData;              /* The display pointer. */
+    int flags;                          /* Should be TCL_READABLE. */
 {
     TkDisplay *dispPtr = (TkDisplay *) clientData;
     Display *display = dispPtr->display;
@@ -649,13 +708,13 @@ DisplayFileProc(clientData, flags)
  *
  * XBell --
  *
- *	Generate a beep.
+ *      Generate a beep.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Plays a sounds out the system speakers.
+ *      Plays a sounds out the system speakers.
  *
  *----------------------------------------------------------------------
  */
@@ -673,14 +732,14 @@ XBell(display, percent)
  *
  * TkWinChildProc --
  *
- *	Callback from Windows whenever an event occurs on a child
- *	window.
+ *      Callback from Windows whenever an event occurs on a child
+ *      window.
  *
  * Results:
- *	Standard Windows return value.
+ *      Standard Windows return value.
  *
  * Side effects:
- *	May process events off the Tk event queue.
+ *      May process events off the Tk event queue.
  *
  *----------------------------------------------------------------------
  */
@@ -695,17 +754,17 @@ TkWinChildProc(hwnd, message, wParam, lParam)
     LRESULT result;
 
     switch (message) {
-        case WM_INPUTLANGCHANGE:
+	case WM_INPUTLANGCHANGE:
 	    UpdateInputLanguage(wParam);
 	    result = 1;
 	    break;
 
-        case WM_IME_COMPOSITION:
-            result = 0;
-            if (HandleIMEComposition(hwnd, lParam) == 0) {
-                result = DefWindowProc(hwnd, message, wParam, lParam);
-            }
-            break;
+	case WM_IME_COMPOSITION:
+	    result = 0;
+	    if (HandleIMEComposition(hwnd, lParam) == 0) {
+		result = DefWindowProc(hwnd, message, wParam, lParam);
+	    }
+	    break;
 
 	case WM_SETCURSOR:
 	    /*
@@ -726,7 +785,7 @@ TkWinChildProc(hwnd, message, wParam, lParam)
 	    result = DefWindowProc(hwnd, message, wParam, lParam);
 	    break;
 
-        case TK_CLAIMFOCUS:
+	case TK_CLAIMFOCUS:
 	case TK_GEOMETRYREQ:
 	case TK_ATTACHWINDOW:
 	case TK_DETACHWINDOW:
@@ -754,14 +813,14 @@ TkWinChildProc(hwnd, message, wParam, lParam)
  *
  * Tk_TranslateWinEvent --
  *
- *	This function is called by widget window procedures to handle
- *	the translation from Win32 events to Tk events.
+ *      This function is called by widget window procedures to handle
+ *      the translation from Win32 events to Tk events.
  *
  * Results:
- *	Returns 1 if the event was handled, else 0.
+ *      Returns 1 if the event was handled, else 0.
  *
  * Side effects:
- *	Depends on the event.
+ *      Depends on the event.
  *
  *----------------------------------------------------------------------
  */
@@ -776,14 +835,14 @@ Tk_TranslateWinEvent(hwnd, message, wParam, lParam, resultPtr)
 {
     *resultPtr = 0;
     switch (message) {
-        default: {
+	default: {
 	    Tk_Window tkwin = (Tk_Window) Tk_HWNDToWindow(hwnd);
 	    if (tkwin) {
 		if (Lang_WinEvent(tkwin, message, wParam, lParam, resultPtr))
-	  	    return 1;
-            }
+		    return 1;
+	    }
 	    break;
-        }
+	}
 
 	case WM_RENDERFORMAT: {
 	    TkWindow *winPtr = (TkWindow *) Tk_HWNDToWindow(hwnd);
@@ -837,7 +896,7 @@ Tk_TranslateWinEvent(hwnd, message, wParam, lParam, resultPtr)
 	case WM_KEYDOWN:
 	case WM_KEYUP:
 	case WM_MOUSEWHEEL:
- 	    GenerateXEvent(hwnd, message, wParam, lParam);
+	    GenerateXEvent(hwnd, message, wParam, lParam);
 	    return 1;
 	case WM_MENUCHAR:
 	    GenerateXEvent(hwnd, message, wParam, lParam);
@@ -853,14 +912,14 @@ Tk_TranslateWinEvent(hwnd, message, wParam, lParam, resultPtr)
  *
  * GenerateXEvent --
  *
- *	This routine generates an X event from the corresponding
- * 	Windows event.
+ *      This routine generates an X event from the corresponding
+ *      Windows event.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	Queues one or more X events.
+ *      Queues one or more X events.
  *
  *----------------------------------------------------------------------
  */
@@ -990,7 +1049,7 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 	    unsigned int state = GetState(message, wParam, lParam);
 	    Time time = TkpGetMS();
 	    POINT clientPoint;
-	    POINTS rootPoint;	/* Note: POINT and POINTS are different */
+	    POINTS rootPoint;   /* Note: POINT and POINTS are different */
 	    DWORD msgPos;
 
 	    /*
@@ -1079,30 +1138,30 @@ GenerateXEvent(hwnd, message, wParam, lParam)
 		     * 1. A series of WM_KEYDOWN & WM_KEYUP messages that
 		     *    cause GetTranslatedKey() to be called and return
 		     *    immediately because the WM_KEYDOWNs have no
-		     *	  associated WM_CHAR messages -- the IME window is
-		     *	  accumulating the characters and translating them
+		     *    associated WM_CHAR messages -- the IME window is
+		     *    accumulating the characters and translating them
 		     *    itself.  In the "bind" command, you get an event
-		     *	  with a mystery keysym and %A == "" for each
-		     *	  WM_KEYDOWN that actually was meant for the IME.
+		     *    with a mystery keysym and %A == "" for each
+		     *    WM_KEYDOWN that actually was meant for the IME.
 		     * 2. A WM_KEYDOWN corresponding to the "confirm typing"
 		     *    character.  This causes GetTranslatedKey() to be
-		     *	  called.
+		     *    called.
 		     * 3. A WM_IME_NOTIFY message saying that the IME is
-		     *	  done.  A side effect of this message is that
+		     *    done.  A side effect of this message is that
 		     *    GetTranslatedKey() thinks this means that there
-		     *	  are no WM_CHAR messages and returns immediately.
+		     *    are no WM_CHAR messages and returns immediately.
 		     *    In the "bind" command, you get an another event
-		     *	  with a mystery keysym and %A == "".
+		     *    with a mystery keysym and %A == "".
 		     * 4. A sequence of WM_CHAR messages that correspond to
-		     *	  the characters in the IME window.  A bunch of
+		     *    the characters in the IME window.  A bunch of
 		     *    simulated KeyPress/KeyRelease events will be
 		     *    generated, one for each character.  Adjacent
 		     *    WM_CHAR messages may actually specify the high
-		     *	  and low bytes of a multi-byte character -- in that
+		     *    and low bytes of a multi-byte character -- in that
 		     *    case the two WM_CHAR messages will be combined into
-		     *	  one event.  It is the event-consumer's
-		     *	  responsibility to convert the string returned from
-		     *	  XLookupString from system encoding to UTF-8.
+		     *    one event.  It is the event-consumer's
+		     *    responsibility to convert the string returned from
+		     *    XLookupString from system encoding to UTF-8.
 		     * 5. And finally we get the WM_KEYUP for the "confirm
 		     *    typing" character.
 		     */
@@ -1144,27 +1203,27 @@ GenerateXEvent(hwnd, message, wParam, lParam)
  *
  * GetState --
  *
- *	This function constructs a state mask for the mouse buttons
- *	and modifier keys as they were before the event occured.
+ *      This function constructs a state mask for the mouse buttons
+ *      and modifier keys as they were before the event occured.
  *
  * Results:
- *	Returns a composite value of all the modifier and button state
- *	flags that were set at the time the event occurred.
+ *      Returns a composite value of all the modifier and button state
+ *      flags that were set at the time the event occurred.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
 
 static unsigned int
 GetState(message, wParam, lParam)
-    UINT message;		/* Win32 message type */
-    WPARAM wParam;		/* wParam of message, used if key message */
-    LPARAM lParam;		/* lParam of message, used if key message */
+    UINT message;               /* Win32 message type */
+    WPARAM wParam;              /* wParam of message, used if key message */
+    LPARAM lParam;              /* lParam of message, used if key message */
 {
     int mask;
-    int prevState;		/* 1 if key was previously down */
+    int prevState;              /* 1 if key was previously down */
     unsigned int state = TkWinGetModifierState();
 
     /*
@@ -1219,16 +1278,16 @@ GetState(message, wParam, lParam)
  *
  * GetTranslatedKey --
  *
- *	Retrieves WM_CHAR messages that are placed on the system queue
- *	by the TranslateMessage system call and places them in the
- *	given KeyPress event.
+ *      Retrieves WM_CHAR messages that are placed on the system queue
+ *      by the TranslateMessage system call and places them in the
+ *      given KeyPress event.
  *
  * Results:
- *	Sets the trans_chars and nbytes member of the key event.
+ *      Sets the trans_chars and nbytes member of the key event.
  *
  * Side effects:
- *	Removes any WM_CHAR messages waiting on the top of the system
- *	event queue.
+ *      Removes any WM_CHAR messages waiting on the top of the system
+ *      event queue.
  *
  *----------------------------------------------------------------------
  */
@@ -1261,16 +1320,16 @@ GetTranslatedKey(xkey)
 	    xkey->nbytes++;
 
 	    if (((unsigned short) msg.wParam) > ((unsigned short) 0xff)) {
-                /*
-                 * Some "addon" input devices, such as the popular
-                 * PenPower Chinese writing pad, generate 16 bit
-                 * values in WM_CHAR messages (instead of passing them
-                 * in two separate WM_CHAR messages containing two
-                 * 8-bit values.
-                 */
+		/*
+		 * Some "addon" input devices, such as the popular
+		 * PenPower Chinese writing pad, generate 16 bit
+		 * values in WM_CHAR messages (instead of passing them
+		 * in two separate WM_CHAR messages containing two
+		 * 8-bit values.
+		 */
 
-	        xkey->trans_chars[xkey->nbytes] = (char) (msg.wParam >> 8);
-	        xkey->nbytes ++;
+		xkey->trans_chars[xkey->nbytes] = (char) (msg.wParam >> 8);
+		xkey->nbytes ++;
 	    }
 	} else {
 	    break;
@@ -1283,7 +1342,7 @@ GetTranslatedKey(xkey)
  *
  * UpdateInputLanguage --
  *
- *	Gets called when a WM_INPUTLANGCHANGE message is received
+ *      Gets called when a WM_INPUTLANGCHANGE message is received
  *      by the TK child window procedure. This message is sent
  *      by the Input Method Editor system when the user chooses
  *      a different input method. All subsequent WM_CHAR
@@ -1292,10 +1351,10 @@ GetTranslatedKey(xkey)
  *      correctly translate the WM_CHAR into unicode.
  *
  * Results:
- *	Records the new encoding in keyInputEncoding.
+ *      Records the new encoding in keyInputEncoding.
  *
  * Side effects:
- *	Old value of keyInputEncoding is freed.
+ *      Old value of keyInputEncoding is freed.
  *
  *----------------------------------------------------------------------
  */
@@ -1312,7 +1371,7 @@ UpdateInputLanguage(charset)
 	return;
     }
     if (TranslateCharsetInfo((DWORD*)charset, &charsetInfo, TCI_SRCCHARSET)
-            == 0) {
+	    == 0) {
 	/*
 	 * Some mysterious failure.
 	 */
@@ -1343,14 +1402,14 @@ UpdateInputLanguage(charset)
  *
  * TkWinGetKeyInputEncoding --
  *
- *	Returns the current keyboard input encoding selected by the
+ *      Returns the current keyboard input encoding selected by the
  *      user (with WM_INPUTLANGCHANGE events).
  *
  * Results:
- *	The current keyboard input encoding.
+ *      The current keyboard input encoding.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -1366,13 +1425,13 @@ TkWinGetKeyInputEncoding()
  *
  * TkWinGetUnicodeEncoding --
  *
- *	Returns the cached unicode encoding.
+ *      Returns the cached unicode encoding.
  *
  * Results:
- *	The unicode encoding.
+ *      The unicode encoding.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -1406,11 +1465,11 @@ TkWinGetUnicodeEncoding()
  *      TK's event queue.
  *
  * Results:
- *	If this function has processed the composition data, returns 1.
+ *      If this function has processed the composition data, returns 1.
  *      Otherwise returns 0.
  *
  * Side effects:
- *	Key events are put into the TK event queue.
+ *      Key events are put into the TK event queue.
  *
  *----------------------------------------------------------------------
  */
@@ -1419,7 +1478,7 @@ static int
 HandleIMEComposition(hwnd, lParam)
     HWND hwnd;                          /* Window receiving the message. */
     LPARAM lParam;                      /* Flags for the WM_IME_COMPOSITION
-                                         * message */
+					 * message */
 {
     HIMC hIMC;
     int i, n;
@@ -1430,11 +1489,11 @@ HandleIMEComposition(hwnd, lParam)
     BOOL isWinNT = (TkWinGetPlatformId() == VER_PLATFORM_WIN32_NT);
 
     if ((lParam & GCS_RESULTSTR) == 0) {
-        /*
-         * Composition is not finished yet.
-         */
+	/*
+	 * Composition is not finished yet.
+	 */
 
-        return 0;
+	return 0;
     }
 
     hIMC = ImmGetContext(hwnd);
@@ -1445,7 +1504,7 @@ HandleIMEComposition(hwnd, lParam)
 	    n = ImmGetCompositionStringA(hIMC, GCS_RESULTSTR, NULL, 0);
 	}
 
-        if ((n > 0) && ((buff = (char *) ckalloc(n)) != NULL)) {
+	if ((n > 0) && ((buff = (char *) ckalloc(n)) != NULL)) {
 	    if (isWinNT) {
 		n = ImmGetCompositionStringW(hIMC, GCS_RESULTSTR, buff, n);
 	    } else {
@@ -1475,47 +1534,47 @@ HandleIMEComposition(hwnd, lParam)
 
 	    /*
 	     * Set up the fields pertinent to key event.
-             *
-             * We set send_event to the special value of -2, so that
-             * TkpGetString() in tkWinKey.c knows that trans_chars[]
-             * already contains a UNICODE char and there's no need to
-             * do encoding conversion.
+	     *
+	     * We set send_event to the special value of -2, so that
+	     * TkpGetString() in tkWinKey.c knows that trans_chars[]
+	     * already contains a UNICODE char and there's no need to
+	     * do encoding conversion.
 	     */
 
-            winPtr = (TkWindow *)Tk_HWNDToWindow(hwnd);
+	    winPtr = (TkWindow *)Tk_HWNDToWindow(hwnd);
 
-            event.xkey.serial = winPtr->display->request++;
-            event.xkey.send_event = -2;
-            event.xkey.display = winPtr->display;
-            event.xkey.window = winPtr->window;
+	    event.xkey.serial = winPtr->display->request++;
+	    event.xkey.send_event = -2;
+	    event.xkey.display = winPtr->display;
+	    event.xkey.window = winPtr->window;
 	    event.xkey.root = RootWindow(winPtr->display, winPtr->screenNum);
 	    event.xkey.subwindow = None;
 	    event.xkey.state = TkWinGetModifierState();
 	    event.xkey.time = TkpGetMS();
 	    event.xkey.same_screen = True;
-            event.xkey.keycode = 0;
-            event.xkey.nbytes = 2;
+	    event.xkey.keycode = 0;
+	    event.xkey.nbytes = 2;
 
-            for (i=0; i<n;) {
-                /*
-                 * Simulate a pair of KeyPress and KeyRelease events
-                 * for each UNICODE character in the composition.
-                 */
+	    for (i=0; i<n;) {
+		/*
+		 * Simulate a pair of KeyPress and KeyRelease events
+		 * for each UNICODE character in the composition.
+		 */
 
-                event.xkey.trans_chars[0] = (char) buff[i++];
-                event.xkey.trans_chars[1] = (char) buff[i++];
+		event.xkey.trans_chars[0] = (char) buff[i++];
+		event.xkey.trans_chars[1] = (char) buff[i++];
 
-                event.type = KeyPress;
-                Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
+		event.type = KeyPress;
+		Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
 
-                event.type = KeyRelease;
-                Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
-            }
+		event.type = KeyRelease;
+		Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
+	    }
 
-            ckfree(buff);
-        }
-        ImmReleaseContext(hwnd, hIMC);
-        return 1;
+	    ckfree(buff);
+	}
+	ImmReleaseContext(hwnd, hIMC);
+	return 1;
     }
 
     return 0;
@@ -1526,13 +1585,13 @@ HandleIMEComposition(hwnd, lParam)
  *
  * Tk_FreeXId --
  *
- *	This interface is not needed under Windows.
+ *      This interface is not needed under Windows.
  *
  * Results:
- *	None.
+ *      None.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -1549,14 +1608,14 @@ Tk_FreeXId(display, xid)
  *
  * TkWinResendEvent --
  *
- *	This function converts an X event into a Windows event and
- *	invokes the specified windo procedure.
+ *      This function converts an X event into a Windows event and
+ *      invokes the specified windo procedure.
  *
  * Results:
- *	A standard Windows result.
+ *      A standard Windows result.
  *
  * Side effects:
- *	Invokes the window procedure
+ *      Invokes the window procedure
  *
  *----------------------------------------------------------------------
  */
@@ -1616,14 +1675,14 @@ TkWinResendEvent(wndproc, hwnd, eventPtr)
  *
  * TkpGetMS --
  *
- *	Return a relative time in milliseconds.  It doesn't matter
- *	when the epoch was.
+ *      Return a relative time in milliseconds.  It doesn't matter
+ *      when the epoch was.
  *
  * Results:
- *	Number of milliseconds.
+ *      Number of milliseconds.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -1641,10 +1700,10 @@ TkpGetMS()
  *
  *
  * Results:
- *	Number of milliseconds.
+ *      Number of milliseconds.
  *
  * Side effects:
- *	None.
+ *      None.
  *
  *----------------------------------------------------------------------
  */
@@ -1663,23 +1722,23 @@ TkWinUpdatingClipboard(int mode)
  *
  * Tk_SetCaretPos --
  *
- *	This enables correct movement of focus in the MS Magnifier, as well
- *	as allowing us to correctly position the IME Window.  The following
- *	Win32 APIs are used to work with MS caret:
+ *      This enables correct movement of focus in the MS Magnifier, as well
+ *      as allowing us to correctly position the IME Window.  The following
+ *      Win32 APIs are used to work with MS caret:
  *
- *	CreateCaret	DestroyCaret	SetCaretPos	GetCaretPos
+ *      CreateCaret     DestroyCaret    SetCaretPos     GetCaretPos
  *
- *	Only one instance of caret can be active at any time
- *	(e.g. DestroyCaret API does not take any argument such as handle).
- *	Since do-it-right approach requires to track the create/destroy
- *	caret status all the time in a global scope among windows (or
- *	widgets), we just implement this minimal setup to get the job done.
+ *      Only one instance of caret can be active at any time
+ *      (e.g. DestroyCaret API does not take any argument such as handle).
+ *      Since do-it-right approach requires to track the create/destroy
+ *      caret status all the time in a global scope among windows (or
+ *      widgets), we just implement this minimal setup to get the job done.
  *
  * Results:
- *	None
+ *      None
  *
  * Side effects:
- *	Sets the global Windows caret position.
+ *      Sets the global Windows caret position.
  *
  *----------------------------------------------------------------------
  */
