@@ -25,7 +25,7 @@ typedef struct Lang_CmdInfo
   SV          *image;
   Tk_Font     tkfont;
  } Lang_CmdInfo;
- 
+
 #include "vtab.def"
 
 
@@ -54,8 +54,8 @@ extern XS(XStoWidget);
 EXTERN void ClearErrorInfo _ANSI_ARGS_((SV *interp));
 EXTERN Tk_Window mainWindow;
 EXTERN void DumpStack _ANSI_ARGS_((CONST char *who));
-EXTERN void  Boot_Glue _ANSI_ARGS_((void));
-EXTERN void  Boot_Tix  _ANSI_ARGS_((void));
+EXTERN void  Boot_Glue (pTHX);
+EXTERN void  Boot_Tix  (pTHX);
 EXTERN void install_vtab _ANSI_ARGS_((char *name, void *table, size_t size));
 extern SV *TagIt _((SV *sv, char *type));
 extern void Font_DESTROY _((SV *sv));
@@ -64,6 +64,7 @@ extern void Tk_CheckHash _((SV *sv,struct pTkCheckChain *chain));
 
 extern int	has_highbit(CONST char *s,int l);
 extern SV *	sv_maybe_utf8(SV *sv);
+extern SV *	Lang_SystemEncoding(void);
 
 #ifndef WIN32
 #define HWND void *
